/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

import java.io.File;
import java.io.IOException;
import java.io.PrintStream;
import teajay.bytecode.ClassAccessFlag;
import teajay.bytecode.ClassFile;
import teajay.bytecode.attributes.code.*;
import teajay.bytecode.cpool.*;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.bytecode.methods.MethodAccessFlag;
import teajay.bytecode.methods.MethodInfo;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public class HalloWeltGenerator {
    public static void main(String[] args) throws ByteCodeException, IOException {
        ClassFile halloWelt = new ClassFile("HalloWelt", ClassAccessFlag.PUBLIC, ClassAccessFlag.SUPER);
        MethodInfo main = new MethodInfo("main",
                new MethodDescriptor(FieldDescriptor.getVoid(), FieldDescriptor.fromClass(String[].class)),
                halloWelt.getThis(), MethodAccessFlag.STATIC, MethodAccessFlag.PUBLIC);
        halloWelt.addMethod(main);
        JVMInstructionList lst = main.getCode().getInstructionList();
        lst.add(new GetStatic(FieldRefInfo.make(ClassInfo.fromClass(System.class),
                "out", FieldDescriptor.fromClass(PrintStream.class))));
        lst.add(new Ldc("Hallo Welt"));
        lst.add(new InvokeVirtual(MethodRefInfo.make(ClassInfo.fromClass(PrintStream.class),
                "println", FieldDescriptor.getVoid(), FieldDescriptor.getObject())));       
        lst.pushInt(-1);
        lst.add(new InvokeStatic(MethodRefInfo.make("java/lang/System", "exit", "void", "int")));
        lst.add(new Return());       
        halloWelt.write(new File("HalloWelt.class"));
    }
}
