/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.bytecode;

import java.io.DataOutputStream;
import java.io.IOException;
import teajay.bytecode.cpool.ConstantPool;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 * A class implementing this interface is part of a class file and can be written as a stream of bytes.
 * @author Michael Kußmann <michi@outx.de>
 */
public interface ByteCodeWritable {
    /**
     * Writes the byte code representation of this entity to <code> out </code>.
     * @param out the destination stream
     * @param cpool the value of cpool
     * @throws IOException if out throws an <tt>IOException</tt>
     * @throws ByteCodeException if the entity not yet holds all required information or the content does not meet certain constraints
     */
    public void write(DataOutputStream out, ConstantPool cpool) throws IOException, ByteCodeException;
}
