/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.bytecode;

import teajay.bytecode.OpCode.OpSize;
import teajay.bytecode.cpool.ConstantPool;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 * All operands to an instruction must implement this interface.
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public interface Operand extends IndexWritable {

    /**
     * The number of bytes this entity will take as an operand.
     *
     * @return length as operand
     */
    public abstract OpSize getDefaultLengthAsOperand();

    /**
     * The value of this entity as an operand.
     *
     * @param cpool the value of cpool
     * @return value of this entity as operand
     * @throws ByteCodeException if the required data to calculate the result is
     * not yet available
     */
    public abstract long getAddress(ConstantPool cpool) throws ByteCodeException;
}
