/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.bytecode;

import java.io.DataOutputStream;
import java.io.IOException;
import teajay.bytecode.OpCode.OpSize;
import teajay.bytecode.cpool.ConstantPool;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 * unsinged byte
 * @author Michael Kußmann <michi@outx.de>
 */
public final class U1 implements Operand {

    public static final U1 ZERO = new U1();
    private final int value;

    private U1() {
        this.value = 0;
    }

    public U1(int value) throws ByteCodeException {
        if (value > 0xFF || value < 0) {
            throw new ByteCodeException("usigned byte (u1) out of bounds: " + value);
        }
        this.value = value;
    }

    public U1(long value) throws ByteCodeException {
        if (value > 0xFF || value < 0) {
            throw new ByteCodeException("usigned byte (u1) out of bounds: " + value);
        }
        this.value = (int) value;
    }

    public final int getValue() {
        return value;
    }

    @Override
    public long getAddress(ConstantPool cpool) {
        return value;
    }

    public U2 toU2() throws ByteCodeException {
        return new U2(value);
    }

    public U4 toU4() throws ByteCodeException {
        return new U4(value);
    }

    @Override
    public void write(DataOutputStream out, ConstantPool cpool) throws IOException {
        out.writeByte(value);
    }

    @Override
    public void writeIndex(DataOutputStream out, ConstantPool cpool) throws IOException {
        this.write(out, cpool);
    }

    @Override
    public OpSize getDefaultLengthAsOperand() {
        return OpSize.U1;
    }

    @Override
    public String toString() {
        return "U1: " + value;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj instanceof U1) {
            U1 other = (U1) obj;
            return other.value == value;
        }
        return false;
    }

    @Override
    public int hashCode() {
        int hash = 7;
        hash = 53 * hash + this.value;
        return hash;
    }
}
