/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.bytecode.attributes.annotations;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import teajay.bytecode.U4;
import teajay.bytecode.attributes.AttributeInfo;
import teajay.bytecode.cpool.ConstantPool;
import teajay.bytecode.cpool.Utf8Info;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public class AnnotationDefault extends AttributeInfo {
    public static final String NAME = "AnnotationDefault";
    
    private final ElementValue value;
    
    public AnnotationDefault(ElementValue value) {
        this(new Utf8Info(NAME), value);
    }
    
    private AnnotationDefault(Utf8Info name, ElementValue value) {
        super(name);
        checkName(NAME, name);
        this.value = value;
    }
    
    public ElementValue getValue() {
        return value;
    }

    @Override
    public U4 getLength(ConstantPool cpool) throws ByteCodeException {
        return new U4(value.getLength());
    }
    
    @Override
    public void write(DataOutputStream out, ConstantPool cpool) throws IOException, ByteCodeException {
        super.write(out, cpool);
        value.write(out, cpool);
    }
    
    public static AnnotationDefault read(DataInputStream in, ConstantPool cpool, int major, int minor, int length, Utf8Info name) throws ByteCodeException, IOException {
        return new AnnotationDefault(name, ElementValue.read(in, cpool, major, minor));
    }
}
