/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.bytecode.attributes.code;

import java.io.DataOutputStream;
import java.io.IOException;
import teajay.bytecode.OpCode;
import teajay.bytecode.Operand;
import teajay.bytecode.cpool.ConstantPool;
import teajay.bytecode.exceptions.ByteCodeAddressOverflow;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
abstract class CompareInstruction extends JumpInstruction {
    private final Operand branchTarget;

    protected CompareInstruction(OpCode opcode, Operand branchTarget) {
        this.setOpcode(opcode);
        this.setOperands(branchTarget);
        this.branchTarget = branchTarget;
    }

    @Override
    boolean validate(ConstantPool cpool) throws ByteCodeException {
        long diff = branchTarget.getAddress(cpool) - getAddress(cpool);
        if (diff > Short.MAX_VALUE || diff < Short.MIN_VALUE) {
            throw new ByteCodeAddressOverflow(opcode() + ": Target address exceeds signed short range.");
        } else {
            return true;
        }
    }

    @Override
    public void write(DataOutputStream out, ConstantPool cpool) throws IOException, ByteCodeException {
        opcode().write(out, cpool);
        this.diffS2(branchTarget, cpool).write(out, cpool);
    }
}
