/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.bytecode.attributes.code;

import java.io.DataOutputStream;
import java.io.IOException;
import teajay.bytecode.ByteCodeWritable;
import teajay.bytecode.Operand;
import teajay.bytecode.U2;
import teajay.bytecode.cpool.ClassInfo;
import teajay.bytecode.cpool.ConstantPool;
import teajay.bytecode.cpool.ConstantPoolAddable;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public final class ExceptionTableEntry implements ByteCodeWritable, ConstantPoolAddable {

    private final Operand startPC;
    private final Operand endPC;
    private final Operand handlerPC;
    private final ClassInfo catchType;

    ExceptionTableEntry(Operand startPC, Operand endPC, Operand handlerPC, ClassInfo catchType) {
        this.startPC = startPC;
        this.endPC = endPC;
        this.handlerPC = handlerPC;
        this.catchType = catchType;
    }

    @Override
    public void write(DataOutputStream out, ConstantPool cpool) throws IOException, ByteCodeException {
        startPC.writeIndex(out, cpool);
        endPC.writeIndex(out, cpool);
        handlerPC.writeIndex(out, cpool);
        if (catchType != null) {
            catchType.writeIndex(out, cpool);
        } else {
            U2.ZERO.write(out, cpool);
        }
    }

    @Override
    public void add(ConstantPool cpool) throws ByteCodeException {
        if (catchType != null) {
            catchType.add(cpool);
        }
    }
    
    public ClassInfo getCatchType() {
        return catchType;
    }

    public Operand getEndPC() {
        return endPC;
    }

    public Operand getHandlerPC() {
        return handlerPC;
    }

    public Operand getStartPC() {
        return startPC;
    }
}
