/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.bytecode.attributes.code;

import teajay.bytecode.OpCode;
import teajay.bytecode.U1;
import teajay.bytecode.U2;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public final class FStore extends SimpleInstruction {
    private final int n;

    public FStore(int n) throws ByteCodeException {
        this.n = n;
        switch (n) {
            case 0:
                this.setOpcode(OpCode.fstore_0);
                break;
            case 1:
                this.setOpcode(OpCode.fstore_1);
                break;
            case 2:
                this.setOpcode(OpCode.fstore_2);
                break;
            case 3:
                this.setOpcode(OpCode.fstore_3);
                break;
            default:
                if (n <= 0xFF && n >= 0) {
                    this.setOpcode(OpCode.fstore);
                    this.setOperands(new U1(n));
                } else if (n <= 0xFFFF && n >= 0) {
                    this.setOpcode(OpCode.wide1);
                    this.setOperands(OpCode.fstore, new U2(n));
                } else {
                    throw new ByteCodeException("fstore: local variable index out of bounds " + n);
                }
        }
    }

    @Override
    int getLocalModifier() {
        return n + 1;
    }
}