/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.bytecode.attributes.code;

import teajay.bytecode.*;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 * This class implements the iinc instruction.
 * @author Michael Kußmann <michi@outx.de>
 */
public final class IInc extends SimpleInstruction {
    private final int index;
    
    /**
     * Creates a new instruction.
     * @param index The index of the local variable.
     * @param incr The amount to be added. The amount is signed and may be inside the range of a short.
     * @throws ByteCodeException if index or incr are out of bounds.
     */
    public IInc(int index, int incr) throws ByteCodeException {
        this.index = index;
        if (index <= 0xFF && incr <= Byte.MAX_VALUE && incr >= Byte.MIN_VALUE && index >= 0) {
            this.setOpcode(OpCode.iinc);
            this.setOperands(new U1(index), new S1(incr));
        } else if (index <= 0xFFFF && incr <= Short.MAX_VALUE && incr >= Short.MIN_VALUE && index >= 0) {
            this.setOpcode(OpCode.wide2);
            this.setOperands(OpCode.iinc, new U2(index), new S2(incr));
        } else {
            throw new ByteCodeException("iinc: index or increment out of bounds: (" + index + ", " + incr + ")");
        }
    }

    @Override
    int getLocalModifier() {
        return index + 1;
    }
}
