/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.bytecode.attributes.code;

import teajay.bytecode.OpCode;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public enum IMode {

    eq(OpCode.if_icmpeq, OpCode.ifeq),
    ne(OpCode.if_icmpne, OpCode.ifne),
    lt(OpCode.if_icmplt, OpCode.iflt),
    ge(OpCode.if_icmpge, OpCode.ifge),
    gt(OpCode.if_icmpgt, OpCode.ifgt),
    le(OpCode.if_icmple, OpCode.ifle);
    
    final OpCode[] opcode;

    private IMode(OpCode... code) {
        this.opcode = code;
    }

    public IMode negate() {
        switch (this) {
            case eq:
                return ne;
            case ne:
                return eq;
            case lt:
                return ge;
            case ge:
                return lt;
            case gt:
                return le;
            case le:
                return gt;
            default:
                throw new RuntimeException("Unreachable switch statement reached!");
        }
    }
    
    OpCode getForIfICmp() {
        return opcode[0];
    }
    
    OpCode getForIf0() {
        return opcode[1];
    }

}
