/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.bytecode.attributes.code;

import java.util.List;
import teajay.bytecode.OpCode;
import teajay.bytecode.cpool.FieldDescriptor;
import teajay.bytecode.cpool.MethodDescriptor;
import teajay.bytecode.cpool.MethodRefInfo;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
abstract class InvokeInstruction extends SimpleInstruction {
    private final List<FieldDescriptor> params;
    private final FieldDescriptor returnType;
    private final boolean popsRef;
    
    protected InvokeInstruction(MethodRefInfo method, OpCode opcode) {
        MethodDescriptor descr = method.getNameAndType().getDescriptor();
        this.params = descr.getParameters();
        this.returnType = descr.getReturnType();
        switch (opcode) {
            case invokestatic:
                this.popsRef = false;
                break;
            default:
                this.popsRef = true;
        }
        this.setOpcode(opcode);
    }

    @Override
    int getPush() {
        return calcPush(returnType);
    }

    @Override
    int getPop() {
        return popsRef ? calcPop(params) + 1 : calcPop(params);
    }
    
    @Override
    int getLocalModifier() {
        return 0;
    }

    protected static int calcPop(List<FieldDescriptor> fields) {
        int count = fields.size();
        for (FieldDescriptor fd : fields) {
            if (fd.isLongOrDouble()) {
                count++;
            }
        }
        return count;
    }
    
    protected static int calcPush(FieldDescriptor type) {
        if (type.isVoid()) {
            return 0;
        } else if (type.isLongOrDouble()) {
            return 2;
        } else {
            return 1;
        }
    }
}
