/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.bytecode.attributes.code;

import java.io.DataOutputStream;
import java.io.IOException;
import teajay.bytecode.OpCode.OpSize;
import teajay.bytecode.Operand;
import teajay.bytecode.U2;
import teajay.bytecode.cpool.ConstantPool;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public class Label implements Operand {

    private JVMInstruction branchTarget = null;
    private final String name;

    public Label() {
        this.name = null;
    }

    public Label(String name) {
        this.name = name;
    }

    public String getName() {
        return name;
    }

    @Override
    public OpSize getDefaultLengthAsOperand() {
        return OpSize.U2;
    }

    @Override
    public long getAddress(ConstantPool cpool) throws ByteCodeException {
        if (branchTarget != null) {
            return branchTarget.getAddress(cpool);
        } else {
            throw new ByteCodeException("Unbound label encountered: " + this);
        }
    }

    public JVMInstruction getTarget() {
        return branchTarget;
    }

    @Override
    public void writeIndex(DataOutputStream out, ConstantPool cpool) throws IOException, ByteCodeException {
        new U2(getAddress(cpool)).write(out, cpool);
    }

    @Override
    public void write(DataOutputStream out, ConstantPool cpool) throws IOException, ByteCodeException {
        throw new ByteCodeException("illegal invocation of Label.write(DataOutputStream)");
    }

    final void setTarget(JVMInstruction branchTarget) throws ByteCodeException {
        if (this.branchTarget == null) {
            this.branchTarget = branchTarget;
        } else {
            throw new ByteCodeException(this + " is already in use");
        }
    }
    
    public boolean isBound() {
        return this.branchTarget != null;
    }

    final void reset() {
        this.branchTarget = null;
    }

    @Override
    public final boolean equals(Object obj) {
        return obj == this;
    }

    @Override
    public final int hashCode() {
        return super.hashCode();
    }

    @Override
    public String toString() {
        if (name != null) {
            return "label (" + name + "): " + (branchTarget != null ? branchTarget : "<unbound>");
        } else {
            return "unnamed label: " + (branchTarget != null ? branchTarget : "<unbound>");
        }
    }
}
