/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.bytecode.attributes.code;

import java.io.DataOutputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import teajay.bytecode.U2;
import teajay.bytecode.U4;
import teajay.bytecode.attributes.AttributeInfo;
import teajay.bytecode.cpool.ConstantPool;
import teajay.bytecode.cpool.Utf8Info;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public class LineNumberTable extends AttributeInfo {

    public static final String NAME = "LineNumberTable";
    Map<JVMInstruction, U2> lineNumberMap = new HashMap<>();

    public LineNumberTable() {
        super(new Utf8Info(NAME));
    }

    @Override
    public U4 getLength(ConstantPool cpool) throws ByteCodeException {
        return new U4(2 + lineNumberMap.size() * 4);
    }

    public void addMapping(JVMInstruction startPC, int lineNumber) throws ByteCodeException {
        if (lineNumber < 0 || lineNumber > 0xFFFF) {
            throw new ByteCodeException("LineNumberTable: line number out of bounds (16bit unsigned short): " + lineNumber);
        }
        lineNumberMap.put(startPC, new U2(lineNumber));
    }

    @Override
    public void write(DataOutputStream out, ConstantPool cpool) throws ByteCodeException, IOException {
        super.write(out, cpool);
        new U2(lineNumberMap.size()).write(out, cpool);
        for (Entry<JVMInstruction, U2> entry : lineNumberMap.entrySet()) {
            entry.getKey().writeIndex(out, cpool);
            entry.getValue().write(out, cpool);
        }
    }
}
