/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.bytecode.attributes.code;

import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import teajay.bytecode.cpool.ClassInfo;
import teajay.bytecode.cpool.ConstantPool;
import teajay.bytecode.cpool.FieldDescriptor;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
abstract class LocalVerifierStackMapFrame extends StackMapFrame {

    private final VerificationTypeList list;

    protected LocalVerifierStackMapFrame(JVMInstruction start, VerificationTypeList list) {
        super(start);
        this.list = list;
    }

    public void addLocal(VerificationType... type) throws ByteCodeException {
        addLocal(Arrays.asList(type));
    }

    public void addLocal(List<VerificationType> types) throws ByteCodeException {
        for (VerificationType type : types) {
            list.add(type);
        }
    }

    @Deprecated
    public void addLocalTop() throws ByteCodeException {
        list.add(new VerificationType(VariableInfo.Top));
    }

    @Deprecated
    public void addLocalInteger() throws ByteCodeException {
        list.add(new VerificationType(VariableInfo.Integer));
    }

    @Deprecated
    public void addLocalLong() throws ByteCodeException {
        list.add(new VerificationType(VariableInfo.Long));
    }

    @Deprecated
    public void addLocalDouble() throws ByteCodeException {
        list.add(new VerificationType(VariableInfo.Double));
    }

    @Deprecated
    public void addLocalFloat() throws ByteCodeException {
        list.add(new VerificationType(VariableInfo.Float));
    }

    @Deprecated
    public void addLocalNull() throws ByteCodeException {
        list.add(new VerificationType(VariableInfo.Null));
    }

    @Deprecated
    public void addLocalUninitializedThis() throws ByteCodeException {
        list.add(new VerificationType(VariableInfo.UninitializedThis));
    }

    @Deprecated
    public void addLocalObject(ClassInfo className) throws ByteCodeException {
        list.add(new VerificationType(className));
    }

    @Deprecated
    public void addLocalUninitialized(JVMInstruction newInstr) throws ByteCodeException {
        list.add(new VerificationType(newInstr));
    }

    public void addLocalObjects(ClassInfo... classNames) throws ByteCodeException {
        for (ClassInfo className : classNames) {
            list.add(VerificationType.valueOf(className));
        }
    }

    public void addLocalObjects(Collection<ClassInfo> classNames) throws ByteCodeException {
        for (ClassInfo className : classNames) {
            list.add(VerificationType.valueOf(className));
        }
    }

    public void addLocalTypes(Collection<FieldDescriptor> types) throws ByteCodeException {
        for (FieldDescriptor type : types) {
            list.add(VerificationType.valueOf(type));
        }
    }

    public final VerificationTypeList getLocalTypeList() {
        return this.list;
    }

    @Override
    public void add(ConstantPool cpool) throws ByteCodeException {
        getLocalTypeList().add(cpool);
    }
}
