/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.bytecode.attributes.code;

import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Collection;
import teajay.bytecode.OpCode;
import teajay.bytecode.Operand;
import teajay.bytecode.cpool.ConstantPool;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public class PlaceholderInstruction extends JVMInstruction {

    private JVMInstruction instruction;
    private final String name;
    private boolean locked = false;

    public PlaceholderInstruction() {
        this.name = null;
    }

    public PlaceholderInstruction(String name) {
        this.name = name;
    }

    public void setInstruction(JVMInstruction instruction) {
        if (!locked) {
            this.instruction = instruction;
        } else {
            throw new IllegalStateException("Placeholder has already been written.");
        }
    }

    private void checkNull() throws ByteCodeException {
        if (instruction == null) {
            throw new ByteCodeException("unbound placeholder (" + name + ")");
        }
    }

    @Override
    int getLength() throws ByteCodeException {
        checkNull();
        return instruction.getLength();
    }

    @Override
    public OpCode opcode() throws ByteCodeException {
        checkNull();
        return instruction.opcode();
    }

    @Override
    protected Collection<? extends Operand> operands() throws ByteCodeException {
        checkNull();
        return instruction.operands();
    }

    @Override
    int getLocalModifier() throws ByteCodeException {
        checkNull();
        return instruction.getLocalModifier();
    }

    @Override
    int getPush() throws ByteCodeException {
        checkNull();
        return instruction.getPush();
    }

    @Override
    int getPop() throws ByteCodeException {
        checkNull();
        return instruction.getPop();
    }

    @Override
    boolean validate(ConstantPool cpool) throws ByteCodeException {
        checkNull();
        instruction.setAddress(getAddress(cpool));
        return instruction.validate(cpool);
    }

    @Override
    public void write(DataOutputStream out, ConstantPool cpool) throws IOException, ByteCodeException {
        checkNull();
        instruction.setAddress(getAddress(cpool));
        instruction.write(out, cpool);
        locked = true;
    }
    
    public String getName() {
        return name;
    }
    
    @Override
    public String toString() {
        if (instruction != null) {
            return "bound placeholder for " + super.toString() + " (" + name + ")";
        } else {
            return "unbound placeholder instruction (" + name + ")";
        }
    }
}
