/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.bytecode.attributes.code;

import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Collections;
import java.util.List;
import teajay.bytecode.U1;
import teajay.bytecode.U4;
import teajay.bytecode.cpool.ConstantPool;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
final class SameFrameExtended extends StackMapFrame {
    public static final int FRAME_TYPE = 251;

    SameFrameExtended(JVMInstruction start) {
        super(start);
    }

    @Override
    U1 getFrameType() throws ByteCodeException {
        return new U1(FRAME_TYPE);
    }

    @Override
    public void add(ConstantPool cpool) throws ByteCodeException {
    }

    @Override
    U4 getLength() throws ByteCodeException {
        return new U4(1 + 2);
    }

    @Override
    List<VerificationType> getStack() throws ByteCodeException {
        return Collections.emptyList();
    }

    @Override
    List<VerificationType> getLocals() throws ByteCodeException {
        return this.getPredecessor().getLocals();
    }
    
    @Override
    public void write(DataOutputStream out, ConstantPool cpool) throws IOException, ByteCodeException {
        super.write(out, cpool);
        offsetDelta().write(out, cpool);
    }
}
