/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.bytecode.attributes.code;

import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import teajay.bytecode.cpool.ClassInfo;
import teajay.bytecode.cpool.ConstantPool;
import teajay.bytecode.cpool.FieldDescriptor;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
abstract class StackVerifierStackMapFrame extends StackMapFrame {

    private final VerificationTypeList list;

    protected StackVerifierStackMapFrame(JVMInstruction start, VerificationTypeList list) {
        super(start);
        this.list = list;
    }
    
    public void addStack(VerificationType... type) throws ByteCodeException {
        addStack(Arrays.asList(type));
    }
    
    public void addStack(List<VerificationType> types) throws ByteCodeException {
        for (VerificationType type : types) {
            list.add(type);
        }
    }
    
    @Deprecated
    public void addStackTop() throws ByteCodeException {
        list.add(new VerificationType(VariableInfo.Top));
    }
    @Deprecated
    public void addStackInteger() throws ByteCodeException {
        list.add(new VerificationType(VariableInfo.Integer));
    }
    @Deprecated
    public void addStackFloat() throws ByteCodeException {
        list.add(new VerificationType(VariableInfo.Float));
    }
    @Deprecated
    public void addStackDouble() throws ByteCodeException {
        list.add(new VerificationType(VariableInfo.Double));
    }
    @Deprecated
    public void addStackLong() throws ByteCodeException {
        list.add(new VerificationType(VariableInfo.Long));
    }
    @Deprecated
    public void addStackNull() throws ByteCodeException {
        list.add(new VerificationType(VariableInfo.Null));
    }
    @Deprecated
    public void addStackUninitializedThis() throws ByteCodeException {
        list.add(new VerificationType(VariableInfo.UninitializedThis));
    }
    @Deprecated
    public void addStackUninitialized(JVMInstruction newInstr) throws ByteCodeException {
        list.add(new VerificationType(newInstr));
    }
    @Deprecated
    public void addStackObject(ClassInfo className) throws ByteCodeException {
        list.add(new VerificationType(className));
    }

    public void addStackObjects(ClassInfo... classNames) throws ByteCodeException {
        for (ClassInfo className : classNames) {
            list.add(VerificationType.makeObject(className));
        }
    }

    public void addStackObjects(Collection<ClassInfo> classNames) throws ByteCodeException {
        for (ClassInfo className : classNames) {
            list.add(VerificationType.makeObject(className));
        }
    }

    public void addStackTypes(Collection<FieldDescriptor> types) throws ByteCodeException {
        for (FieldDescriptor descr : types) {
            list.add(VerificationType.valueOf(descr));
        }
    }

    public final VerificationTypeList getStackTypeList() {
        return this.list;
    }

    @Override
    public void add(ConstantPool cpool) throws ByteCodeException {
        getStackTypeList().add(cpool);
    }
}
