/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.bytecode.cpool;

import java.io.DataInputStream;
import java.io.IOException;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public enum CPoolTag {

    Class(7),
    FieldRef(9),
    MethodRef(10),
    InterfaceMethodRef(11),
    String(8),
    Integer(3),
    Float(4),
    Long(5),
    Double(6),
    NameAndType(12),
    Utf8(1),
    MethodHandle(15),
    MethodType(16),
    InvokeDynamic(18),
    Dummy(-1);
    public final int intValue;

    private CPoolTag(int value) {
        this.intValue = value;
    }

    static CPoolTag forValue(int value) {
        if (value > 0) {
            for (CPoolTag tag : CPoolTag.values()) {
                if (value == tag.intValue) {
                    return tag;
                }
            }
        }
        return null;
    }

    static CPoolTag read(DataInputStream in) throws IOException, ByteCodeException {
        CPoolTag tag = forValue(in.readUnsignedByte());
        if (tag != null) {
            return tag;
        } else {
            throw new ByteCodeException("unknown constant pool tag");
        }
    }
}
