/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.bytecode.cpool;

import teajay.bytecode.exceptions.MalformedClassNameException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public final class FieldRefInfo extends CPoolRef<FieldDescriptor> {
    
    public static FieldRefInfo make(ClassInfo className, String name, FieldDescriptor type) {
        return new FieldRefInfo(className, new NameAndTypeInfo<>(name, type));
    }
    
    public static FieldRefInfo make(String className, String name, FieldDescriptor type) throws MalformedClassNameException {
        return new FieldRefInfo(new ClassInfo(className), new NameAndTypeInfo<>(name, type));
    }
    
    public static FieldRefInfo make(String className, String name, String type) throws MalformedClassNameException {
        return new FieldRefInfo(new ClassInfo(className), new NameAndTypeInfo<>(name, FieldDescriptor.fromString(type)));
    }
    
    public FieldRefInfo(ClassInfo className, NameAndTypeInfo<FieldDescriptor> nameAndType) {
        super(CPoolTag.FieldRef, className, nameAndType);
    }
    
    @Override
    public boolean equals(Object obj) {
        return (obj instanceof FieldRefInfo) && super.equals(obj);
    }

    @Override
    public int hashCode() {
        return 71 * super.hashCode();
    }
}
