/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.bytecode.cpool;

import teajay.bytecode.exceptions.MalformedClassNameException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public class MethodRefInfo extends CPoolRef<MethodDescriptor> {
    
    public static MethodRefInfo make(ClassInfo className, String name, FieldDescriptor returnType, FieldDescriptor... parameterTypes) {
        return new MethodRefInfo(className, new NameAndTypeInfo<>(name, new MethodDescriptor(returnType, parameterTypes)));
    }
    
    public static MethodRefInfo make(String className, String name, FieldDescriptor returnType, FieldDescriptor... parameterTypes) throws MalformedClassNameException {
        return new MethodRefInfo(className, new NameAndTypeInfo<>(name, new MethodDescriptor(returnType, parameterTypes)));
    }

    public static MethodRefInfo make(String className, String name, String returnType, String... parameterTypes) throws MalformedClassNameException {
        FieldDescriptor[] parameters = new FieldDescriptor[parameterTypes.length];
        for (int i = 0; i < parameters.length; i++) {
            parameters[i] = FieldDescriptor.fromString(parameterTypes[i]);
        }
        return new MethodRefInfo(className, new NameAndTypeInfo<>(name, new MethodDescriptor(FieldDescriptor.fromString(returnType), parameters)));
    }

    
    public MethodRefInfo(ClassInfo className, NameAndTypeInfo<MethodDescriptor> nameAndType) {
        this(CPoolTag.MethodRef, className, nameAndType);
    }
    
    public MethodRefInfo(String className, NameAndTypeInfo<MethodDescriptor> nameAndType) throws MalformedClassNameException {
        this(CPoolTag.MethodRef, new ClassInfo(className), nameAndType);
    }
    
    protected MethodRefInfo(CPoolTag tag, ClassInfo className, NameAndTypeInfo<MethodDescriptor> nameAndType) {
        super(tag, className, nameAndType);
        switch (tag) {
            case MethodRef:
            case InterfaceMethodRef:
                break;
            default:
                throw new RuntimeException("Invalid constant pool tag for methodref info");
        }
    }
    
    @Override
    public boolean equals(Object obj) {
        return (obj instanceof MethodRefInfo) && super.equals(obj);
    }

    @Override
    public int hashCode() {
        return 37 * super.hashCode();
    }
}
