/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.bytecode.cpool;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import teajay.bytecode.OpCode.OpSize;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public class U2Index extends CPoolIndex {

    public U2Index(int index) {
        super(index);
    }

    @Override
    public void write(DataOutputStream out, ConstantPool cpool) throws IOException {
        out.writeShort(this.getValue());
    }
    
    @Override
    public void writeIndex(DataOutputStream out, ConstantPool cpool) throws IOException {
        write(out, cpool);
    }    

    @Override
    public U1Index toU1Index() {
        return new U1Index(getValue());
    }

    @Override
    public U2Index toU2Index() {
        return this;
    }

    @Override
    public OpSize getDefaultLengthAsOperand() {
        return OpSize.U2;
    }
    
    @Override
    public boolean equals(Object obj) {
        return (obj instanceof U2Index) && super.equals(obj);
    }

    @Override
    public int hashCode() {
        int hash = 79 * super.hashCode();
        return hash;
    }
    
    public static U2Index read(DataInputStream in) throws IOException {
        return new U2Index(in.readUnsignedShort());
    }
}
