/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package abstractsyntaxtree;

import codegeneration.Visitor;
import java.util.ArrayList;
import lexer.Symbol;
import exceptions.CompilerException;

/**
 *
 * @author Michael Burzan
 */
public class ClassBodyDeclarationNode extends SyntaxNode {

    private boolean isSemicolon = false;
    private ArrayList<ModifierNode> memberModifiers = null;
    private MemberDeclNode member = null;
    private boolean hasStatic = false;
    private BlockNode block = null;

    public ClassBodyDeclarationNode(Symbol s, boolean isSemicolon) {
        super(s);
        this.isSemicolon = isSemicolon;
    }

    public ClassBodyDeclarationNode(Symbol s, ArrayList<ModifierNode> memberModifiers,
            MemberDeclNode member) {
        super(s);
        this.memberModifiers = memberModifiers;
        this.member = member;
    }

    public ClassBodyDeclarationNode(Symbol s, boolean hasStatic, BlockNode block) {
        super(s);
        this.hasStatic = hasStatic;
        this.block = block;
    }

    public boolean isEmptyClass() {
        return this.isSemicolon;
    }

    public boolean hasStaticBlock() {
        return this.hasStatic;
    }

    @Override
    public void accept(Visitor visitor) throws CompilerException {
        visitor.begin(this);
        Visitor v = visitor.visit(this);
        checkNull(this.memberModifiers, v);
        checkNull(this.member, v);
        checkNull(this.block, v);

        visitor.end(this);
    }
}
