/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package abstractsyntaxtree;

import codegeneration.Visitor;
import lexer.Symbol;
import exceptions.CompilerException;

/**
 *
 * @author Michael Burzan
 */
public class ForControlNode extends SyntaxNode {

    private ForVarControlNode forVarControl = null;
    private ForInitNode forInit = null;
    private ExpressionNode expression = null;
    private ForUpdateNode forUpdate = null;

    public ForControlNode(Symbol s, ForVarControlNode forVarControl) {
        super(s);
        this.forVarControl = forVarControl;
    }

    public ForControlNode(Symbol s, ForInitNode forInit) {
        super(s);
        this.forInit = forInit;
    }

    public ForControlNode(Symbol s, ForInitNode forInit, ExpressionNode expression) {
        super(s);
        this.forInit = forInit;
        this.expression = expression;
    }

    public ForControlNode(Symbol s, ForInitNode forInit, ExpressionNode expression,
            ForUpdateNode forUpdate) {
        super(s);
        this.forInit = forInit;
        this.expression = expression;
        this.forUpdate = forUpdate;
    }

    public ForControlNode(Symbol s, ForInitNode forInit, ForUpdateNode forUpdate) {
        super(s);
        this.forInit = forInit;
        this.forUpdate = forUpdate;
    }

    @Override
    public void accept(Visitor visitor) throws CompilerException {
        visitor.begin(this);
        Visitor v = visitor.visit(this);
        checkNull(this.forInit, v);
        checkNull(this.expression, v);
        checkNull(this.forUpdate, v);
        checkNull(this.forVarControl, v);
        visitor.end(this);
    }
}
