/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package abstractsyntaxtree;

import codegeneration.Visitor;
import java.util.LinkedList;
import java.util.List;
import lexer.Symbol;
import exceptions.CompilerException;

/**
 *
 * @author Michael Burzan
 */
public class IdentifierSuffixNode extends SyntaxNode {

    private int edgeClampsCnt = 0;

    public static enum Suffixes {

        CLASS,
        THIS,
        SUPER,
        NEW
    }

    public static enum SuffixType {

        SUFFIX_CALL, CALL, SUFFIX, INNER_CREATOR, EXPLICIT_GENERIC_INVOCATION,
        NON_WILDCARD_TYPE_ARGUMENTS, ARRAY_EXPRESSION, ARRAY_SUFFIX;
    }
    private String identifier = null;
    private List<String> identifierList = null;
    private SuffixType suffixType = null;
    private Suffixes suffix = null;
    private ExplicitGenericInvocationNode explicitGenericInvoc = null;
    private NonWildcardTypeArgumentsNode nonWildcardTypeArgs = null;
    private InnerCreatorNode innerCreator = null;

    public SuffixType getSuffixType() {
        return this.suffixType;
    }

    public static class EdgeClampsExpression extends SyntaxNode {

        private ExpressionNode expression = null;

        public EdgeClampsExpression(Symbol s, ExpressionNode expression) {
            super(s);
            this.expression = expression;
        }

        @Override
        public void accept(Visitor visitor) throws CompilerException {
            checkNull(this.expression, visitor);
        }
    }
    private LinkedList<EdgeClampsExpression> edgeClampsExpressionList = null;
    private ArgumentsNode arguments = null;

    public IdentifierSuffixNode(Symbol s, IdentifierSuffixNode node, String identifier, List<String> identifierList) {
        super(s);
        this.arguments = node.arguments;
        this.edgeClampsCnt = node.edgeClampsCnt;
        this.edgeClampsExpressionList = node.edgeClampsExpressionList;
        this.explicitGenericInvoc = node.explicitGenericInvoc;
        this.innerCreator = node.innerCreator;
        this.nonWildcardTypeArgs = node.nonWildcardTypeArgs;
        this.suffix = node.suffix;
        this.suffixType = node.suffixType;
        this.identifier = identifier;
        this.identifierList = identifierList;
    }

    public IdentifierSuffixNode(Symbol s, int edgeClampsCnt, Suffixes suffix) {
        super(s);
        this.edgeClampsCnt = edgeClampsCnt;
        this.suffix = suffix;
        this.suffixType = SuffixType.ARRAY_SUFFIX;
    }

    public IdentifierSuffixNode(Symbol s, LinkedList<EdgeClampsExpression> edgeClampsExpressionList) {
        super(s);
        this.edgeClampsExpressionList = edgeClampsExpressionList;
        this.edgeClampsCnt = edgeClampsExpressionList.size();
        this.suffixType = SuffixType.ARRAY_EXPRESSION;
    }

    public IdentifierSuffixNode(Symbol s, ArgumentsNode arguments) {
        super(s);
        this.arguments = arguments;
        this.suffixType = SuffixType.CALL;
    }

    public IdentifierSuffixNode(Symbol s, Suffixes suffix) {
        super(s);
        this.suffix = suffix;
        this.suffixType = SuffixType.SUFFIX;
    }

    public IdentifierSuffixNode(Symbol s, ExplicitGenericInvocationNode explicitGenericInvoc) {
        super(s);
        this.explicitGenericInvoc = explicitGenericInvoc;
        this.suffixType = SuffixType.EXPLICIT_GENERIC_INVOCATION;
    }

    public IdentifierSuffixNode(Symbol s, Suffixes suffix, ArgumentsNode arguments) {
        super(s);
        this.suffix = suffix;
        this.arguments = arguments;
        this.suffixType = SuffixType.SUFFIX_CALL;
    }

    public IdentifierSuffixNode(Symbol s, Suffixes suffix,
            NonWildcardTypeArgumentsNode nonWildcardTypeArgs,
            InnerCreatorNode innerCreator) {
        super(s);
        this.suffix = suffix;
        this.nonWildcardTypeArgs = nonWildcardTypeArgs;
        this.innerCreator = innerCreator;
        this.suffixType = SuffixType.NON_WILDCARD_TYPE_ARGUMENTS;

    }

    public IdentifierSuffixNode(Symbol s, Suffixes suffix,
            InnerCreatorNode innerCreator) {
        super(s);
        this.suffix = suffix;
        this.innerCreator = innerCreator;
        this.suffixType = SuffixType.INNER_CREATOR;
    }

    public String getIdentifier() {
        return this.identifier;
    }

    public List<String> getIdentifierList() {
        return this.identifierList;
    }

    public int getDim() {
        return this.edgeClampsCnt;
    }

    public Suffixes getSuffixes() {
        return this.suffix;
    }

    @Override
    public void accept(Visitor visitor) throws CompilerException {
        visitor.begin(this);
        Visitor v = visitor.visit(this);
        checkNull(this.arguments, v);
        checkNull(this.explicitGenericInvoc, v);
        checkNull(this.nonWildcardTypeArgs, v);
        checkNull(this.innerCreator, v);
       // checkNull(this.edgeClampsExpressionList,v);
        if (this.edgeClampsExpressionList != null) {
            for (EdgeClampsExpression exp : this.edgeClampsExpressionList) {
                exp.accept(v);
            }
        }
        visitor.end(this);
    }
}
