/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package abstractsyntaxtree;

import codegeneration.Visitor;
import lexer.Symbol;
import exceptions.CompilerException;

/**
 *
 * @author Michael Burzan
 */
public class ModifierNode extends SyntaxNode {

    public enum Modifier {

        PUBLIC,
        PROTECTED,
        PRIVATE,
        STATIC,
        ABSTRACT,
        FINAL,
        NATIVE,
        SYNCHRONIZED,
        TRANSIENT,
        VOLATILE,
        STRICTFP,
        SYNTHETIC
    }
    private Modifier modifier = null;
    private AnnotationNode annotation = null;

    public ModifierNode(Symbol s, AnnotationNode annotation) {
        super(s);
        this.annotation = annotation;
    }

    public ModifierNode(Symbol s, Modifier modifier) {
        super(s);
        this.modifier = modifier;
    }

    /**
     * Check is null in concrete visitor
     */
    public Modifier getModifier() {
        return this.modifier;
    }

    @Override
    public void accept(Visitor visitor) throws CompilerException {
        visitor.begin(this);
        Visitor v = visitor.visit(this);
        checkNull(this.annotation, v);
        visitor.end(this);
    }
}
