/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package abstractsyntaxtree;

import abstractsyntaxtree.BasicTypeNode.BasicType;
import codegeneration.Visitor;
import java.util.LinkedList;
import java.util.List;
import lexer.Symbol;
import exceptions.CompilerException;

/**
 *
 * @author Michael Burzan
 */
public class PrimaryNode extends SyntaxNode {

    public static enum ThisOrSuper {

        This,
        Super
    }
    private ThisOrSuper thisOrSuper;
    private LiteralNode literal = null;
    private TypeNode castType = null;
    private ExpressionNode castExpression = null;
    private ParExpressionNode parExpression = null;
    private ArgumentsNode arguments = null;
    private SuperSuffixNode superSuffix = null;
    private CreatorNode creator = null;
    private NonWildcardTypeArgumentsNode nonWildcardTypeArgs = null;
    private ExplicitGenericInvocationSuffixNode explicitGenericInvocationSuffix = null;
    private String identifier = null;
    private LinkedList<String> identifierList = null;
    private IdentifierSuffixNode identifierSuffix = null;
    private BasicTypeNode basicType = null;
    private int edgeClampsCnt = 0;
    private boolean isVoidClass = false;
    // private boolean isThis = false;
    private boolean hasDotClass = false;
    private ClosureCreatorNode closureCreator = null;

    public static enum PrimaryType {

        NEW_TYPE,
        LITERAL_TYPE, PARANTHISIS_TYPE
    }
    public PrimaryType primaryType;

    public PrimaryType getPrimaryType() {
        return this.primaryType;
    }

    public String getIdentifier() {
        return this.identifier;
    }

    public int getDim() {
        return edgeClampsCnt;
    }

    public boolean hasDotClass() {
        return this.hasDotClass;
    }
    
    public BasicType getBasicType() {
        if (this.basicType != null) {
            return this.basicType.getType();
        } else {
            return null;
        }
    }

    public boolean isThis() {
        return ThisOrSuper.This == this.thisOrSuper;
    }

    public boolean isVoidClass() {
        return this.isVoidClass;
    }

    public PrimaryNode(Symbol s) {
        super(s);
        this.thisOrSuper = ThisOrSuper.This;
    }

    public PrimaryNode(Symbol s, LiteralNode literal) {
        super(s);
        this.literal = literal;
        this.primaryType = PrimaryType.LITERAL_TYPE;
    }

    public PrimaryNode(Symbol s, ParExpressionNode parExpression) {
        super(s);
        this.parExpression = parExpression;
        this.primaryType = PrimaryType.PARANTHISIS_TYPE;
    }

    public PrimaryNode(Symbol s, ArgumentsNode arguments) {
        super(s);
        this.arguments = arguments;
    }

    public PrimaryNode(Symbol s, SuperSuffixNode superSuffix) {
        super(s);
        this.superSuffix = superSuffix;
    }

    public PrimaryNode(Symbol s, CreatorNode creator) {
        super(s);
        this.creator = creator;
        this.primaryType = PrimaryType.NEW_TYPE;
    }

    public PrimaryNode(Symbol s, ThisOrSuper thisOrSuper) {
        super(s);
        this.thisOrSuper = thisOrSuper;
    }

    public PrimaryNode(Symbol s, NonWildcardTypeArgumentsNode nonWildcardTypeArgs,
            ExplicitGenericInvocationSuffixNode explicitGenericInvocationSuffix) {
        super(s);
        this.nonWildcardTypeArgs = nonWildcardTypeArgs;
        this.explicitGenericInvocationSuffix = explicitGenericInvocationSuffix;
    }

    public PrimaryNode(Symbol s, NonWildcardTypeArgumentsNode nonWildcardTypeArgs,
            ArgumentsNode arguments) {
        super(s);
        this.nonWildcardTypeArgs = nonWildcardTypeArgs;
        this.arguments = arguments;
    }

    public PrimaryNode(Symbol s, String identifier, LinkedList<String> identifierList) {
        super(s);
        this.identifier = identifier;
        this.identifierList = identifierList;
    }

    public PrimaryNode(Symbol s, String identifier, LinkedList<String> identifierList,
            IdentifierSuffixNode identifierSuffix) {
        super(s);
        if (identifierSuffix != null) {
            this.identifier = null;
            this.identifierList = null;
            this.identifierSuffix = new IdentifierSuffixNode(s, identifierSuffix, identifier, identifierList);
        } else {
            this.identifier = identifier;
            this.identifierList = identifierList;
        }
    }
    
    public boolean isPlainIdent() {
        return identifier != null;
    }

    public PrimaryNode(Symbol s, BasicTypeNode basicType, int edgeClampsCnt) {
        super(s);
        this.basicType = basicType;
        this.edgeClampsCnt = edgeClampsCnt;
    }

    public PrimaryNode(Symbol s, BasicTypeNode basicType, int edgeClampsCnt, boolean hasDotClass) {
        super(s);
        this.basicType = basicType;
        this.edgeClampsCnt = edgeClampsCnt;
        this.hasDotClass = hasDotClass;
    }

    public PrimaryNode(Symbol s, boolean isVoidClass) {
        super(s);
        this.isVoidClass = isVoidClass;
    }

    public PrimaryNode(Symbol s, ExpressionNode exp, TypeNode type) {
        super(s);
        this.castType = type;
        this.castExpression = exp;

    }

    public PrimaryNode(Symbol s, ClosureCreatorNode closureCreator) {
        super(s);
        this.closureCreator = closureCreator;
    }

    public List<String> getIdentifierList() {
        return this.identifierList;
    }

    public boolean isSuper() {
        return thisOrSuper == ThisOrSuper.Super;
    }

    public boolean isClosureDef() {
        return this.closureCreator == null ? false : true;
    }
    
    public boolean isCast() {
        return castType != null;
    }

    @Override
    public void accept(Visitor visitor) throws CompilerException {
        visitor.begin(this);
        Visitor v = visitor.visit(this);
        checkNull(this.literal, v);
        checkNull(this.parExpression, v);
        checkNull(this.castType, v);
        checkNull(this.castExpression, v);
        checkNull(this.arguments, v);
        checkNull(this.superSuffix, v);
        checkNull(this.creator, v);
        checkNull(this.nonWildcardTypeArgs, v);
        checkNull(this.explicitGenericInvocationSuffix, v);
        checkNull(this.identifierSuffix, v);
        checkNull(this.basicType, v);
        checkNull(this.closureCreator, v);
        visitor.end(this);
    }
}
