/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package abstractsyntaxtree;

import codegeneration.Visitor;
import lexer.Symbol;
import exceptions.CompilerException;

/**
 *
 * @author Michael Burzan
 */
public class SelectorNode extends SyntaxNode {

    private String identifier = null;
    private ArgumentsNode arguments = null;
    private ExplicitGenericInvocationNode explicitGenericInvoc = null;
    //   private boolean hasThis = false;
    private SuperSuffixNode superSuffix = null;
    private NonWildcardTypeArgumentsNode nonWildcardTypeArguments = null;
    private InnerCreatorNode innerCreator = null;
    private ExpressionNode expression = null;
    private boolean isClosure = false;

    public static enum ThisOrSuper {

        This,
        Super
    }
    private ThisOrSuper thisOrSuper;

    public boolean hasThis() {
        return this.thisOrSuper == ThisOrSuper.This;
    }

    public boolean hasSuper() {
        return this.thisOrSuper == ThisOrSuper.Super;
    }

    public boolean isClosureCall(){
        return this.isClosure;
    }
    
    public String getIdentifier() {
        return this.identifier;
    }

    public SelectorNode(Symbol s,ArgumentsNode node){
        this.arguments = node;
        this.isClosure = true;
    }
    
    public SelectorNode(Symbol s, String identifier) {
        super(s);
        this.identifier = identifier;
    }

    public SelectorNode(Symbol s, String identifier, ArgumentsNode arguments) {
        super(s);
        this.identifier = identifier;
        this.arguments = arguments;
    }

    public SelectorNode(Symbol s, ExplicitGenericInvocationNode explicitGenericInvoc) {
        super(s);
        this.explicitGenericInvoc = explicitGenericInvoc;
    }

    public SelectorNode(Symbol s, ThisOrSuper thisOrSuper) {
        super(s);
        this.thisOrSuper = thisOrSuper;
    }

    public SelectorNode(Symbol s, SuperSuffixNode superSuffix) {
        super(s);
        this.superSuffix = superSuffix;
    }

    public SelectorNode(Symbol s, NonWildcardTypeArgumentsNode nonWildcardTypeArguments,
            InnerCreatorNode innerCreator) {
        super(s);
        this.nonWildcardTypeArguments = nonWildcardTypeArguments;
        this.innerCreator = innerCreator;
    }

    public SelectorNode(Symbol s, InnerCreatorNode innerCreator) {
        super(s);
        this.innerCreator = innerCreator;
    }

    public SelectorNode(Symbol s, ExpressionNode expression) {
        super(s);
        this.expression = expression;
    }

    @Override
    public void accept(Visitor visitor) throws CompilerException {
        visitor.begin(this);
        Visitor v = visitor.visit(this);
        checkNull(this.arguments, v);
        checkNull(this.explicitGenericInvoc, v);
        checkNull(this.superSuffix, v);
        checkNull(this.nonWildcardTypeArguments, v);
        checkNull(this.innerCreator, v);
        checkNull(this.expression, v);
        visitor.end(this);
    }
}
