/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package abstractsyntaxtree;

import codegeneration.Visitor;
import lexer.Symbol;
import exceptions.CompilerException;

/**
 *
 * @author Michael Burzan
 */
public class TryStatementNode extends SyntaxNode {

    private BlockNode block = null;
    private CatchesNode catches = null;
    private FinallyNode finallyNode = null;
    private ResourceSpecificationNode resourceSpec = null;

    public TryStatementNode(Symbol s, BlockNode block, CatchesNode catches) {
        super(s);
        this.block = block;
        this.catches = catches;
    }

    public TryStatementNode(Symbol s, BlockNode block, CatchesNode catches, FinallyNode finallyNode) {
        super(s);
        this.block = block;
        this.catches = catches;
        this.finallyNode = finallyNode;
    }

    public TryStatementNode(Symbol s, BlockNode block, FinallyNode finallyNode) {
        super(s);
        this.block = block;
        this.finallyNode = finallyNode;
    }

    public TryStatementNode(Symbol s, ResourceSpecificationNode resourceSpec, CatchesNode catches) {
        super(s);
        this.catches = catches;
        this.resourceSpec = resourceSpec;
    }

    public TryStatementNode(Symbol s, ResourceSpecificationNode resourceSpec,
            CatchesNode catches, FinallyNode finallyNode) {
        super(s);
        this.catches = catches;
        this.finallyNode = finallyNode;
        this.resourceSpec = resourceSpec;
    }

    public TryStatementNode(Symbol s, ResourceSpecificationNode resourceSpec,
            FinallyNode finallyNode) {
        super(s);
        this.finallyNode = finallyNode;
        this.resourceSpec = resourceSpec;
    }

    public TryStatementNode(Symbol s, ResourceSpecificationNode resourceSpec) {
        super(s);
        this.resourceSpec = resourceSpec;
    }

    @Override
    public void accept(Visitor visitor) throws CompilerException {
        visitor.begin(this);
        Visitor v = visitor.visit(this);
        checkNull(this.block, v);
        checkNull(this.catches, v);
        checkNull(this.finallyNode, v);
        checkNull(this.resourceSpec, v);
        visitor.end(this);
    }
}
