/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package codegeneration.environment;

import abstractsyntaxtree.SyntaxNode;
import exceptions.CompilerException;
import java.io.IOException;
import java.lang.invoke.CallSite;
import java.lang.invoke.MethodHandles;
import java.lang.invoke.MethodType;
import java.util.Deque;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import stage3.states.*;
import stage3.utils.ConstructorFieldInitBuilder;
import stage3.utils.InformationCenter;
import stage3.utils.StaticConstructorFieldInitBuilder;
import stage3.utils.TypeChecker;
import symbol.LocalTable;
import symbol.MethodResolver;
import symbol.SymbolFacade;
import teajay.bytecode.ClassFile;
import teajay.bytecode.attributes.AttributeInfo;
import teajay.bytecode.attributes.bootstrap.BootstrapMethodEntry;
import teajay.bytecode.attributes.bootstrap.BootstrapMethods;
import teajay.bytecode.attributes.code.ALoad;
import teajay.bytecode.attributes.code.GetField;
import teajay.bytecode.attributes.code.InvokeDynamic;
import teajay.bytecode.attributes.code.InvokeSpecial;
import teajay.bytecode.attributes.code.JVMInstructionList;
import teajay.bytecode.attributes.code.PutField;
import teajay.bytecode.attributes.signature.Signature;
import teajay.bytecode.cpool.ClassInfo;
import teajay.bytecode.cpool.FieldDescriptor;
import teajay.bytecode.cpool.FieldRefInfo;
import teajay.bytecode.cpool.InvokeDynamicInfo;
import teajay.bytecode.cpool.MethodDescriptor;
import teajay.bytecode.cpool.MethodHandleInfo;
import teajay.bytecode.cpool.MethodRefInfo;
import teajay.bytecode.cpool.NameAndTypeInfo;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.bytecode.fields.FieldAccessFlag;
import teajay.bytecode.fields.FieldInfo;
import teajay.bytecode.methods.MethodInfo;
import teajay.bytecode.methods.MethodKind;
import teajay.generics.check.TypeUtils;
import teajay.generics.parser.Type;
import teajay.generics.writer.SignatureEncoder;
import teajay.runtime.exceptions.ResolveException;
import teajay.util.io.ClassFinder;
import teajay.util.tjattribute.TeaJayAttribute;

/**
 *
 * @author Michael Burzan
 * Manage information about typeimpls, generics, methods, expressions.
 * Has the interface to communicate with the state machine
 */
public class Stage3Environment {

    private final Stage2Environment env;
    private StaticConstructorFieldInitBuilder staticFieldInitBuilder;
    private ConstructorFieldInitBuilder constructorFieldInitBuilder;
    private MethodInfo currentMethod;
    private Type.ClassType typeOfThis;
    private LocalTable localTable;
    private State state;
    private final BootstrapMethods bootstrapMethods;
    private MethodHandleInfo newMethod;
    private ExpressionEnvironment expressionEnvironment;
    private Type expectedResultType;
    private Type currentMethodExpectedType;
    private LinkedList<HashSet<String>> genericEnv = new LinkedList<>();
    private int closureCnt = 0;
    private int indexOfSyncName = -1;
    private boolean isInSuperCall = false;

    public Stage3Environment(Stage2Environment env) {
        this.bootstrapMethods = env.getBootstrapMethods();
        this.env = env;
        try {
            MethodRefInfo ref = MethodRefInfo.make("teajay/runtime/Bootstraps", "bc",
                    FieldDescriptor.fromClass(CallSite.class), FieldDescriptor.fromClass(MethodHandles.Lookup.class),
                    FieldDescriptor.getString(), FieldDescriptor.fromClass(MethodType.class), FieldDescriptor.fromClass(Class.class));
            newMethod = new MethodHandleInfo(MethodKind.REF_invokeStatic, ref);
            //this.env.getClassFile().addAttribute(bootstrapMethods);
        } catch (ByteCodeException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }

    }
    /**
     * the current return type of the generated method
     * @return the return type of the generated method 
     */
    public Type getCurrentMethodExpectedType() {
        return currentMethodExpectedType;
    }

    public ClassFile getClassFile() {
        return this.env.getClassFile();
    }

    public TypeUtils getTypeUtils() {
        return this.env.getTypeUtils();
    }

    public StaticConstructorFieldInitBuilder getStaticConstructorFieldInitBuilder() {
        return this.staticFieldInitBuilder;
    }

    public ConstructorFieldInitBuilder getConstructorFieldInitBuilder() {
        return this.constructorFieldInitBuilder;
    }
    /**
     * 
     * @param builder to generate field initialize in constructor 
     */
    public void setConstructorFieldInitBuilder(ConstructorFieldInitBuilder builder) {
        this.constructorFieldInitBuilder = builder;
    }
    /**
     * 
     * @param builder to generate static field initialize in static constructor 
     */
    public void setStaticConstructorFieldInitBuilder(StaticConstructorFieldInitBuilder builder) {
        this.staticFieldInitBuilder = builder;
    }
    /**
     * 
     * @return the current method info that will be visit 
     */
    public MethodInfo getMethodInfo() {
        if (this.currentMethod == null) {
            this.currentMethod = this.env.popMethodInfo();
        }
        return this.currentMethod;
    }
    /**
     * remove the current visit method
     */
    public void removeMethodInfo() {
        this.currentMethod = null;
    }
    /**
     * create the "this" type with generics
     */
    public void createThis() {
        this.typeOfThis = getTypeUtils().createThis(this.env.getClassFile().getThis().getClassName());
    }
    /**
     * 
     * @return the current type of this 
     */
    public Type.ClassType getThis() {
        return this.typeOfThis;
    }
    
    public void setThis(Type.ClassType override) {
        this.typeOfThis = override;
    }
    /**
     * save the current local table 
     * @param table 
     */
    public void setLocalTable(LocalTable table) {
        this.localTable = table;
    }
    /**
     * 
     * @return the current local table
     */
    public LocalTable getLocalTable() {
        return this.localTable;
    }

    public SymbolFacade getSymbolFacade() {
        return this.env.getSymbolFacade();
    }
    /**
     * Set state machine to init state
     */
    public void initState(AbstractState state) {
        this.state = state;
    }
    /**
     * change state to AssertState
     * @param state
     * @param node
     * @throws CompilerException 
     */
    public void changeState(AssertState state, SyntaxNode node) throws CompilerException {
        this.state = this.state.changeState(state, node);
    }
    /**
     * change state to BlockState
     * @param state
     * @param node
     * @throws CompilerException 
     */
    public void changeState(BlockState state, SyntaxNode node) throws CompilerException {
        this.state = this.state.changeState(state, node);
    }
    /**
     * Possible start state
     * @param state
     * @param node
     * @throws CompilerException 
     */
    public void changeState(ConstructorState state, SyntaxNode node) throws CompilerException {
        this.state = this.state.changeState(state, node);
    }
    /**
     * change state to DoState
     * @param state
     * @param node
     * @throws CompilerException 
     */
    public void changeState(DoState state, SyntaxNode node) throws CompilerException {
        this.state = this.state.changeState(state, node);
    }
     /**
     * @deprecated
     * @param state
     * @param node
     * @throws CompilerException 
     */
    public void changeState(IdentDoublePointState state, SyntaxNode node) throws CompilerException {
        this.state = this.state.changeState(state, node);
    }
     /**
     * change state to IfState
     * @param state
     * @param node
     * @throws CompilerException 
     */
    public void changeState(IfState state, SyntaxNode node) throws CompilerException {
        this.state = this.state.changeState(state, node);
    }
     /**
     * change state to StatementExpressionState
     * @param state
     * @param node
     * @throws CompilerException 
     */
    public void changeState(StatementExpressionState state, SyntaxNode node) throws CompilerException {
        this.state = this.state.changeState(state, node);
    }
     /**
     * Possible start state
     * @param state
     * @param node
     * @throws CompilerException 
     */
    public void changeState(VoidMethodState state, SyntaxNode node) throws CompilerException {
        this.state = this.state.changeState(state, node);
    }
     /**
     * change state to WhileState
     * @param state
     * @param node
     * @throws CompilerException 
     */
    public void changeState(WhileState state, SyntaxNode node) throws CompilerException {
        this.state = this.state.changeState(state, node);
    }
     /**
     * change state to ForState
     * @param state
     * @param node
     * @throws CompilerException 
     */
    public void changeState(ForState state, SyntaxNode node) throws CompilerException {
        this.state = this.state.changeState(state, node);
    }
     /**
     * change state to BreakState. Is a end state
     * @param state
     * @param node
     * @throws CompilerException 
     */
    public void changeState(BreakState state, SyntaxNode node) throws CompilerException {
        this.state = this.state.changeState(state, node);
    }
     /**
     * change state to ContinueState. Is a end state
     * @param state
     * @param node
     * @throws CompilerException 
     */
    public void changeState(ContinueState state, SyntaxNode node) throws CompilerException {
        this.state = this.state.changeState(state, node);
    }
     /**
     * change state to ReturnState. Is a end state
     * @param state
     * @param node
     * @throws CompilerException 
     */
    public void changeState(ReturnState state, SyntaxNode node) throws CompilerException {
        this.state = this.state.changeState(state, node);
    }
     /**
     * change state to ThrowState. Is a end state
     * @param state
     * @param node
     * @throws CompilerException 
     */
    public void changeState(ThrowState state, SyntaxNode node) throws CompilerException {
        this.state = this.state.changeState(state, node);
    }
     /**
     * change state to SynchronizedState. 
     * @param state
     * @param node
     * @throws CompilerException 
     */
    public void changeState(SynchronizedState state, SyntaxNode node) throws CompilerException {
        this.state = this.state.changeState(state, node);
    }
     /**
     * change state to TryState. 
     * @param state
     * @param node
     * @throws CompilerException 
     */
    public void changeState(TryState state, SyntaxNode node) throws CompilerException {
        this.state = this.state.changeState(state, node);
    }
    /**
     * change state to TypeswitchState. 
     * @param state
     * @param node
     * @throws CompilerException 
     */
    public void changeState(TypeSwitchState state, SyntaxNode node) throws CompilerException {
        this.state = this.state.changeState(state, node);
    }
    /**
     * change state to LocalVariableDeclarationState. 
     * @param state
     * @param node
     * @throws CompilerException 
     */
    public void changeState(LocalVariableDeclarationState state, SyntaxNode node) throws CompilerException {
        this.state = this.state.changeState(state, node);
    }
    /**
     * Possible start state. 
     * @param state
     * @param node
     * @throws CompilerException 
     */
    public void changeState(MethodState state, SyntaxNode node) throws CompilerException {
        this.state = this.state.changeState(state, node);
    }
    /**
     * change state to UnreachableCodeState. 
     * After this call, it is impossible to change the state 
     * @param state
     * @param node
     * @throws CompilerException 
     */
    public void changeState(UnreachableCodeState state, SyntaxNode node) throws CompilerException {
        this.state = this.state.changeState(state, node);
    }
    /**
     * change the state to previous state
     */
    public void prevState() {
        this.state = this.state.getParent();
    }
    /**
     * write field expression to constructors
     * @param node
     * @throws CompilerException 
     */
    public void writeFields(SyntaxNode node) throws CompilerException {
        if (hasState(ConstructorState.class) != null) {
            if (this.constructorFieldInitBuilder != null) {
                this.constructorFieldInitBuilder.writeToInstructionList(this);
            }
        }
        if (hasState(StaticConstructorState.class) != null) {
            if (this.staticFieldInitBuilder != null) {
                this.staticFieldInitBuilder.writeToInstructionList(this);
            }
        }
    }
    /**
     * Write the standard super constructor call: this.super = invokedynamic BaseType();
     */
    public void writeSuperConstructorCall(SyntaxNode node) throws CompilerException {
        //gate for super constructor call: Because the Arguments in "super constructor call" calls this method  
        if(this.isInSuperCall){
            return;
        }
        try {
            AttributeInfo attribute = this.env.getClassFile().getAttribute(TeaJayAttribute.NAME);
            if (attribute == null) {
                throw new Error("Fatal error: TeaJay attribute has not set in class file " + this.env.getClassFile().getThis());
            }
            TeaJayAttribute tjAttr = (TeaJayAttribute) attribute;
            JVMInstructionList instructionList = this.getMethodInfo().getCode().getInstructionList();
            ConstructorState constructorState = hasState(ConstructorState.class);
            //is constructor state
            if (constructorState != null) {
                if (!constructorState.hasSuper()) {
                    if (tjAttr.getType() == TeaJayAttribute.Type.Impl) {
                        ClassFile typeDef = ClassFinder.instance().find(this.env.getClassFile().getSuper().getClassName()).get();
                        MethodInfo existMethod = MethodResolver.existMethod(env.getClassFile(), typeDef.getSuper().getClassName(), "<init>", new String[0]);
                        if (existMethod == null) {
                            throw new CompilerException("Missing super call", node.getSymbol());
                        }
                        if (existMethod.isProtected()) {
                            throw new CompilerException("No such constructor " + existMethod, node.getSymbol());
                        }

                        FieldInfo superField = new FieldInfo("super", FieldDescriptor.fromClassInfo(typeDef.getSuper()), FieldAccessFlag.SYNTHETIC, FieldAccessFlag.PRIVATE);
                        if (this.env.getClassFile().getField("super") == null) {
                            Type.ClassSigType classSignature = TypeUtils.getClassSignature(typeDef.getThis().getClassName());
                            superField.addAttribute(new Signature(SignatureEncoder.encode(classSignature.superclassType)));
                            this.env.getClassFile().addField(superField);
                        }

                        FieldRefInfo thisField = FieldRefInfo.make(this.env.getClassFile().getThis(), "this", FieldDescriptor.fromClassFile(typeDef));
                        instructionList.add(new ALoad(0));
                        if (!typeDef.getSuper().getClassName().equals("java/lang/Object")) {
                            instructionList.add(new ALoad(0));
                            instructionList.add(new GetField(thisField));
                        }

                        FieldDescriptor superClass = FieldDescriptor.fromClassInfo(typeDef.getSuper());
                        if (!typeDef.getSuper().getClassName().equals("java/lang/Object")) {
                            invokeDynamic(currentMethod, typeDef.getSuper(), superClass);
                        } else {
                            invokeDynamic(currentMethod, typeDef.getSuper());
                        }


                        instructionList.add(new PutField(superField.getFieldRefInfo(this.env.getClassFile().getThis())));
                        constructorState.setHasSuper();
                    } else if (tjAttr.getType() == TeaJayAttribute.Type.Class) {
                        MethodRefInfo ref = MethodRefInfo.make(this.env.getClassFile().getSuper(), "<init>", FieldDescriptor.getVoid());
                        MethodInfo existMethod = MethodResolver.existMethod(this.env.getClassFile(), this.env.getClassFile().getSuper().getClassName(), "<init>", new String[0]);
                        if (existMethod == null) {
                            throw new CompilerException("Missing super call", node.getSymbol());
                        }
                        if (existMethod.isPrivate()) {
                            throw new CompilerException("Super constructor is private", node.getSymbol());
                        }
                        instructionList.add(new ALoad(0));
                        instructionList.add(new InvokeSpecial(ref));
                        constructorState.setHasSuper();
                    }


                    writeFields(node);
                }
            }
            //REMOVE
            //instructionList.add(new Return());
        } catch (ByteCodeException | IOException | ResolveException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }

    }
    /**
     * insert invoke dynmaic instruction to instruction list
     * @param info current method info
     * @param type 
     * @throws CompilerException 
     */
    public void invokeDynamic(MethodInfo info, ClassInfo type) throws CompilerException {
        try {
            JVMInstructionList instructionList = info.getCode().getInstructionList();
            BootstrapMethodEntry addBoostrapMethod = this.bootstrapMethods.addBootstrapMethod(this.newMethod, type);
            NameAndTypeInfo<MethodDescriptor> nInfo = new NameAndTypeInfo<>("teajaydummynameforanotsearcheadmethod", new MethodDescriptor(FieldDescriptor.fromClassInfo(type)));
            InvokeDynamicInfo dynamicInfo = new InvokeDynamicInfo(addBoostrapMethod, nInfo);
            instructionList.add(new InvokeDynamic(dynamicInfo));
        } catch (ByteCodeException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }
    }
    /**
     * exist State in state machine
     * @param clazz 
     * @return the state from the state machine or null if not exist
     */
    public <T extends State> T hasState(Class<T> clazz) {
        State iter = state;
        while (iter != null) {
            if (clazz.isInstance(iter)) {
                return (T) iter;
            }
            iter = iter.getParent();
        }
        return null;
    }

    public void setExpressionEnvironment(ExpressionEnvironment expEnv) {
        this.expressionEnvironment = expEnv;
    }

    public ExpressionEnvironment getExpressionEnvironment() {
        return this.expressionEnvironment;
    }
    /**
     * 
     * @return true if current class file is a typeimplementation 
     */
    public boolean isTeaJayImpl() {
        AttributeInfo attribute = this.env.getClassFile().getAttribute(TeaJayAttribute.NAME);
        if (attribute == null) {
            throw new Error("Fatal error: TeaJay attribute has not set in class file " + this.env.getClassFile().getThis());
        }
        TeaJayAttribute tjAttr = (TeaJayAttribute) attribute;
        
        if (tjAttr.getType() == TeaJayAttribute.Type.Impl) {
            return true;
        }
        return false;
    }

    public void setExpectedResultType(Type type) {
        this.expectedResultType = type;
    }

    public Type getExpectedResultType() {
        return this.expectedResultType;
    }

    public Type tryToTypeInference(SyntaxNode node) throws CompilerException {
        LocalVariableDeclarationState localVarDecState = hasState(LocalVariableDeclarationState.class);
        if (localVarDecState != null) {
            return localVarDecState.getType();
        } else {
            ReturnState returnState = this.hasState(ReturnState.class);
            if (returnState != null) {
                return this.currentMethodExpectedType;
            }
            return this.expectedResultType;
        }
    }

    public void setCurrentMethodExpectedType(Type type) {
        this.currentMethodExpectedType = type;
    }
    /**
     * 
     * @param node for CompilerException
     * @return class file of the super type of typeimplementation
     * @throws CompilerException 
     */
    public ClassFile getSuper(SyntaxNode node) throws CompilerException {
        try {
            switch (InformationCenter.getTeaJayAttributeType(this.env.getClassFile())) {
                case Closure:
                    throw new CompilerException("Closure has no super class", node.getSymbol());
                case Impl:
                    ClassFile sFile = ClassFinder.instance().find(this.env.getClassFile().getSuper().getClassName()).get();
                    return ClassFinder.instance().find(sFile.getSuper().getClassName()).get();
                case Typedef:
                    throw new Error("Fatal error: existing code for super call in constructor of typedef ");
                case Class:
                    return ClassFinder.instance().find(this.env.getClassFile().getSuper().getClassName()).get();
                case Enum:
                    throw new CompilerException("Closure has no super class", node.getSymbol());
                case Interface:
                    throw new CompilerException("Interface has no super class", node.getSymbol());
                case TJBuiltinType:
                    return ClassFinder.instance().find(this.env.getClassFile().getSuper().getClassName()).get();
                case Hidden:
                    return ClassFinder.instance().find(this.env.getClassFile().getSuper().getClassName()).get();
                default:
                    throw new AssertionError(InformationCenter.getTeaJayAttributeType(this.env.getClassFile()).name());

            }

        } catch (ByteCodeException | IOException | ResolveException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }
        return null;
    }
    /**
     * 
     * @param info current method
     * @param type 
     * @param callParameters 
     */
    public void invokeDynamic(MethodInfo info, ClassInfo type, FieldDescriptor... callParameters) {
        try {
            JVMInstructionList instructionList = info.getCode().getInstructionList();
            BootstrapMethodEntry addBoostrapMethod = this.bootstrapMethods.addBootstrapMethod(this.newMethod, type);
            NameAndTypeInfo<MethodDescriptor> nInfo = new NameAndTypeInfo<>("withParam", new MethodDescriptor(FieldDescriptor.fromClassInfo(type), callParameters));
            InvokeDynamicInfo dynamicInfo = new InvokeDynamicInfo(addBoostrapMethod, nInfo);
            instructionList.add(new InvokeDynamic(dynamicInfo));
        } catch (ByteCodeException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }
    }
    /**
     * 
     * @param info current method
     * @param refInfo descriptor of a Method 
     * @param type
     */
    public void invokeDynamic(MethodInfo info, MethodDescriptor refInfo, ClassInfo type) {
        try {
            JVMInstructionList instructionList = info.getCode().getInstructionList();
            BootstrapMethodEntry addBoostrapMethod = this.bootstrapMethods.addBootstrapMethod(this.newMethod, type);
            NameAndTypeInfo<MethodDescriptor> nInfo = new NameAndTypeInfo<>("withParam", refInfo);
            InvokeDynamicInfo dynamicInfo = new InvokeDynamicInfo(addBoostrapMethod, nInfo);
            instructionList.add(new InvokeDynamic(dynamicInfo));
            System.out.println(new InvokeDynamic(dynamicInfo));
        } catch (ByteCodeException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }
    }

    public void addGenericFrame() {
        genericEnv.addLast(new HashSet<String>());
    }

    public void dropGenericFrame() {
        genericEnv.removeLast();
    }

    public void addTypeVariable(String identifier) {
        genericEnv.getLast().add(identifier);
    }

    public boolean isTypeVariable(String identifier) {
        Iterator<HashSet<String>> descendingIterator = genericEnv.descendingIterator();
        while (descendingIterator.hasNext()) {
            if (descendingIterator.next().contains(identifier)) {
                return true;
            }
        }
        return false;
    }
    /**
     * 
     * @return the all type variables 
     */
    public LinkedList<HashSet<String>> getTypeVariables() {
        return genericEnv;
    }
    /**
     * Overwrite all type variables
     * @param vars types
     */
    public void setTypeVariables(LinkedList<HashSet<String>> vars) {
        this.genericEnv = vars;
    }

    public Stage1Environment getStage1() {
        return env.getStage1Env();
    }

    public Stage2Environment getStage2() {
        return env;
    }
    /**
     * Creates a unique closure name
     * @return closure name 
     */
    public String createClosureName() {
        String first = this.env.getClassFile().getThis().getClassName();
        first = first + "$Closure" + closureCnt++;
        return first;
    }
    private Deque<ExpressionEnvironment> tempExpressionStack = new LinkedList<>();

    /**
     * create a new expression environment. Save the old expression environment if exists.
     */
    public void createNewExpressionEnvironment() {
        tempExpressionStack.push(this.expressionEnvironment);
        this.expressionEnvironment = new ExpressionEnvironment(this);
    }
    /**
     * restore the old expression environment if exists
     */
    public void restoreOldExpressionEnvironment() {
        if (!tempExpressionStack.isEmpty()) {
            ShortCircuitEnvironment shortCircuitEnvironment = this.expressionEnvironment.getShortCircuit();
            this.expressionEnvironment = tempExpressionStack.pop();
            this.expressionEnvironment.setShortCircuit(shortCircuitEnvironment);
   
        }
    }
    /**
     * create a new expression environment and copy the prefix stack from old expression environment
     * to new environment
     * @param withPrefix 
     */
    public void createNewExpressionEnvironment(boolean withPrefix) {
        tempExpressionStack.push(this.expressionEnvironment);
        this.expressionEnvironment = new ExpressionEnvironment(this, withPrefix);
    }

    public String getSynchronizedVarName() {
        this.indexOfSyncName++;
        return "$sync" + indexOfSyncName + "$$";
    }

    public void setIsInSuperConstructorCall() {
        this.isInSuperCall = true;
    }
    public void resetIsInSuperConstructorCall() {
        this.isInSuperCall = false;
    }
}
