/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package codegeneration.javavisitor;

import codegeneration.javavisitor.method.MethodeOrFieldDeclVisitor;
import codegeneration.javavisitor.method.VoidMethodeDeclaratorRestHelperVisitor;
import codegeneration.javavisitor.method.ConstructorDeclaratorRestHelperVisitor;
import abstractsyntaxtree.ClassDeclarationNode;
import abstractsyntaxtree.ConstructorDeclaratorRestHelperNode;
import abstractsyntaxtree.GenericMethodeOrConstructorDeclNode;
import abstractsyntaxtree.InterfaceDeclarationNode;
import abstractsyntaxtree.MethodeOrFieldDeclNode;
import abstractsyntaxtree.ModifierNode;
import abstractsyntaxtree.ModifierNode.Modifier;
import abstractsyntaxtree.VoidMethodeDeclaratorRestHelperNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractConstructorDeclaratorRestHelperVisitor;
import codegeneration.abstractvisitors.AbstractMemberDeclVisitor;
import codegeneration.abstractvisitors.AbstractVoidMethodeDeclaratorRestHelperVisitor;
import exceptions.CompilerException;
import java.util.List;
import lexer.Symbol;
import teajay.bytecode.ClassFile;

/**
 *
 * @author cefour
 */
public class MemberDeclVisitor extends AbstractMemberDeclVisitor {

    private final ClassFile classFile;
    private MethodeOrFieldDeclVisitor methodeOrFieldDeclVisitor;
    private final List<ModifierNode> modis;
    private AbstractVoidMethodeDeclaratorRestHelperVisitor voidMethodeDeclaratorRestHelperVisitor;
    private AbstractConstructorDeclaratorRestHelperVisitor constructorDeclaratorRestHelperVisitor;

    public MemberDeclVisitor(ClassFile classFile, List<ModifierNode> modis) {
        this.classFile = classFile;
        this.modis = modis;
    }

    @Override
    public void begin(MethodeOrFieldDeclNode node) {
        if (modis != null) {

            this.methodeOrFieldDeclVisitor = new MethodeOrFieldDeclVisitor(classFile, modis, node.getIdentifier());
        } else {
            this.methodeOrFieldDeclVisitor = new MethodeOrFieldDeclVisitor(classFile, node.getIdentifier());
        }
    }

    @Override
    public void begin(VoidMethodeDeclaratorRestHelperNode node) {
        if (modis != null) {
            this.voidMethodeDeclaratorRestHelperVisitor = new VoidMethodeDeclaratorRestHelperVisitor(classFile, modis, node.getMethodName());
        } else {
            this.voidMethodeDeclaratorRestHelperVisitor = new VoidMethodeDeclaratorRestHelperVisitor(classFile, node.getMethodName());
        }
    }

    @Override
    public void begin(ConstructorDeclaratorRestHelperNode node) throws CompilerException {
        if (modis != null) {
            this.constructorDeclaratorRestHelperVisitor = new ConstructorDeclaratorRestHelperVisitor(classFile, modis, node.getIdentifier());
        } else {
            this.constructorDeclaratorRestHelperVisitor = new ConstructorDeclaratorRestHelperVisitor(classFile, node.getIdentifier());
        }
    }

    @Override
    public void begin(GenericMethodeOrConstructorDeclNode node) {
    }

    @Override
    public void begin(ClassDeclarationNode node) {
    }

    @Override
    public void begin(InterfaceDeclarationNode node) {
    }

    @Override
    public Visitor visit(MethodeOrFieldDeclNode node) {
        return this.methodeOrFieldDeclVisitor;
    }

    @Override
    public Visitor visit(VoidMethodeDeclaratorRestHelperNode node) {
        return this.voidMethodeDeclaratorRestHelperVisitor;
    }

    @Override
    public Visitor visit(ConstructorDeclaratorRestHelperNode node) {
        return this.constructorDeclaratorRestHelperVisitor;
    }

    @Override
    public Visitor visit(GenericMethodeOrConstructorDeclNode node) {
        return null;
    }

    @Override
    public Visitor visit(ClassDeclarationNode node) {
        return null;
    }

    @Override
    public Visitor visit(InterfaceDeclarationNode node) {
        return null;
    }

    @Override
    public void end(MethodeOrFieldDeclNode node) {
    }

    @Override
    public void end(VoidMethodeDeclaratorRestHelperNode node) {
    }

    @Override
    public void end(ConstructorDeclaratorRestHelperNode node) {
    }

    @Override
    public void end(GenericMethodeOrConstructorDeclNode node) {
    }

    @Override
    public void end(ClassDeclarationNode node) {
    }

    @Override
    public void end(InterfaceDeclarationNode node) {
    }
}
