/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package codegeneration.javavisitor.classes;

import codegeneration.javavisitor.classes.ClassBodyVisitor;
import abstractsyntaxtree.ClassBodyNode;
import abstractsyntaxtree.TypeListNode;
import abstractsyntaxtree.TypeNode;
import abstractsyntaxtree.TypeParametersNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractClassBodyVisitor;
import codegeneration.abstractvisitors.AbstractNormalClassDeclarationVisitor;
import codegeneration.abstractvisitors.AbstractTypeVisitor;
import codegeneration.javavisitor.TypeListVisitor;
import codegeneration.javavisitor.TypeVisitor;
import exceptions.CompilerException;
import java.io.IOException;
import java.util.List;
import symbol.MethodResolver;
import teajay.bytecode.*;
import teajay.bytecode.cpool.ClassInfo;
import teajay.bytecode.cpool.FieldDescriptor;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.bytecode.exceptions.MalformedClassNameException;
import teajay.bytecode.methods.MethodInfo;
import teajay.runtime.exceptions.ResolveException;
import teajay.util.io.ClassFileRef;
import teajay.util.io.ClassFinder;

/**
 *
 * @author cefour
 */
public class NormalClassDeclarationVisitor extends AbstractNormalClassDeclarationVisitor {

    private ClassFile classFile;
    private AbstractTypeVisitor typeVisitor;
    private AbstractClassBodyVisitor classBodyVisitor;
    private TypeListVisitor typeListVisitor;

    public NormalClassDeclarationVisitor(ClassFile classFile) {
        this.classFile = classFile;
    }

    @Override
    public void begin(TypeParametersNode node) throws CompilerException {
    }

    @Override
    public Visitor visit(TypeParametersNode node) throws CompilerException {
        return null;
    }

    @Override
    public void end(TypeParametersNode node) throws CompilerException {
    }

    @Override
    public void begin(TypeNode node) throws CompilerException {
        try {
            if (!node.isReferenceType()) {
                throw new CompilerException("It is not possible to extends from " + "basic types");
            }
            this.typeVisitor = new TypeVisitor();
        } catch (CompilerException ex) {
            System.err.println(ex.getMessage());
            System.exit(-1);
        }
    }

    @Override
    public Visitor visit(TypeNode node) throws CompilerException {
        return this.typeVisitor;
    }

    @Override
    public void end(TypeNode node) throws CompilerException {
        try {
            String extendsFrom = this.typeVisitor.getTypeName();
            this.classFile.setSuper(extendsFrom);
        } catch (ByteCodeException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }
    }

    @Override
    public void begin(TypeListNode node) throws CompilerException {
        this.typeListVisitor = new TypeListVisitor();
    }

    @Override
    public Visitor visit(TypeListNode node) throws CompilerException {
        return this.typeListVisitor;
    }

    @Override
    public void end(TypeListNode node) throws CompilerException {
        //  if (CompileManager.instance().getStage() == 1) {
        for (String iface : typeListVisitor.getContainer()) {
            try {
                classFile.addInterface(new ClassInfo(iface));
                //TODO add source files
            } catch (MalformedClassNameException ex) {
                ex.printStackTrace(System.err);
                System.exit(-1);
            }
        }
        //} else if (CompileManager.instance().getStage() == 2) {
        System.err.println("iface: " + classFile.getInterfaces());
        for (ClassInfo info : classFile.getInterfaces()) {
            ClassFileRef ref = ClassFinder.instance().find(info.getClassName());
            if (ref != null) {
                try {
                    ClassFile cl = ref.get();
                    for (List<MethodInfo> methods : cl.getMethods()) {
                        for (MethodInfo minfo : methods) {
                       //     if (MethodResolver.existMethod(classFile.getThis().getClassName(), minfo.getName().getValue(), minfo.getDescriptor().getParameters().toArray(new FieldDescriptor[0])) == null) {
                         //       throw new Error(classFile.getThis() + " does not implement interface method " + minfo.getName());
                           // }
                        }
                    }
                } catch (ByteCodeException | IOException | ResolveException ex) {
                    ex.printStackTrace(System.err);
                    System.exit(-1);
                }
            } else {
                //throw new CompilerException("");
                //Kann keine CompilerException werfen da die super methode dies nicht annotiert!
                throw new Error("Class not found: " + info.getClassName());
            }
        }
        //}
    }

    @Override
    public void begin(ClassBodyNode node) throws CompilerException {
    }

    @Override
    public Visitor visit(ClassBodyNode node) throws CompilerException {
        return this.classBodyVisitor = new ClassBodyVisitor(classFile);
    }

    @Override
    public void end(ClassBodyNode node) throws CompilerException {
    }
}
