/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package codegeneration.javavisitor.method;

import abstractsyntaxtree.ArgumentsNode;
import abstractsyntaxtree.BasicTypeNode;
import abstractsyntaxtree.BlockNode;
import abstractsyntaxtree.ClosureCreatorNode;
import abstractsyntaxtree.CreatorNode;
import abstractsyntaxtree.ExplicitGenericInvocationSuffixNode;
import abstractsyntaxtree.ExpressionNode;
import abstractsyntaxtree.FormalParametersNode;
import abstractsyntaxtree.IdentifierSuffixNode;
import abstractsyntaxtree.LiteralNode;
import abstractsyntaxtree.ModifierNode;
import abstractsyntaxtree.NonWildcardTypeArgumentsNode;
import abstractsyntaxtree.ParExpressionNode;
import abstractsyntaxtree.QualifiedIdentifierListNode;
import abstractsyntaxtree.SuperSuffixNode;
import abstractsyntaxtree.TypeNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractFormalParametersVisitor;
import codegeneration.abstractvisitors.AbstractQualifiedIdentifierListVisitor;
import codegeneration.abstractvisitors.AbstractVoidMethodeDeclaratorRestVisitor;
import codegeneration.abstractvisitors.AbstractPrimaryVisitor;
import codegeneration.javavisitor.BlockVisitor;
import codegeneration.javavisitor.QualifiedIdentifierListVisitor;
import datastructs.Pair;
import exceptions.CompilerException;
import java.util.LinkedList;
import java.util.List;
import resolve.SymbolTable;
import symbol.LocalTable;
import symbol.SymbolFacade;
import teajay.bytecode.ClassFile;
import teajay.bytecode.attributes.Exceptions;
import teajay.bytecode.attributes.code.Return;
import teajay.bytecode.cpool.ClassInfo;
import teajay.bytecode.cpool.FieldDescriptor;
import teajay.bytecode.cpool.MethodDescriptor;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.bytecode.methods.MethodAccessFlag;
import teajay.bytecode.methods.MethodInfo;
import teajay.generics.check.TypeUtils;
import teajay.generics.parser.Type;

/**
 *
 * @author cefour
 */
public class VoidMethodeDeclaratorRestVisitor extends AbstractVoidMethodeDeclaratorRestVisitor {

    private final String methodName;
    private List<ModifierNode> modis;
    private final ClassFile classFile;
    private AbstractFormalParametersVisitor<List<Pair<FieldDescriptor, String>>> formalParametersVisitor;
    private AbstractQualifiedIdentifierListVisitor qualifiedIdentifierListVisitor;
    private MethodInfo info;

    VoidMethodeDeclaratorRestVisitor(ClassFile classFile, List<ModifierNode> modis, String methodName) {
        this.classFile = classFile;
        this.modis = modis;
        this.methodName = methodName;
    }

    VoidMethodeDeclaratorRestVisitor(ClassFile classFile, String methodName) {
        this.classFile = classFile;
        this.methodName = methodName;
    }

    @Override
    public void begin(FormalParametersNode node) throws CompilerException {
    }

    @Override
    public void begin(QualifiedIdentifierListNode node) throws CompilerException {
    }

    @Override
    public void begin(BlockNode node) throws CompilerException {
        System.out.println("Begin " + info.getName());
        node.accept(new MethodCallVisitor());

    }

    @Override
    public Visitor visit(FormalParametersNode node) throws CompilerException {
        return this.formalParametersVisitor = new FormalParametersVisitor();
    }

    @Override
    /**
     * Visit the throw list of the method declaration
     *
     * @return QualifiedIdentifierListVisitor
     */
    public Visitor visit(QualifiedIdentifierListNode node) throws CompilerException {
        return this.qualifiedIdentifierListVisitor = new QualifiedIdentifierListVisitor();
    }

    @Override
    public Visitor visit(BlockNode node) throws CompilerException {

        return new BlockVisitor(this.classFile, this.info);
    }

    @Override
    public void end(FormalParametersNode node) throws CompilerException {
        try {

            MethodAccessFlag[] flags = this.getMethodAccessFlags();
            List<Pair<FieldDescriptor, String>> list = this.formalParametersVisitor.getFieldContainer();
            List<FieldDescriptor> fList = new LinkedList<>();

            for (Pair<FieldDescriptor, String> p : list) {
                fList.add(p.value1);
            }


            MethodDescriptor desc = new MethodDescriptor(FieldDescriptor.getVoid(), fList);


            this.info = new MethodInfo(this.methodName, desc,
                    this.classFile.getThis(), flags);
            /*
             * Set parameter in local table. this variable is automatic set by constructor
             * if and only if the method is static
             */
            LocalTable table = new LocalTable(info);
            for (Pair<FieldDescriptor, String> p : list) {
                table.addEntry(p.value2, new Type.ClassType(null,p.value1.getClassInfo().getClassName(),null));
            }
          //  symbol.SymbolFacade.instance().setLocalTable(table);
            this.classFile.addMethod(this.info);
        } catch (ByteCodeException | CompilerException ex) {
            System.err.println(ex.getMessage());
            System.exit(-1);
        }
    }

    @Override
    /**
     * Method generate throws list on the method declaration.
     */
    public void end(QualifiedIdentifierListNode node) throws CompilerException {
        List<String> qIdList = this.qualifiedIdentifierListVisitor.getQualifiedIdentifiers();
            Exceptions ex = new Exceptions();
            for (String q : qIdList) {
          //      ex.addException(new ClassInfo(SymbolFacade.instance().getFullQualifiedId(q)));
            }
            info.addAttribute(ex);
        
    }

    @Override
    public void end(BlockNode node) throws CompilerException {
        try {
            this.info.getCode().getInstructionList().add(new Return());
        } catch (ByteCodeException ex) {
            System.err.println(ex.getMessage());
            System.exit(-1);
        }
        System.out.println("End " + info.getName());
    }

    /**
     * Method checks the access flags of correctnes and uniquenes.
     *
     * @return Array of MethodAccessFlags
     * @throws CompilerException
     */
    private MethodAccessFlag[] getMethodAccessFlags() throws CompilerException {
        MethodAccessFlag[] flags = new MethodAccessFlag[modis.size()];
        boolean[] uniqueAccess = {false, false, false,
            false, false, false,
            false, false, false};
        int i = 0;
        for (ModifierNode mod : modis) {
            switch (mod.getModifier()) {
                case ABSTRACT:
                    if (!uniqueAccess[0]) {
                        flags[i] = MethodAccessFlag.ABSTRACT;
                        uniqueAccess[0] = true;
                    } else {
                        throw new CompilerException("Dublicate modifier at " + methodName, mod.getSymbol());
                    }
                    break;
                case FINAL:
                    if (!uniqueAccess[1]) {
                        flags[i] = MethodAccessFlag.FINAL;
                        uniqueAccess[1] = true;
                    } else {
                        throw new CompilerException("Dublicate modifier at " + methodName, mod.getSymbol());
                    }
                    break;
                case STATIC:
                    if (!uniqueAccess[2]) {
                        flags[i] = MethodAccessFlag.STATIC;
                        uniqueAccess[2] = true;
                    } else {
                        throw new CompilerException("Dublicate modifier at " + methodName, mod.getSymbol());
                    }
                    break;
                case SYNCHRONIZED:
                    if (!uniqueAccess[3]) {
                        flags[i] = MethodAccessFlag.SYNCHRONIZED;
                        uniqueAccess[3] = true;
                    } else {
                        throw new CompilerException("Dublicate modifier at " + methodName, mod.getSymbol());
                    }
                    break;
                case PUBLIC:
                    if (!uniqueAccess[4]) {
                        flags[i] = MethodAccessFlag.PUBLIC;
                        uniqueAccess[4] = true;
                    } else {
                        throw new CompilerException("Dublicate modifier at " + methodName, mod.getSymbol());
                    }
                    break;
                case PROTECTED:
                    if (!uniqueAccess[5]) {
                        flags[i] = MethodAccessFlag.PROTECTED;
                        uniqueAccess[5] = true;
                    } else {
                        throw new CompilerException("Dublicate modifier at " + methodName, mod.getSymbol());
                    }
                    break;
                case PRIVATE:
                    if (!uniqueAccess[6]) {
                        flags[i] = MethodAccessFlag.PRIVATE;
                        uniqueAccess[6] = true;
                    } else {
                        throw new CompilerException("Dublicate modifier at " + methodName, mod.getSymbol());
                    }
                    break;
                case NATIVE:
                    if (!uniqueAccess[7]) {
                        flags[i] = MethodAccessFlag.NATIVE;
                        uniqueAccess[7] = true;
                    } else {
                        throw new CompilerException("Dublicate modifier at " + methodName, mod.getSymbol());
                    }
                    break;
                case STRICTFP:
                    if (!uniqueAccess[8]) {
                        flags[i] = MethodAccessFlag.STRICT;
                        uniqueAccess[8] = true;
                    } else {
                        throw new CompilerException("Dublicate modifier at " + methodName, mod.getSymbol());
                    }
                    break;
                default:
            }
            i++;
        }
        return flags;
    }
}

class MethodCallVisitor extends AbstractPrimaryVisitor {

    @Override
    public void begin(LiteralNode node) {
    }

    @Override
    public void begin(TypeNode node) {
    }

    @Override
    public void begin(ParExpressionNode node) {
    }

    @Override
    public void begin(ArgumentsNode node) {
    }

    @Override
    public void begin(SuperSuffixNode node) {
    }

    @Override
    public void begin(CreatorNode node) {
    }

    @Override
    public void begin(NonWildcardTypeArgumentsNode node) {
    }

    @Override
    public void begin(ExplicitGenericInvocationSuffixNode node) {
    }

    @Override
    public void begin(IdentifierSuffixNode node) {
    }

    @Override
    public void begin(BasicTypeNode node) {
    }

    @Override
    public void begin(ClosureCreatorNode node) {
    }

    @Override
    public void visit(LiteralNode node) {
    }

    @Override
    public Visitor visit(TypeNode node) {
        return null;
    }

    @Override
    public Visitor visit(ParExpressionNode node) {
        return null;
    }

    @Override
    public Visitor visit(ArgumentsNode node) {
        return null;
    }

    @Override
    public Visitor visit(SuperSuffixNode node) {
        return null;
    }

    @Override
    public Visitor visit(CreatorNode node) {
        return null;
    }

    @Override
    public Visitor visit(NonWildcardTypeArgumentsNode node) {
        return null;
    }

    @Override
    public Visitor visit(ExplicitGenericInvocationSuffixNode node) {
        return null;
    }

    @Override
    public Visitor visit(IdentifierSuffixNode node) throws CompilerException {
        node.accept(new ArgumentsVisitorLiteral());
        return this;
    }

    @Override
    public void visit(BasicTypeNode node) {
    }

    @Override
    public Visitor visit(ClosureCreatorNode node) {
        return null;
    }

    @Override
    public void end(LiteralNode node) {
    }

    @Override
    public void end(TypeNode node) {
    }

    @Override
    public void end(ParExpressionNode node) {
    }

    @Override
    public void end(ArgumentsNode node) {
    }

    @Override
    public void end(SuperSuffixNode node) {
    }

    @Override
    public void end(CreatorNode node) {
    }

    @Override
    public void end(NonWildcardTypeArgumentsNode node) {
    }

    @Override
    public void end(ExplicitGenericInvocationSuffixNode node) {
    }

    @Override
    public void end(IdentifierSuffixNode node) {
    }

    @Override
    public void end(BasicTypeNode node) {
    }

    @Override
    public void end(ClosureCreatorNode node) {
    }

    @Override
    public void begin(ExpressionNode node) {
    }

    @Override
    public Visitor visit(ExpressionNode node) {
        return null;
    }

    @Override
    public void end(ExpressionNode node) {
    }
}

class ArgumentsVisitorLiteral extends Visitor {

    public void begin(LiteralNode node) {
    }

    public void end(LiteralNode node) {
    }

    public void visit(LiteralNode node) {
    }
}