/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package minicompiler;

import abstractsyntaxtree.CompilationUnitHelperNode;
import abstractsyntaxtree.EnumDeclarationNode;
import abstractsyntaxtree.NormalInterfaceDeclarationNode;
import abstractsyntaxtree.TjImplDeclarationNode;
import abstractsyntaxtree.TjTypeDefNode;
import codegeneration.Visitor;
import codegeneration.javavisitor.compilationunit.CompilationUnitHelperVisitor;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public class PlainTypenameVisitor extends Visitor {
    private CompilationUnitHelperVisitor packageVisitor;
    private String name = null;
    
    @Override
    public void begin(TjTypeDefNode node) {
        name = node.getIdentifier();
    }
    
    @Override
    public Visitor visit(CompilationUnitHelperNode node) {
        return packageVisitor = new CompilationUnitHelperVisitor();
    }
    
    @Override
    public void begin(TjImplDeclarationNode node) {
        name = node.getImplName();
    }
    
    @Override
    public void begin(NormalInterfaceDeclarationNode node) {
        name = node.getInterfaceName();
    }
    
    @Override
    public void begin(EnumDeclarationNode node) {
        name = node.getName();
    }
    
    @Override
    public Visitor visit(TjTypeDefNode node) {
        return null;
    }
    
    @Override
    public Visitor visit(TjImplDeclarationNode node) {
        return null;
    }
    
    @Override
    public Visitor visit(NormalInterfaceDeclarationNode node) {
        return null;
    }
    
    @Override
    public Visitor visit(EnumDeclarationNode node) {
        return null;
    }
    
    public String getQID() {
        if (packageVisitor == null || packageVisitor.getPackageName() == null || packageVisitor.getPackageName().equals("")) {
            return name;
        }
        if (name != null) {
            return packageVisitor.getPackageName() + "/" + name;
        }
        return null;
    }
}
