/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package minicompiler;

import abstractsyntaxtree.SyntaxNode;
import exceptions.CompilerException;
import java.io.File;
import teajay.util.QIDTools;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public class SourceFileNameChecker {

    public static void checkSourceFile(File file, String qid, SyntaxNode node) throws CompilerException {
        String filename = file.getName();
        if (filename.toLowerCase().endsWith(".tj")) {
            filename = filename.substring(0, filename.length() - 3);
            String simpleName = QIDTools.simpleName(qid);
            if (!filename.equals(simpleName)) {
                throw new CompilerException(simpleName + " must be declared in a file named " + simpleName + ".tj", node.getSymbol());
            }
        } else {
            throw new CompilerException("not a TeaJay source file", node.getSymbol());
        }
    }

    public static void checkSourceFile(String filename, String qid, SyntaxNode node) throws CompilerException {
        checkSourceFile(new File(filename), qid, node);
    }
}
