/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package minicompiler;

import exceptions.CompilerException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import symbol.SymbolFacade;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.runtime.exceptions.ResolveException;
import teajay.util.io.ClassFinder;
import teajay.util.io.Input;

/**
 *
 * @author Michael Burzan
 */
public class WorkingNode {

    private final SymbolFacade facade;
    private final boolean isStaticImport;
    private final Map<String, Input> imports;
    private final String importName;
    private final Input toImport;
    private final String typeName;
    private final Input typeInput;
    private boolean done;

//    private WorkingNode(SymbolFacade facade, String importName, Map<String, Input> imports, boolean isStatic) {
//        this.facade = facade;
//        this.isStaticImport = isStatic;
//        this.imports = imports;
//        this.importName = importName;
//        this.toImport = null;
//        this.typeName = null;
//        this.typeInput = null;
//    }
//
//    private WorkingNode(SymbolFacade facade, String importName, Input toImport, boolean isStatic) {
//        this.facade = facade;
//        this.isStaticImport = isStatic;
//        this.imports = null;
//        this.importName = importName;
//        this.toImport = toImport;
//        this.typeName = null;
//        this.typeInput = null;
//    }
    public WorkingNode(SymbolFacade facade, String typeName, Input typeInput, boolean done) {
        this.typeName = typeName;
        this.typeInput = typeInput;
        this.facade = facade;
        this.isStaticImport = false;
        this.imports = null;
        this.importName = null;
        this.toImport = null;
        this.done = done;
    }

    public WorkingNode(SymbolFacade facade, String typeName, Input typeInput) {
        this(facade, typeName, typeInput, false);
    }

    public Collection<Input> getInputs() {
        if (done) {
            return Collections.emptyList();
        }
        if (typeName != null) {
            //ArrayList<Input> res = new ArrayList<>();
            //if (ClassFinder.instance().find(typeName) == null) {
            //res.add(typeInput);
            //}
            return Arrays.asList(typeInput);
        }
        ArrayList<Input> res = new ArrayList<>();
        if (toImport != null) {
            if (ClassFinder.instance().find(importName) == null) {
                res.add(toImport);

            }
            return res;
        }
        if (imports != null) {
            Set<String> set = imports.keySet();
            for (String s : set) {
                if (ClassFinder.instance().find(importName + "/" + s) == null) {
                    res.add(imports.get(s));
                }
            }
        }
        return res;

    }

    public void setCompileFileToFacade() throws CompilerException, ByteCodeException, IOException, ResolveException {
        /*if (isStaticImport && importName != null) {
         this.facade.addNameFromStaticImport(importName);
         }*/
        if (facade != null) {
            if (typeInput != null) {
                this.facade.addNameFromPackage(typeName);
            }

            if (isStaticImport && imports != null) {
                Set<String> keySet = imports.keySet();
                for (String s : keySet) {
                    this.facade.addNameFromStaticImport(importName + "/" + s);
                }
            }
            /*if (!isStaticImport && importName != null) {
             this.facade.addNameFromImport(importName);
             }*/
            if (!isStaticImport && imports != null) {
                Set<String> keySet = imports.keySet();
                for (String s : keySet) {
                    this.facade.addNameFromImport(importName + "/" + s);
                }
            }
        }
        this.done = true;
    }

    public boolean isDone() {
        return done;
    }

    public String getTypeName() {
        return typeName;
    }

    @Override
    public int hashCode() {
        int hash = 7;
        hash = 89 * hash + Objects.hashCode(this.importName);
        hash = 89 * hash + Objects.hashCode(this.typeName);
        return hash;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final WorkingNode other = (WorkingNode) obj;

        if (!Objects.equals(this.importName, other.importName)) {
            return false;
        }
        if (!Objects.equals(this.typeName, other.typeName)) {
            return false;
        }

        return true;
    }

    @Override
    public String toString() {
        return "WorkingNode{" + "typeName=" + typeName + '}';
    }
}
