/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package parser;

import exceptions.CompilerException;
import lexer.Sym;
import lexer.LexerWrapper;
import lexer.Symbol;
import abstractsyntaxtree.*;
import abstractsyntaxtree.LiteralNode.LiteralType;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
//import java_cup.runtime.Symbol;

/**
 * TODO: Alle { * } RULES müssen überprüft werden da listen wenn sie größer als
 * 1 sind eine Regel erfüllt haben somit muss die nächste Regel auch erfüllt
 * werden
 *
 * @author Michael Burzan
 */
public class Parser {

    private LexerWrapper lex;
    private Symbol lexSymbol;
    private String lastTokenText;

    public Parser(LexerWrapper lex) {
        this.lex = lex;
        SyntaxNode.lexer = this.lex;

    }

    public SyntaxNode parse() throws IOException, CompilerException {
        this.lexSymbol = lex.yylex();

        CompilationUnitNode compilationUnit = this.parseCompilationUnit();
        if (compilationUnit != null) {
            return compilationUnit;
        }
        return null;
    }

    private boolean accept(int symbol) throws IOException, CompilerException {
        Symbol l = this.lexSymbol;
        if (symbol == l.sym) {
            this.lastTokenText = l.value;
            this.lexSymbol = lex.yylex();
            return true;
        }
        return false;

    }

    /**
     * RULE: [Annotations] "package" QualifiedIdentfier ";"
     */
    private CompilationUnitHelperNode parseCompilationUnitHelperRule() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.lex.lookahead(Sym.AT, Sym.INTERFACE)) {
            return null;
        }
        AnnotationsNode annotations = this.parseAnnotations();

        if (this.accept(Sym.PACKAGE)) {
            QualifiedIdentifierNode qualifiedIdentifier = this.parseQualifiedIdentfier();
            if (qualifiedIdentifier != null) {
                if (this.accept(Sym.SEMICOLON)) {
                    //Konsumiere überschüssige Semicolons;
                    if (this.lex.lookahead(Sym.SEMICOLON, Sym.SEMICOLON)) {
                        while (this.accept(Sym.SEMICOLON)) {
                            if (!this.lex.lookahead(Sym.SEMICOLON, Sym.SEMICOLON)) {
                                break;
                            }
                        }
                    }
                    return new CompilationUnitHelperNode(s, annotations, qualifiedIdentifier);
                }
                throw new CompilerException("Expect ; in package definition", this.lexSymbol);
            }
            throw new CompilerException("Missing qualified identifier after package",
                    this.lexSymbol);
        }
        return null;
    }

    /**
     * RULE: [CompilationUnitHelperRule ] { ImportDeclaration } {
     * TypeDeclaration }
     */
    private CompilationUnitNode parseCompilationUnit() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        ArrayList<ImportDeclarationNode> importList = new ArrayList<>();
        ArrayList<TypeDeclarationNode> declList = new ArrayList<>();

        CompilationUnitHelperNode helperNode = this.parseCompilationUnitHelperRule();

        ImportDeclarationNode importNode = this.parseImportDeclaration();

        while (importNode != null) {
            importList.add(importNode);
            importNode = this.parseImportDeclaration();
        }
        /*TeaJay Support*/
        boolean isTypedef = this.lex.lookahead(Sym.TYPEDEF);
        boolean isPublicTypedef = this.lex.lookahead(Sym.PUBLIC, Sym.TYPEDEF);
        boolean isFinalTypedef = this.lex.lookahead(Sym.FINAL, Sym.TYPEDEF);
        boolean isPublicFinalTypedef = this.lex.lookahead(Sym.FINAL, Sym.PUBLIC, Sym.TYPEDEF) || this.lex.lookahead(Sym.PUBLIC, Sym.FINAL, Sym.TYPEDEF);
        
        if (isTypedef || isPublicTypedef || isFinalTypedef || isPublicFinalTypedef) {
            TjTypeDefNode tjType = this.parseTjTypeDef();
            if (tjType != null) {
                if (this.lexSymbol.sym == Sym.EOF) {
                    return new CompilationUnitNode(s, helperNode, importList, tjType);
                }
                throw new CompilerException("Unexpected end of file", this.lexSymbol);
            }
        }
        /*END TeaJaySupport*/


        TypeDeclarationNode decl = this.parseTypeDeclaration();
        
        if (this.lexSymbol.sym == Sym.EOF) {
            return new CompilationUnitNode(s, helperNode, importList, decl);
        }
        throw new CompilerException("Unexpected end of file", this.lexSymbol);
    }

    /**
     * RULE: Annotation { Annotation }
     */
    private AnnotationsNode parseAnnotations() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        AnnotationNode annotation = this.parseAnnotation();
        if (annotation != null) {
            ArrayList<AnnotationNode> aList = new ArrayList<>();
            AnnotationNode a = this.parseAnnotation();
            while (a != null) {
                aList.add(a);
                a = this.parseAnnotation();
            }
            return new AnnotationsNode(s, annotation, aList);
        }

        return null;
    }

    /*
     * RULE: Identfier {"."Identfier}
     */
    private QualifiedIdentifierNode parseQualifiedIdentfier() throws IOException, CompilerException {
        ArrayList<IdentifierNode> identifiers = new ArrayList<>();
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.IDENTIFIER)) {
            identifiers.add(new IdentifierNode(s, this.lastTokenText));
            while (this.accept(Sym.DOT)) {
                s = this.lexSymbol;
                if (!this.accept(Sym.IDENTIFIER)) {
                    throw new CompilerException("Expect identifier after .", this.lexSymbol);
                }
                identifiers.add(new IdentifierNode(s, this.lastTokenText));

            }
            return new QualifiedIdentifierNode(identifiers);
        }
        return null;
    }

    /*
     * RULE: QualifiedIdentifier {"," QualifiedIdentifier}
     */
    private QualifiedIdentifierListNode parseQualifiedIdentifierList() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        QualifiedIdentifierNode q = this.parseQualifiedIdentfier();

        if (q != null) {
            ArrayList<QualifiedIdentifierNode> list = new ArrayList<>();
            while (this.accept(Sym.COMMA)) {
                QualifiedIdentifierNode qId = this.parseQualifiedIdentfier();
                if (qId != null) {
                    list.add(qId);
                } else {
                    throw new CompilerException("Expectetd qualified identifier", this.lexSymbol);
                }
            }
            return new QualifiedIdentifierListNode(s, q, list);

        }
        return null;
    }

    /*
     * RULE: "import" ["static"] QualifiedIdentfier [".*"] ";"
     */
    private ImportDeclarationNode parseImportDeclaration() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        //BLOCK 1
        if (this.accept(Sym.IMPORT)) {
            if (this.accept(Sym.STATIC)) {
                QualifiedIdentifierNode qIn = this.parseQualifiedIdentfier();
                if (qIn != null) {
                    if (this.accept(Sym.DOTSTAR)) {
                        //BLOCK 1a
                        if (this.accept(Sym.SEMICOLON)) {
                            //Konsumiere überflüssige Smikolons;
                            if (!this.lex.lookahead(Sym.SEMICOLON, Sym.EOF)) {
                                while (this.accept(Sym.SEMICOLON)) {
                                    if (this.lex.lookahead(Sym.SEMICOLON, Sym.EOF)) {
                                        break;
                                    }
                                }
                            }
                            return new ImportDeclarationNode(s, qIn, true, true);
                        }
                        throw new CompilerException("Expect ; in import declaration", this.lexSymbol);
                    }
                    //BLOCK 1b
                    if (this.accept(Sym.SEMICOLON)) {
                        //Konsumiere überflüssige Smikolons;
                        if (!this.lex.lookahead(Sym.SEMICOLON, Sym.EOF)) {
                            while (this.accept(Sym.SEMICOLON)) {
                                if (this.lex.lookahead(Sym.SEMICOLON, Sym.EOF)) {
                                    break;
                                }
                            }
                        }
                        return new ImportDeclarationNode(s, qIn, true, false);
                    }
                    throw new CompilerException("Expect ; in import declaration", this.lexSymbol);
                }
            }
            //BLOCK 2
            QualifiedIdentifierNode qIn = this.parseQualifiedIdentfier();
            if (qIn != null) {
                if (this.accept(Sym.DOTSTAR)) {
                    //BLOCK 2a
                    if (this.accept(Sym.SEMICOLON)) {
                        //Konsumiere überflüssige Smikolons;
                        if (!this.lex.lookahead(Sym.SEMICOLON, Sym.EOF)) {
                            while (this.accept(Sym.SEMICOLON)) {
                                if (this.lex.lookahead(Sym.SEMICOLON, Sym.EOF)) {
                                    break;
                                }
                            }
                        }
                        return new ImportDeclarationNode(s, qIn, false, true);
                    }
                    throw new CompilerException("Expect ; in import declaration", this.lexSymbol);
                }
                //BLOCK 2b
                if (this.accept(Sym.SEMICOLON)) {
                    //Konsumiere überflüssige Smikolons;
                    if (!this.lex.lookahead(Sym.SEMICOLON, Sym.EOF)) {
                        while (this.accept(Sym.SEMICOLON)) {
                            if (this.lex.lookahead(Sym.SEMICOLON, Sym.EOF)) {
                                break;
                            }
                        }
                    }
                    return new ImportDeclarationNode(s, qIn, false, false);
                }
                throw new CompilerException("Expect ; in import declaration", this.lexSymbol);
            }
        }
        return null;
    }

    /*
     * RULE: ClassOrInterfaceDeclaration | TjImplDeclaration | ;
     */
    private TypeDeclarationNode parseTypeDeclaration() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        ClassOrInterfaceDeclaratioNode classOrInterfaceNode = this.parseClassOrInterfaceDeclaration();
        if (classOrInterfaceNode != null) {
            return new TypeDeclarationNode(s, classOrInterfaceNode);
        }
        TjImplDeclarationNode tjImpl = this.parseTjImplDeclaration();
        if (tjImpl != null) {
            return new TypeDeclarationNode(s, tjImpl);
        }
        if (this.accept(Sym.SEMICOLON)) {
            return new TypeDeclarationNode(s, true);
        }
        return null;
    }

    private ClassOrInterfaceDeclaratioNode parseClassOrInterfaceDeclaration() throws IOException, CompilerException {
        return this.parseClassOrInterfaceDeclaration(null);
    }
    /*
     * RULE: {Modifier} ( ClassDeclaration | InterfaceDeclaration )
     */

    private ClassOrInterfaceDeclaratioNode parseClassOrInterfaceDeclaration(List<VariableModifierNode> vList) throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        ArrayList<ModifierNode> modifiers = new ArrayList<>();
        if (vList != null) {
            for (VariableModifierNode v : vList) {
                if (v.isAnnotation()) {
                    modifiers.add(new ModifierNode(v.getSymbol(), v.getAnnotationNode()));
                } else {
                    modifiers.add(new ModifierNode(v.getSymbol(), ModifierNode.Modifier.FINAL));
                }
            }
        }

        ModifierNode modifier = this.parseModifier();
        while (modifier != null) {
            modifiers.add(modifier);
            modifier = this.parseModifier();
        }
        ClassDeclarationNode classDeclaration = this.parseClassDeclaration();
        if (classDeclaration != null) {
           // throw new CompilerException("Class declaration not allowed in TeaJay",s);
            return new ClassOrInterfaceDeclaratioNode(s, modifiers, classDeclaration, null);
        }
        InterfaceDeclarationNode interfaceDeclaration = this.parseInterfaceDeclaration();
        if (interfaceDeclaration != null) {
            return new ClassOrInterfaceDeclaratioNode(s, modifiers, null, interfaceDeclaration);
        }

        return null;
    }
    /*
     * RULE: Annotations | public | protected | private | static | abstract |
     * final | native | synchronized | transient | volatile | strictfp
     */

    private ModifierNode parseModifier() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.lex.lookahead(Sym.STATIC, Sym.CURLY_OPEN)) {
            return null;
        }
        if (this.lex.lookahead(Sym.SYNCHRONIZED, Sym.BRACKET_OPEN)) {
            return null;
        }
        if (this.accept(Sym.PUBLIC)) {
            return new ModifierNode(s, ModifierNode.Modifier.PUBLIC);
        }
        if (this.accept(Sym.PROTECTED)) {
            return new ModifierNode(s, ModifierNode.Modifier.PROTECTED);
        }
        if (this.accept(Sym.PRIVATE)) {
            return new ModifierNode(s, ModifierNode.Modifier.PRIVATE);
        }
        if (this.accept(Sym.STATIC)) {
            return new ModifierNode(s, ModifierNode.Modifier.STATIC);
        }
        if (this.accept(Sym.ABSTRACT)) {
            return new ModifierNode(s, ModifierNode.Modifier.ABSTRACT);
        }
        if (this.accept(Sym.FINAL)) {
            return new ModifierNode(s, ModifierNode.Modifier.FINAL);
        }
        if (this.accept(Sym.NATIVE)) {
            return new ModifierNode(s, ModifierNode.Modifier.NATIVE);
        }
        if (this.accept(Sym.SYNCHRONIZED)) {
            return new ModifierNode(s, ModifierNode.Modifier.SYNCHRONIZED);
        }
        if (this.accept(Sym.TRANSIENT)) {
            return new ModifierNode(s, ModifierNode.Modifier.TRANSIENT);
        }
        if (this.accept(Sym.VOLATILE)) {
            return new ModifierNode(s, ModifierNode.Modifier.VOLATILE);
        }
        if (this.accept(Sym.STRICTFP)) {
            return new ModifierNode(s, ModifierNode.Modifier.STRICTFP);
        }

        boolean isAnnotationInterface = this.lex.lookahead(Sym.AT, Sym.INTERFACE);
        if (isAnnotationInterface) {
            return null;
        }

        AnnotationNode annotation = this.parseAnnotation();
        if (annotation != null) {
            return new ModifierNode(s, annotation);
        }
        return null;
    }

    /*
     * RULE: NormalClassDeclaration | EnumDeclaration
     */
    private ClassDeclarationNode parseClassDeclaration() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        NormalClassDeclarationNode normalClassDecl = this.parseNormalClassDeclaration();
        if (normalClassDecl != null) {
           throw new CompilerException("Class declaration not allowed in TeaJay",s);
            // return new ClassDeclarationNode(s, normalClassDecl);
        }
        EnumDeclarationNode enumDecl = this.parseEnumDeclaration();
        if (enumDecl != null) {
            return new ClassDeclarationNode(s, enumDecl);
        }
        return null;
    }

    /*
     *
     * RULE: NormalInterfaceDeclaration | AnnotationTypeDeclaration
     */
    private InterfaceDeclarationNode parseInterfaceDeclaration() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        NormalInterfaceDeclarationNode normalInterface = this.parseNormalInterfaceDeclaration();
        if (normalInterface != null) {
            return new InterfaceDeclarationNode(s, normalInterface);
        }
        AnnotationTypeDeclarationNode annotationType = this.parseAnnotationTypeDeclaration();
        if (annotationType != null) {
            throw new CompilerException("Annotation type declaration not supported yet",s);
            //return new InterfaceDeclarationNode(s, annotationType);
        }
        return null;
    }

    /*
     * RULE: "class" Identifier [TypeParameters] [extends Type] [implements
     * TypeList] ClassBody *
     */
    private NormalClassDeclarationNode parseNormalClassDeclaration() throws IOException, CompilerException {

        if (!this.accept(Sym.CLASS)) {
            return null;
        }
        Symbol s = this.lexSymbol;
        if (!this.accept(Sym.IDENTIFIER)) {
            throw new CompilerException("Missing identifier after class declaration", this.lexSymbol);
        }
        String className = this.lastTokenText;

        TypeParametersNode typeParameters = this.parseTypeParameters();
        TypeNode type = null;
        if (this.accept(Sym.EXTENDS)) {
            type = this.parseType();
            if (type == null) {
                throw new CompilerException("Missing type after extends declaration", this.lexSymbol);
            }
        }
        TypeListNode typeList = null;
        if (this.accept(Sym.IMPLEMENTS)) {
            typeList = this.parseTypeList();
            if (typeList == null) {
                throw new CompilerException("Missing interface type after interface declaration", this.lexSymbol);
            }
        }
        ClassBodyNode classBody = this.parseClassBody();
        if (classBody != null) {
            return new NormalClassDeclarationNode(s, className, typeParameters,
                    type, typeList, classBody);
        }

        return null;
    }

    /*
     * RULE: "enum" Identifier ["implements" TypeList] EnumBody
     */
    private EnumDeclarationNode parseEnumDeclaration() throws IOException, CompilerException {
        if (!this.accept(Sym.ENUM)) {
            return null;
        }
        Symbol s = this.lexSymbol;
        if (!this.accept(Sym.IDENTIFIER)) {
            throw new CompilerException("Missing identifier after enum declaration", this.lexSymbol);
        }
        String enumName = this.lastTokenText;
        TypeListNode typeList = null;
        if (this.accept(Sym.IMPLEMENTS)) {
            typeList = this.parseTypeList();
            if (typeList == null) {
                throw new CompilerException("Missing interface type after interface declaration", this.lexSymbol);
            }
            throw new CompilerException("interface declaration not allowed here",s);
        }
        EnumBodyNode enumBody = this.parseEnumBody();
        if (enumBody != null) {
            return new EnumDeclarationNode(s, enumName, typeList, enumBody);
        } else {
            throw new CompilerException("Missing { after enum declaration", this.lexSymbol);
        }
    }

    /*
     * RULE: "interface" Identifier [TypeParameters] ["extends" TypeList]
     * InterfaceBody
     */
    private NormalInterfaceDeclarationNode parseNormalInterfaceDeclaration() throws IOException, CompilerException {
        if (!this.accept(Sym.INTERFACE)) {
            return null;
        }
        Symbol s = this.lexSymbol;
        if (!this.accept(Sym.IDENTIFIER)) {
            throw new CompilerException("Missing identifier after Interface declaration", this.lexSymbol);
        }
        String interfaceName = this.lastTokenText;

        TypeParametersNode parameters = this.parseTypeParameters();
        TypeListNode typeList = null;
        if (this.accept(Sym.EXTENDS)) {
            typeList = this.parseTypeList();
            if (typeList == null) {
                throw new CompilerException("Missing type list after extends", this.lexSymbol);
            }
        }
        InterfaceBodyNode interfaceBody = this.parseInterfaceBody();
        if (interfaceBody != null) {
            return new NormalInterfaceDeclarationNode(s, interfaceName, parameters, typeList, interfaceBody);
        }
        return null;
    }

    /*
     * RULE: "@" "interface" Identifier AnnotationTypeBodyNode
     */
    private AnnotationTypeDeclarationNode parseAnnotationTypeDeclaration() throws IOException, CompilerException {
        if (!this.accept(Sym.AT)) {
            return null;
        }
        if (!this.accept(Sym.INTERFACE)) {
            throw new CompilerException("Missing interface after annotation declaration", this.lexSymbol);
        }
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.IDENTIFIER)) {
            String ident = this.lastTokenText;
            AnnotationTypeBodyNode annotationBody = this.parseAnnotationTypeBody();
            if (annotationBody != null) {
                return new AnnotationTypeDeclarationNode(s, ident, annotationBody);
            }
        }
        throw new CompilerException("Missing identifier after annotation declaration", this.lexSymbol);
    }

    /*
     * RULE: "[" "]"
     */
    private EdgeClampsOpenCloseHelperNode parseEdgeClampsOpenCloseHelper() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.EDGECLAMPS_OPEN)) {
            if (this.accept(Sym.EDGECLAMPS_CLOSE)) {
                return new EdgeClampsOpenCloseHelperNode(s);
            }
            throw new CompilerException("Expect ] after [", this.lexSymbol);
        }
        return null;
    }

    /*
     * RULE: "<" TypeParameter {, TypeParameter } ">"
     */
    private TypeParametersNode parseTypeParameters() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.OP_LESS)) {
            TypeParameterNode typeParam = this.parseTypeParameter();

            if (typeParam != null) {
                ArrayList<TypeParameterNode> tList = new ArrayList<>();
                while (this.accept(Sym.COMMA)) {
                    TypeParameterNode tParam = this.parseTypeParameter();
                    if (tParam != null) {
                        tList.add(tParam);
                    } else {
                        throw new CompilerException("Expected type parameter", this.lexSymbol);
                    }
                }
                if (this.accept(Sym.OP_GREATER)) {
                    return new TypeParametersNode(s, typeParam, tList);
                }
                throw new CompilerException("Missing > after <", this.lexSymbol);
            }
            throw new CompilerException("Missing identifier after <", this.lexSymbol);
        }
        return null;

    }

    /**
     * BasicType: {EdgeClampsOpenCloseHelper} | ReferenceType
     * {EdgeClampsOpenCloseHelper} RULE: Type {EdgeClampsOpenCloseHelper}
     * closure [Env] <Return> (ParamList)
     *
     * public closure <RETURN>(ParamList) methode(Number a) Closure <RETURN>
     * (Parameter) ck
     *
     */
    private TypeNode parseType() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        BasicTypeNode basicType = parseBasicType();
        if (basicType != null) {
            ArrayList<EdgeClampsOpenCloseHelperNode> edgeClamps =
                    new ArrayList<>();
            EdgeClampsOpenCloseHelperNode edgeClamp =
                    this.parseEdgeClampsOpenCloseHelper();
            while (edgeClamp != null) {
                edgeClamps.add(edgeClamp);
                edgeClamp = this.parseEdgeClampsOpenCloseHelper();
            }
           // throw new CompilerException("Java primitives in TeaJay not allowed",s);
            return new TypeNode(s, basicType, null, edgeClamps);
        }
        ReferenceTypeNode referenceType = this.parseReferenceType();
        if (referenceType != null) {

            ArrayList<EdgeClampsOpenCloseHelperNode> edgeClamps =
                    new ArrayList<>();
            EdgeClampsOpenCloseHelperNode edgeClamp =
                    this.parseEdgeClampsOpenCloseHelper();
            while (edgeClamp != null) {
                edgeClamps.add(edgeClamp);
                edgeClamp = this.parseEdgeClampsOpenCloseHelper();
            }
            
            return new TypeNode(s, null, referenceType, edgeClamps);
        }
        //TeaJay
        ClosureTypeDecNode closureTypeDec = this.parseClosureTypeDec();
        if (closureTypeDec != null) {
            return new TypeNode(s, closureTypeDec);
        }

        return null;
    }

    /*
     * RULE: ReferenceType {"," ReferenceType }
     * =>  RULE: Type  { "," Type}
     */
    private TypeListNode parseTypeList() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        TypeNode r = this.parseType();
        if (r != null) {
            ArrayList<TypeNode> list = new ArrayList<>();
            while (this.accept(Sym.COMMA)) {
                TypeNode rtype = this.parseType();
                if (rtype != null) {
                    list.add(rtype);
                } else {
                    throw new CompilerException("Expectetd reference type", this.lexSymbol);
                }
            }
            return new TypeListNode(s, r, list);
        }
        return null;
    }

    /*
     * RULE: "{" { ClassBodyDeclaration } "}"
     */
    private ClassBodyNode parseClassBody() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.CURLY_OPEN)) {
            ClassBodyDeclarationNode bodyDecl = this.parseClassBodyDeclaration();
            ArrayList<ClassBodyDeclarationNode> cList = new ArrayList<>();
            while (bodyDecl != null) {
                cList.add(bodyDecl);
                bodyDecl = this.parseClassBodyDeclaration();
            }
            if (this.accept(Sym.CURLY_CLOSE)) {
                return new ClassBodyNode(s, cList);
            }
            throw new CompilerException("Missing } after {", this.lexSymbol);
        }
        return null;
    }

    /*
     * RULE: "{" [EnumConstants] [","] [EnumBodyDeclarations] "}"
     */
    private EnumBodyNode parseEnumBody() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.CURLY_OPEN)) {
            EnumConstantsNode enumConstants = this.parseEnumConstantsParse();
            if (enumConstants != null) {
                if (this.accept(Sym.COMMA)) {
                    EnumBodyDeclarationsNode enumBodyDecls = this.parseEnumBodyDeclarations();
                    if (enumBodyDecls != null) {
                        if (this.accept(Sym.CURLY_CLOSE)) {
                            return new EnumBodyNode(s, enumConstants, true, enumBodyDecls);
                        }
                        throw new CompilerException("Missing } after { enum constants , enum body declaration", this.lexSymbol);
                    }
                }
                EnumBodyDeclarationsNode enumBodyDecls = this.parseEnumBodyDeclarations();
                if (enumBodyDecls != null) {
                    if (this.accept(Sym.CURLY_CLOSE)) {
                        return new EnumBodyNode(s, enumConstants, true, enumBodyDecls);
                    }
                    throw new CompilerException("Missing } after { enum constants , enum body declaration", this.lexSymbol);
                }
                if (this.accept(Sym.CURLY_CLOSE)) {
                    return new EnumBodyNode(s, enumConstants);
                }
                throw new CompilerException("Missing } after { enum constants , enum body declaration", this.lexSymbol);

            }
            if (this.accept(Sym.COMMA)) {
                EnumBodyDeclarationsNode enumBodyDecls = this.parseEnumBodyDeclarations();
                if (enumBodyDecls != null) {
                    if (this.accept(Sym.CURLY_CLOSE)) {
                        return new EnumBodyNode(s, true, enumBodyDecls);
                    }
                    throw new CompilerException("Missing } after { enum constants , enum body declaration", this.lexSymbol);
                }
                if (this.accept(Sym.CURLY_CLOSE)) {
                    return new EnumBodyNode(s, true);
                }
                throw new CompilerException("Missing } after { enum constants , enum body declaration", this.lexSymbol);
            }
            EnumBodyDeclarationsNode enumBodyDecls = this.parseEnumBodyDeclarations();
            if (enumBodyDecls != null) {
                if (this.accept(Sym.CURLY_CLOSE)) {
                    return new EnumBodyNode(s, true, enumBodyDecls);
                }
                throw new CompilerException("Missing } after { enum constants , enum body declaration", this.lexSymbol);
            }
            if (this.accept(Sym.CURLY_CLOSE)) {
                return new EnumBodyNode(s, true);
            }
            throw new CompilerException("Missing } after { enum constants , enum body declaration", this.lexSymbol);

        }
//        if (this.accept(Sym.CURLY_OPEN)) {
//            EnumConstantsNode enumConstants = this.parseEnumConstantsParse();
//            if (enumConstants != null) {
//                if (this.accept(Sym.COMMA)) {
//                    EnumBodyDeclarationsNode enumBodyDecl = this.parseEnumBodyDeclarations();
//                    if (enumBodyDecl != null) {
//                        if (this.accept(Sym.CURLY_CLOSE)) {
//                            return new EnumBodyNode(enumConstants, true, enumBodyDecl);
//                        }
//                        throw new CompilerException("Missing } after { enum constants , enum body declaration",
//                                "Message from parseEnumBody", 2);
//                    }
//                    throw new CompilerException("Missing } after { enum constants  ",
//                            "Message from parseEnumBody", 2);
//                }
//                throw new CompilerException("Missing } after { enum constants , ",
//                        "Message from parseEnumBody", 2);
//            }
//            if (this.accept(Sym.COMMA)) {
//                EnumBodyDeclarationsNode enumBodyDecl = this.parseEnumBodyDeclarations();
//                if (enumBodyDecl != null) {
//                    if (this.accept(Sym.CURLY_CLOSE)) {
//                        return new EnumBodyNode(true, enumBodyDecl);
//                    }
//                    throw new CompilerException("Missing } after {  , enum body declaration",
//                            "Message from parseEnumBody", 2);
//                }
//                throw new CompilerException("Missing } after {, ",
//                        "Message from parseEnumBody", 2);
//            }
//            EnumBodyDeclarationsNode enumBodyDecl = this.parseEnumBodyDeclarations();
//            if (enumBodyDecl != null) {
//                if (this.accept(Sym.CURLY_CLOSE)) {
//                    return new EnumBodyNode(enumBodyDecl);
//                }
//                throw new CompilerException("Missing } after {  , enum body declaration",
//                        "Message from parseEnumBody", 2);
//            }
//            if (this.accept(Sym.CURLY_CLOSE)) {
//                    return new EnumBodyNode();
//            }
//            throw new CompilerException("Missing } after {  ",
//                    "Message from parseEnumBody", 2);
//        }
        return null;
    }
    /*
     * RULE: "{" {InterfaceBodyDeclaration} "}"
     */

    private InterfaceBodyNode parseInterfaceBody() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.CURLY_OPEN)) {
            InterfaceBodyDeclarationNode interfaceBodyDecl;
            ArrayList<InterfaceBodyDeclarationNode> interfaceList = new ArrayList<>();
            interfaceBodyDecl = this.parseInterfaceBodyDeclaration();
            while (interfaceBodyDecl != null) {
                interfaceList.add(interfaceBodyDecl);
                interfaceBodyDecl = this.parseInterfaceBodyDeclaration();
            }
            if (this.accept(Sym.CURLY_CLOSE)) {
                return new InterfaceBodyNode(s, interfaceList);
            } else {
                throw new CompilerException("Missing } after { ", this.lexSymbol);
            }
        }
        return null;
    }
    /*
     * RULE: "{" [AnnotationTypeElementDeclarations]"}"
     */

    private AnnotationTypeBodyNode parseAnnotationTypeBody() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.CURLY_OPEN)) {
            AnnotationTypeElementDeclarationsNode annotTypeElementDecls = this.parseAnnotationTypeElementDeclarations();
            if (annotTypeElementDecls != null) {
                if (this.accept(Sym.CURLY_CLOSE)) {
                    return new AnnotationTypeBodyNode(s, annotTypeElementDecls);
                }
                throw new CompilerException("Missing } after { ", this.lexSymbol);
            }
            if (this.accept(Sym.CURLY_CLOSE)) {
                return new AnnotationTypeBodyNode(s);
            }
            throw new CompilerException("Missing } after { ", this.lexSymbol);
        }
        return null;
    }

    /*
     * RULE: "byte" | "short" | "char" | "int" |"long" | "float" | "double" |
     * "boolean"
     */
    private BasicTypeNode parseBasicType() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.BYTE)) {
            return new BasicTypeNode(s, BasicTypeNode.BasicType.BYTE);
        }
        if (this.accept(Sym.SHORT)) {
            return new BasicTypeNode(s, BasicTypeNode.BasicType.SHORT);
        }
        if (this.accept(Sym.CHAR)) {
            return new BasicTypeNode(s, BasicTypeNode.BasicType.CHAR);
        }
        if (this.accept(Sym.INT)) {
            return new BasicTypeNode(s, BasicTypeNode.BasicType.INT);
        }
        if (this.accept(Sym.LONG)) {
            return new BasicTypeNode(s, BasicTypeNode.BasicType.LONG);
        }
        if (this.accept(Sym.FLOAT)) {
            return new BasicTypeNode(s, BasicTypeNode.BasicType.FLOAT);
        }
        if (this.accept(Sym.DOUBLE)) {
            return new BasicTypeNode(s, BasicTypeNode.BasicType.DOUBLE);
        }
        if (this.accept(Sym.BOOLEAN)) {
            return new BasicTypeNode(s, BasicTypeNode.BasicType.BOOLEAN);
        }
        return null;
    }

    /*
     * RULE: Identifier [TypeArguments] { DotIdentifierTypeArgumentsHelper }
     */
    private ReferenceTypeNode parseReferenceType() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.IDENTIFIER)) {
            String ident = this.lastTokenText;
            TypeArgumentsNode typeArguments = this.parseTypeArguments();
            DotIdentifierTypeArgumentsHelperNode dotedArgument = this.parseDotIdentifierTypeArgumentsHelper();

            ArrayList<DotIdentifierTypeArgumentsHelperNode> dotedArguments = new ArrayList<>();

            while (dotedArgument != null) {
                dotedArguments.add(dotedArgument);
                dotedArgument = this.parseDotIdentifierTypeArgumentsHelper();
            }

            return new ReferenceTypeNode(s, ident, typeArguments, dotedArguments);
        }
        return null;
    }

    /*
     * "<" TypeArgument {TypeArgumentsListHelper } ">"
     */
    private TypeArgumentsNode parseTypeArguments() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.OP_LESS)) {
            this.lex.setTemplateState();
            TypeArgumentNode typeArgument = parseTypeArgument();
            if (typeArgument != null) {
                ArrayList<TypeArgumentsListHelperNode> tlist = new ArrayList<>();
                TypeArgumentsListHelperNode thelp = this.parseTypeArgumentsListHelper();
                while (thelp != null) {
                    tlist.add(thelp);
                    thelp = this.parseTypeArgumentsListHelper();
                }

                if (this.accept(Sym.OP_GREATER)) {
                    this.lex.resetTemplateState();
                    return new TypeArgumentsNode(s, typeArgument, tlist);
                }
                throw new CompilerException("Missing > after < ", this.lexSymbol);
            }
            throw new CompilerException("Missing type argument after < ", this.lexSymbol);
        }
        return null;
    }

    /*
     * RULE: "." Identifier [TypeArguments]
     */
    private DotIdentifierTypeArgumentsHelperNode parseDotIdentifierTypeArgumentsHelper() throws IOException, CompilerException {
        if (this.accept(Sym.DOT)) {
            Symbol s = this.lexSymbol;
            if (this.accept(Sym.IDENTIFIER)) {
                String ident = this.lastTokenText;
                TypeArgumentsNode arguments = this.parseTypeArguments();
                return new DotIdentifierTypeArgumentsHelperNode(s, ident, arguments);
            }
            throw new CompilerException("Missing identifier after . ", this.lexSymbol);

        }
        return null;
    }


    /*
     * RULE: "," TypeArgument
     */
    private TypeArgumentsListHelperNode parseTypeArgumentsListHelper() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.COMMA)) {
            TypeArgumentNode type = this.parseTypeArgument();
            if (type != null) {
                return new TypeArgumentsListHelperNode(s, type);
            }
        }
        return null;
    }

    /*
     * RULE: Type | "?" [(extends | super) Type]
     */
    private TypeArgumentNode parseTypeArgument() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        TypeNode type = this.parseType();
        if (type != null) {
            return new TypeArgumentNode(s, type);
        }
        if (this.accept(Sym.QUESTIONMARK)) {

            if (this.accept(Sym.SUPER)) {
                type = this.parseType();
                if (type != null) {
                    return new TypeArgumentNode(s, true, false, true, type);
                }
                //Elsewise parseReferenceType send exception
            }
            if (this.accept(Sym.EXTENDS)) {
                type = this.parseType();
                if (type != null) {
                    return new TypeArgumentNode(s, true, true, false, type);
                }
                //Elsewise parseReferenceType send exception
            }
            return new TypeArgumentNode(s, true, false, false, null);
        }
        return null;
    }

    /*
     * RULE: "<" TypeList ">"
     */
    private NonWildcardTypeArgumentsNode parseNonWildcardTypeArguments() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.OP_LESS)) {
            this.lex.setTemplateState();
            TypeListNode typeList = this.parseTypeList();
            if (typeList != null) {
                if (!this.accept(Sym.OP_GREATER)) {
                    throw new CompilerException("Missing > after <", this.lexSymbol);
                }
                this.lex.resetTemplateState();
                return new NonWildcardTypeArgumentsNode(s, typeList);
            }
        }
        return null;
    }

    /*
     * RULE: "<" ">" | TypeArguments
     */
    private TypeArgumentsOrDiamondNode parseTypeArgumentsOrDiamond() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        boolean isDiamond = this.lex.lookahead(Sym.OP_LESS, Sym.OP_GREATER);// new RuleSwitcher(lexSymbol, lex, Sym.OP_LESS, Sym.OP_GREATER).test();
        if (isDiamond) {
            if (this.accept(Sym.OP_LESS)) {
                if (this.accept(Sym.OP_GREATER)) {
                    return new TypeArgumentsOrDiamondNode(s, true);
                }
                throw new CompilerException("Missing > after <", this.lexSymbol);
            }
        }
        TypeArgumentsNode typeArguments = this.parseTypeArguments();
        if (typeArguments != null) {
            return new TypeArgumentsOrDiamondNode(s, typeArguments);
        }
        return null;
    }

    /*
     * RULE: "<" ">"| NonWildcardTypeArgumentsOrDiamond
     */
    private NonWildcardTypeArgumentsOrDiamond parseNonWildcardTypeArgumentsOrDiamond() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.lex.lookahead(Sym.OP_LESS, Sym.OP_GREATER)) {
            if (this.accept(Sym.OP_LESS)) {
                this.lex.setTemplateState();
                if (this.accept(Sym.OP_GREATER)) {
                    this.lex.resetTemplateState();
                    return new NonWildcardTypeArgumentsOrDiamond(s, true);
                }
                throw new CompilerException("Missing > after <", this.lexSymbol);
            }

        }
        NonWildcardTypeArgumentsNode typeArguments = this.parseNonWildcardTypeArguments();
        if (typeArguments != null) {
            return new NonWildcardTypeArgumentsOrDiamond(s, typeArguments);
        }
        return null;
    }
    /*
     * RULE: Identifier [ extends Bound ]
     */

    private TypeParameterNode parseTypeParameter() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.IDENTIFIER)) {
            String ident = this.lastTokenText;
            if (this.accept(Sym.EXTENDS)) {
                BoundNode bound = this.parseBound();
                if (bound != null) {
                    return new TypeParameterNode(s, ident, bound);
                }
                throw new CompilerException("Bound statement error", this.lexSymbol);
            }
            return new TypeParameterNode(s, ident, null);
        }
        return null;
    }

    /*
     * RULE: ReferenceType { & ReferenceType }
     */
    private BoundNode parseBound() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        ReferenceTypeNode referenceTypeNode = this.parseReferenceType();
        if (referenceTypeNode != null) {
            ArrayList<ReferenceTypeNode> rList = new ArrayList<>();
            while (this.accept(Sym.OP_AND)) {
                ReferenceTypeNode ref = this.parseReferenceType();
                if (ref != null) {
                    rList.add(ref);
                } else {
                    throw new CompilerException("Missing reference type ", this.lexSymbol);
                }
            }
            return new BoundNode(s, referenceTypeNode, rList);
        }
        return null;
    }

    /*
     * RULE: "@" QualifiedIdentfier [ "(" [ AnnotationElement ] ")" ]
     */
    private AnnotationNode parseAnnotation() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.lex.lookahead(Sym.AT, Sym.INTERFACE)) {
            return null;
        }

        if (this.accept(Sym.AT)) {
            QualifiedIdentifierNode qId = this.parseQualifiedIdentfier();
            if (qId != null) {

                if (this.accept(Sym.BRACKET_OPEN)) {
                    AnnotationElementNode element = this.parseAnnotationElement();
                    if (element != null) {
                        if (this.accept(Sym.BRACKET_CLOSE)) {
                            return new AnnotationNode(s, qId, element);
                        } else {
                            throw new CompilerException("Missing ) after ( ", this.lexSymbol);
                        }
                    }
                    if (this.accept(Sym.BRACKET_CLOSE)) {
                        return new AnnotationNode(s, qId, null);
                    } else {
                        throw new CompilerException("Missing ) after ( ", this.lexSymbol);
                    }

                }
                return new AnnotationNode(s, qId, null);
            } else {
                throw new CompilerException("Missing qualified identifier after @", this.lexSymbol);
            }
        }
        return null;
    }

    /*
     * RULE: ElementValuePairs | ElementValue
     */
    private AnnotationElementNode parseAnnotationElement() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        boolean isValuePair = this.lex.lookahead(Sym.IDENTIFIER, Sym.OP_EQ);
        if (isValuePair) {
            ElementValuePairsNode valuePairs = this.parseElementValuePairs();

            if (valuePairs != null) {
                return new AnnotationElementNode(s, valuePairs);
            }
        }
        ElementValueNode value = this.parseElementValue();
        if (value != null) {
            return new AnnotationElementNode(s, value);
        }
        return null;
    }

    /*
     * RULE: ElementValuePair {"," ElementValuePair }
     */
    private ElementValuePairsNode parseElementValuePairs() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        ElementValuePairNode valuePair = this.parseElementValuePair();
        if (valuePair != null) {
            ArrayList<ElementValuePairNode> eList = new ArrayList<>();

            while (this.accept(Sym.COMMA)) {
                ElementValuePairNode v = this.parseElementValuePair();
                if (v != null) {
                    eList.add(v);
                } else {
                    throw new CompilerException("Missing element value", this.lexSymbol);
                }
            }
            return new ElementValuePairsNode(s, valuePair, eList);
        }
        return null;
    }

    /*
     * RULE: Annotation | Expression1 | ElementValueArrayInitializer
     */
    private ElementValueNode parseElementValue() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        AnnotationNode annotation = this.parseAnnotation();
        if (annotation != null) {
            return new ElementValueNode(s, annotation);
        }
        Expression1Node expr = this.parseExpression1();
        if (expr != null) {
            return new ElementValueNode(s, expr);
        }
        ElementValueArrayInitializerNode valueArray = this.parseElementValueArrayInitializer();
        if (valueArray != null) {
            return new ElementValueNode(s, valueArray);
        }
        return null;
    }

    /*
     * RULE: IDENTIFIER = ElemetValue
     */
    private ElementValuePairNode parseElementValuePair() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.IDENTIFIER)) {
            String ident = this.lastTokenText;
            if (this.accept(Sym.OP_EQ)) {
                ElementValueNode value = this.parseElementValue();
                if (value != null) {
                    return new ElementValuePairNode(s, ident, value);
                } else {
                    throw new CompilerException("Expected element value aufter =", this.lexSymbol);
                }
            } else {
                throw new CompilerException("Missing = after identifier", this.lexSymbol);
            }
        }
        return null;
    }

    private Expression1Node parseExpression1() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        Expression2Node exp2 = this.parseExpression2();
        if (exp2 != null) {
            Expression1RestNode exp1Rest = this.parseExpression1Rest();
            if (exp1Rest != null) {
                return new Expression1Node(s, exp2, exp1Rest);
            }
            return new Expression1Node(s, exp2);
        }
        return null;
    }


    /*
     * "{"[ElementValues] [","] "}"
     * ==> "{"[ElementValues] "}" 
     */
    private ElementValueArrayInitializerNode parseElementValueArrayInitializer() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.CURLY_OPEN)) {
            ElementValuesNode values = this.parseElementValues();
            if (this.accept(Sym.CURLY_CLOSE)) {
                return new ElementValueArrayInitializerNode(s, values, false);

            } else {
                throw new CompilerException("Missing } after {", this.lexSymbol);
            }
        }
        return null;
    }

    /* RULE: ElementValue { "," ElementValue }
     * RULE ==>: ElementValue { "," ElementValue } [","]
     */
    private ElementValuesNode parseElementValues() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        ElementValueNode value = this.parseElementValue();
        if (value != null) {

            ArrayList<ElementValueNode> eList = new ArrayList<>();
            while (this.accept(Sym.COMMA)) {
                ElementValueNode v = this.parseElementValue();
                if (v != null) {
                    eList.add(v);

                } else {
                    break;
                }
            }
            return new ElementValuesNode(s, value, eList);
        }
        return null;
    }

    /*
     * RULE: ";" | {Modifier} MemberDecl | static Block
     */
    private ClassBodyDeclarationNode parseClassBodyDeclaration() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.SEMICOLON)) {
            return new ClassBodyDeclarationNode(s, true);
        }
        ModifierNode modifier = this.parseModifier();
        if (modifier != null) {
            ArrayList<ModifierNode> mList = new ArrayList<>();

            while (modifier != null) {
                mList.add(modifier);
                modifier = this.parseModifier();
            }
            MemberDeclNode member = this.parseMemberDecl();
            if (member != null) {
                return new ClassBodyDeclarationNode(s, mList, member);
            } else {
                throw new CompilerException("Expected member declaration after modifiers", this.lexSymbol);
            }
        }
        MemberDeclNode member = this.parseMemberDecl();
        if (member != null) {
            return new ClassBodyDeclarationNode(s, null, member);
        }
        if (this.accept(Sym.STATIC)) {
            BlockNode block = this.parseBlock();
            if (block != null) {
                return new ClassBodyDeclarationNode(s, true, block);
            } else {
                throw new CompilerException("Expected block after static ", this.lexSymbol);
            }
        }
        
        return null;
    }
    
    /*
     * RULE: MethodOrFieldDecl | void Identifier VoidMethodDeclaratorRest |
     * Identifier ConstructorDeclaratorRest | GenericMethodOrConstructorDecl |
     * ClassDeclaration | InterfaceDeclaration
     */
    private MemberDeclNode parseMemberDecl() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        boolean isConstructor = this.lex.lookahead(Sym.IDENTIFIER, Sym.BRACKET_OPEN);//new RuleSwitcher(lexSymbol, lex, Sym.IDENTIFIER, Sym.BRACKET_OPEN).test();

        if (!isConstructor) {
            MethodeOrFieldDeclNode methodOrFieldDeclaration = this.parseMethodeOrFieldDecl();
            if (methodOrFieldDeclaration != null) {
                return new MemberDeclNode(s, methodOrFieldDeclaration);
            }
        }
        if (this.accept(Sym.VOID)) {
            VoidMethodeDeclaratorRestHelperNode voidMethodDeclaratorRest = this.parseVoidMethodeDeclaratorRestHelper();
            if (voidMethodDeclaratorRest != null) {
                return new MemberDeclNode(s, voidMethodDeclaratorRest);
            }
            throw new CompilerException("missing identifier after void", this.lexSymbol);
        }
        ConstructorDeclaratorRestHelperNode constructorRest = this.parseConstructorDeclaratorRestHelper();
        if (constructorRest != null) {
            return new MemberDeclNode(s, constructorRest);
        }
        GenericMethodeOrConstructorDeclNode genericMethodOrConstructorDeclaration = this.parseGenericMethodeOrConstructorDecl();
        if (genericMethodOrConstructorDeclaration != null) {
            return new MemberDeclNode(s, genericMethodOrConstructorDeclaration);
        }
        ClassDeclarationNode classDeclaration = this.parseClassDeclaration();
        if (classDeclaration != null) {
            return new MemberDeclNode(s, classDeclaration);
        }
        InterfaceDeclarationNode interfaceDeclaration = this.parseInterfaceDeclaration();
        if (interfaceDeclaration != null) {
            return new MemberDeclNode(s, interfaceDeclaration);
        }
        return null;
    }

    /*
     * RULE: "{" BlockStatements"}"
     */
    private BlockNode parseBlock() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.CURLY_OPEN)) {
            BlockStatementsNode blockStatements = this.parseBlockStatements();
            if (blockStatements != null) {
                if (this.accept(Sym.CURLY_CLOSE)) {
                    return new BlockNode(s, blockStatements);
                }
                throw new CompilerException("Missing } after {", this.lexSymbol);
            }
            throw new CompilerException("Missing statements after {", this.lexSymbol);
        }
        return null;
    }

    /*
     * RULE: TypeParameters GenericMethodOrConstructorRest
     */
    private GenericMethodeOrConstructorDeclNode parseGenericMethodeOrConstructorDecl() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        TypeParametersNode typeParameters = this.parseTypeParameters();
        if (typeParameters != null) {
            GenericMethodeOrConstructorRestNode rest = this.parseGenericMethodeOrConstructorRest();
            if (rest != null) {
                return new GenericMethodeOrConstructorDeclNode(s, typeParameters, rest);
            } else {
                throw new CompilerException("Missing generic method or constructor", this.lexSymbol);
            }
        }
        return null;
    }

    /*
     * RULE: Identifier ConstructorDeclaratorRest
     */
    private ConstructorDeclaratorRestHelperNode parseConstructorDeclaratorRestHelper() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.IDENTIFIER)) {
            String ident = this.lastTokenText;
            ConstructorDeclaratorRestNode constructorDeclRestNode = this.parseConstructorDeclaratorRest();
            if (constructorDeclRestNode != null) {
                return new ConstructorDeclaratorRestHelperNode(s, ident, constructorDeclRestNode);
            }
            throw new CompilerException("Missing constructor declaration after identifier", this.lexSymbol);

        }
        return null;
    }
    /*
     * RULE: Identifier VoidMethodDeclaratorRest
     */

    private VoidMethodeDeclaratorRestHelperNode parseVoidMethodeDeclaratorRestHelper() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.IDENTIFIER)) {
            String ident = this.lastTokenText;
            VoidMethodeDeclaratorRestNode voidMethodeDeclRest =
                    this.parseVoidMethodeDeclaratorRest();
            if (voidMethodeDeclRest != null) {
                return new VoidMethodeDeclaratorRestHelperNode(s, ident, voidMethodeDeclRest);
            } else {
                throw new CompilerException("Missing method declaration after void", this.lexSymbol);
            }
        }
        return null;
    }
    /*
     * RULE: Type Identifier MethodOrFieldRest
     */

    private MethodeOrFieldDeclNode parseMethodeOrFieldDecl() throws IOException, CompilerException {
        TypeNode tn = this.parseType();
        if (tn != null) {
            Symbol s = this.lexSymbol;
            if (this.accept(Sym.IDENTIFIER)) {
                String ident = this.lastTokenText;
                MethodeOrFieldRestNode mofrn = this.parseMethodeOrFieldRest();
                if (mofrn != null) {
                    return new MethodeOrFieldDeclNode(s, tn, ident, mofrn);
                }
                throw new CompilerException("Error after identifier", this.lexSymbol);
            }
            throw new CompilerException("Missing identifier after type", this.lexSymbol);
        }
        return null;
    }

    /*
     * Rule: FieldDeclaratorsRest | MethodDeclaratorRest
     */
    private MethodeOrFieldRestNode parseMethodeOrFieldRest() throws IOException, CompilerException {
        boolean isMethode = this.lex.lookahead(Sym.BRACKET_OPEN);//new RuleSwitcher(lexSymbol, lex, Sym.BRACKET_OPEN).test();
        Symbol s = this.lexSymbol;

        if (!isMethode) {
            FieldDeclaratorsRestNode fieldDeclaratorRest = this.parseFieldDeclaratorsRest();
            if (fieldDeclaratorRest != null) {
                if (this.accept(Sym.SEMICOLON)) {
                    return new MethodeOrFieldRestNode(s, fieldDeclaratorRest);
                }
                throw new CompilerException("Missing ; after field declarators", this.lexSymbol);
            }
        }
        MethodeDeclaratorRestNode methodDeclaratorRest = this.parseMethodeDeclaratorRest();
        if (methodDeclaratorRest != null) {
            return new MethodeOrFieldRestNode(s, methodDeclaratorRest);
        }
        return null;
    }

    /*
     * Rule: VariableDeclaratorRest { "," VariableDeclaratorRest }
     */
    private FieldDeclaratorsRestNode parseFieldDeclaratorsRest() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        VariableDeclaratorRestNode varDecl = this.parseVariableDeclaratorRest();
        ArrayList<VariableDeclaratorNode> varDecList = new ArrayList<>();
        while (accept(Sym.COMMA)) {
            VariableDeclaratorNode varDec = this.parseVariableDeclarator();
            if (varDec != null) {
                varDecList.add(varDec);
            } else {
                throw new CompilerException("Missing variable declarator after ,", this.lexSymbol);
            }
        }
        return new FieldDeclaratorsRestNode(s, varDecl, varDecList);
    }

    /*
     *  FormalParameters {"[" "]"} ["throws" QualifierIdentifierList] (Block | ";")
     */
    private MethodeDeclaratorRestNode parseMethodeDeclaratorRest() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        FormalParametersNode formalParameter = this.parseFormalParameters();
        if (formalParameter == null) {
            return null;
        }
        int edgeClampsCnt = 0;
        while (this.accept(Sym.EDGECLAMPS_OPEN)) {
            if (this.accept(Sym.EDGECLAMPS_CLOSE)) {
                edgeClampsCnt++;
            } else {
                throw new CompilerException("Missing ] after [", this.lexSymbol);
            }
        }
        if(edgeClampsCnt > 0){
            throw new CompilerException("Array declaration not allowed in TeaJay",s);
        }

        if (this.accept(Sym.THROWS)) {
            QualifiedIdentifierListNode qList = this.parseQualifiedIdentifierList();
            if (qList != null) {
                BlockNode block = this.parseBlock();
                if (block != null) {
                    return new MethodeDeclaratorRestNode(s, formalParameter, edgeClampsCnt,
                            qList, block, false);
                }
                if (this.accept(Sym.SEMICOLON)) {
                    return new MethodeDeclaratorRestNode(s, formalParameter, edgeClampsCnt,
                            qList, null, true);
                }
                throw new CompilerException("Missing Block or ; in method declaration", this.lexSymbol);
            } else {
                throw new CompilerException("Missing Qualifier after throws", this.lexSymbol);
            }
        }
        BlockNode block = this.parseBlock();
        if (block != null) {
            return new MethodeDeclaratorRestNode(s, formalParameter, edgeClampsCnt,
                    null, block, false);
        }
        if (this.accept(Sym.SEMICOLON)) {
            return new MethodeDeclaratorRestNode(s, formalParameter, edgeClampsCnt,
                    null, null, true);
        }
        throw new CompilerException("Missing Block or ; in method declaration", this.lexSymbol);
    }
    /*
     * Rule:    {"[" "]"} [ "=" VariableInitializer ] 
     */
    private VariableDeclaratorRestNode parseVariableDeclaratorRest() throws IOException, CompilerException {
        int cnt = 0;
        Symbol s = this.lexSymbol;
        while (this.accept(Sym.EDGECLAMPS_OPEN)) {
            if (this.accept(Sym.EDGECLAMPS_CLOSE)) {
                cnt++;
            } else {
                throw new CompilerException("Missing ] after [", this.lexSymbol);
            }
        }
        if (this.accept(Sym.OP_EQ)) {
            VariableInitializerNode varInit = this.parseVariableInitializer();
            if (varInit != null) {
                return new VariableDeclaratorRestNode(s, cnt, varInit);
            }
            throw new CompilerException("Missing variable initialisizer after = ", this.lexSymbol);
        }
        //if (cnt > 0) {
        return new VariableDeclaratorRestNode(s, cnt);
        //}
        // return null;
    }
    /*
     * Rule: Identifier VariableDeclaratorRest
     */
    private VariableDeclaratorNode parseVariableDeclarator() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.IDENTIFIER)) {
            String ident = this.lastTokenText;
            VariableDeclaratorRestNode varDecRest = this.parseVariableDeclaratorRest();
            if (varDecRest != null) {
                return new VariableDeclaratorNode(s, ident, varDecRest);
            }
            throw new CompilerException("Missing variable initializiation after identifier", this.lexSymbol);
        }
        return null;
    }

    /*
     * Rule: "("  [ FormalParameterDecls ] ")"
     */
    private FormalParametersNode parseFormalParameters() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.BRACKET_OPEN)) {
            FormalParameterDeclsNode formParamDecls = this.parseFormalParameterDecls();
            if (formParamDecls != null) {
                if (this.accept(Sym.BRACKET_CLOSE)) {
                    return new FormalParametersNode(s, formParamDecls);
                }
                throw new CompilerException("Missing ) after formal parameter declarations", this.lexSymbol);
            }
            if (this.accept(Sym.BRACKET_CLOSE)) {
                return new FormalParametersNode(s);
            }
            throw new CompilerException("Missing ( after )", this.lexSymbol);

        }
        return null;
    }

    /*
     * Rule: FormalParameters ["throws" QualifiedIdentifierList ] (Block | ";")
     */
    private VoidMethodeDeclaratorRestNode parseVoidMethodeDeclaratorRest() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        FormalParametersNode formalParameter = this.parseFormalParameters();
        if (formalParameter == null) {
            return null;
        }


        if (this.accept(Sym.THROWS)) {
            QualifiedIdentifierListNode qList = this.parseQualifiedIdentifierList();
            if (qList != null) {
                BlockNode block = this.parseBlock();
                if (block != null) {
                    return new VoidMethodeDeclaratorRestNode(s, formalParameter,
                            qList, block, false);
                }
                if (this.accept(Sym.SEMICOLON)) {
                    return new VoidMethodeDeclaratorRestNode(s, formalParameter,
                            qList, null, true);
                }
                throw new CompilerException("Missing Block or ; in method declaration", this.lexSymbol);
            } else {
                throw new CompilerException("Missing Qualifier after throws", this.lexSymbol);
            }
        }
        BlockNode block = this.parseBlock();
        if (block != null) {
            return new VoidMethodeDeclaratorRestNode(s, formalParameter,
                    null, block, false);
        }
        if (this.accept(Sym.SEMICOLON)) {
            return new VoidMethodeDeclaratorRestNode(s, formalParameter,
                    null, null, true);
        }
        throw new CompilerException("Missing Block or ; in method declaration", this.lexSymbol);
    }
    /*
     * Rule: FormalParameters [throws QualifiedIdentifierList] Block 
     */
     private ConstructorDeclaratorRestNode parseConstructorDeclaratorRest() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        FormalParametersNode formalParameter = this.parseFormalParameters();
        if (formalParameter == null) {
            return null;
        }


        if (this.accept(Sym.THROWS)) {
            QualifiedIdentifierListNode qList = this.parseQualifiedIdentifierList();
            if (qList != null) {
                BlockNode block = this.parseBlock();
                if (block != null) {
                    return new ConstructorDeclaratorRestNode(s, formalParameter, qList, block);
                }

                throw new CompilerException("Missing Block in method declaration", this.lexSymbol);
            } else {
                throw new CompilerException("Missing Qualifier after throws", this.lexSymbol);
            }
        }
        BlockNode block = this.parseBlock();
        if (block != null) {
            return new ConstructorDeclaratorRestNode(s, formalParameter, null, block);
        }
        throw new CompilerException("Missing Block in method declaration", this.lexSymbol);

    }
    /*
     * Rule: (Type | "void") Identifier MethodDeclaratorRest
     *       | Identifier ConstructorDeclaratorRest
     */
    private GenericMethodeOrConstructorRestNode parseGenericMethodeOrConstructorRest() throws IOException, CompilerException {

        if (!this.lex.lookahead(Sym.IDENTIFIER, Sym.BRACKET_OPEN)) {
            TypeNode type = this.parseType();
            if (type != null) {
                Symbol s = this.lexSymbol;
                if (this.accept(Sym.IDENTIFIER)) {
                    String identifier = this.lastTokenText;
                    MethodeDeclaratorRestNode methDeclRest = this.parseMethodeDeclaratorRest();
                    if (methDeclRest != null) {
                        return new GenericMethodeOrConstructorRestNode(s, type, identifier, methDeclRest);
                    }
                    throw new CompilerException("Missing method declaration after identifier", this.lexSymbol);
                }
                throw new CompilerException("Missing identifier after type", this.lexSymbol);

            }
            if (this.accept(Sym.VOID)) {
                Symbol s = this.lexSymbol;
                if (this.accept(Sym.IDENTIFIER)) {
                    String identifier = this.lastTokenText;
                    MethodeDeclaratorRestNode methDeclRest = this.parseMethodeDeclaratorRest();
                    if (methDeclRest != null) {
                        return new GenericMethodeOrConstructorRestNode(s, true, identifier, methDeclRest);
                    }
                    throw new CompilerException("Missing method declaration after identifier", this.lexSymbol);
                }
                throw new CompilerException("Missing identifier after type", this.lexSymbol);
            }
        }
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.IDENTIFIER)) {

            String identifier = this.lastTokenText;
            ConstructorDeclaratorRestNode constDeclRest = this.parseConstructorDeclaratorRest();
            if (constDeclRest != null) {
                return new GenericMethodeOrConstructorRestNode(s, identifier, constDeclRest);
            }
            throw new CompilerException("Missing Constructor declaration after identifier", this.lexSymbol);
        }
        return null;
    }
    /*
     * Rule: ";" | {Modifier} InterfaceMemberDecl
     */
    private InterfaceBodyDeclarationNode parseInterfaceBodyDeclaration() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.SEMICOLON)) {
            return new InterfaceBodyDeclarationNode(s, true);
        }
        ModifierNode modifier = this.parseModifier();
        ArrayList<ModifierNode> modsList = new ArrayList<>();
        while (modifier != null) {
            modsList.add(modifier);
            modifier = this.parseModifier();
        }
        InterfaceMemberDeclNode interfaceMemberDecl = this.parseInterfaceMemberDecl();
        if (interfaceMemberDecl != null) {
            return new InterfaceBodyDeclarationNode(s, modsList, interfaceMemberDecl);
        }
        return null;
    }

    /*
     *   InterfaceMethodOrFieldDecl |
     *   "void" Identifier VoidInterfaceMethodDeclaratorRest |
     *   InterfaceGenericMethodDecl |
     *   ClassDeclaration |
     *   InterfaceDeclaration |
     */
    private InterfaceMemberDeclNode parseInterfaceMemberDecl() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        InterfaceMethodOrFieldDeclNode interfaceMethodOrFieldDecl = this.parseInterfaceMethodOrFieldDecl();
        if (interfaceMethodOrFieldDecl != null) {
            return new InterfaceMemberDeclNode(s, interfaceMethodOrFieldDecl);
        }
        if (this.accept(Sym.VOID)) {

            if (this.accept(Sym.IDENTIFIER)) {
                String ident = this.lastTokenText;
                VoidInterfaceMethodDeclaratorRestNode voidInterfaceMethodDeclaratorRestNode = this.parseVoidInterfaceMethodDeclaratorRest();
                if (voidInterfaceMethodDeclaratorRestNode != null) {
                    return new InterfaceMemberDeclNode(s, ident, voidInterfaceMethodDeclaratorRestNode);
                }
                throw new CompilerException("Missing void method declaration in interface", this.lexSymbol);
            }
            throw new CompilerException("Missing identifier after void", this.lexSymbol);
        }
        InterfaceGenericMethodDeclNode interfaceGenericMethodDeclNode = this.parseInterfaceGenericMethodDecl();
        if (interfaceGenericMethodDeclNode != null) {
            return new InterfaceMemberDeclNode(s, interfaceGenericMethodDeclNode);
        }
        ClassDeclarationNode classDeclaration = this.parseClassDeclaration();
        if (classDeclaration != null) {
            throw new CompilerException("Inner class declaration not allowed in TeaJay",s);
            //return new InterfaceMemberDeclNode(s, classDeclaration);
        }
        InterfaceDeclarationNode interfaceDecl = this.parseInterfaceDeclaration();
        if (interfaceDecl != null) {
            throw new CompilerException("Inner interface declaration not allowed in TeaJay",s);
            //return new InterfaceMemberDeclNode(s, interfaceDecl);
        }
        return null;
    }
    /*
     * Rule: TypeParameters (Type | "void" ) Identifier InterfaceMethodDeclaratorRest
     */
    private InterfaceGenericMethodDeclNode parseInterfaceGenericMethodDecl() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        TypeParametersNode typeParameters = this.parseTypeParameters();

        if (typeParameters != null) {
            TypeNode type = this.parseType();
            if (type == null) {
                if (!this.accept(Sym.VOID)) {
                    throw new CompilerException("Missing void or type after type parameters", this.lexSymbol);
                }
            }
            if (this.accept(Sym.IDENTIFIER)) {
                String ident = this.lastTokenText;
                InterfaceMethodDeclaratorRestNode intMethDeclRest = this.parseInterfaceMethodDeclaratorRest();
                if (intMethDeclRest != null) {
                    if (type != null) {
                        return new InterfaceGenericMethodDeclNode(s, typeParameters, type, ident, intMethDeclRest);
                    }
                    return new InterfaceGenericMethodDeclNode(s, typeParameters, true, ident, intMethDeclRest);
                }
                throw new CompilerException("Missing interface method after type", this.lexSymbol);
            }
            throw new CompilerException("Missing identifier method after type", this.lexSymbol);

        }

        return null;
    }
    /*
     * Rule: FormalParameters ["throws" QualifiedIdentifierList] ";"
     */
    private VoidInterfaceMethodDeclaratorRestNode parseVoidInterfaceMethodDeclaratorRest() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        FormalParametersNode formalParamaters = this.parseFormalParameters();
        if (formalParamaters != null) {

            if (this.accept(Sym.THROWS)) {
                QualifiedIdentifierListNode qList = this.parseQualifiedIdentifierList();
                if (qList != null) {
                    if (this.accept(Sym.SEMICOLON)) {
                        return new VoidInterfaceMethodDeclaratorRestNode(s, formalParamaters, qList);
                    }
                    throw new CompilerException("Missing ; after identifier ", this.lexSymbol);

                }
                throw new CompilerException("Missing identifier after throws ", this.lexSymbol);
            }

            if (this.accept(Sym.SEMICOLON)) {
                return new VoidInterfaceMethodDeclaratorRestNode(s, formalParamaters);
            }
            throw new CompilerException("Missing ; after method declaration", this.lexSymbol);

        }
        return null;
    }

    /*
     * Rule: Type Identifier InterfaceMethodOrFieldRest
     */
    private InterfaceMethodOrFieldDeclNode parseInterfaceMethodOrFieldDecl() throws IOException, CompilerException {
        TypeNode type = this.parseType();
        if (type != null) {
            Symbol s = this.lexSymbol;
            if (this.accept(Sym.IDENTIFIER)) {
                String ident = this.lastTokenText;
                InterfaceMethodOrFieldRestNode interfaceMethOrFieldRest = this.parseInteraceMethodOrFieldRest();
                if (interfaceMethOrFieldRest != null) {
                    return new InterfaceMethodOrFieldDeclNode(s, type, ident, interfaceMethOrFieldRest);
                }

                throw new CompilerException("Missing method or field after identifier", this.lexSymbol);
            }
            throw new CompilerException("Missing identifier after type", this.lexSymbol);
        }
        return null;
    }
    /*
     * Rule: ConstantDeclaratorsRest ";" |
     *       InterfaceMethodDeclaratorRest
     */
    private InterfaceMethodOrFieldRestNode parseInteraceMethodOrFieldRest() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        ConstantDeclaratorsRestNode constantDeclaratorsRest = this.parseConstantDeclaratorsRest();
        if (constantDeclaratorsRest != null) {
            if (this.accept(Sym.SEMICOLON)) {
                throw new CompilerException("Constant declaration in Interface not allowed",s);
                //return new InterfaceMethodOrFieldRestNode(s, constantDeclaratorsRest);
            }
            throw new CompilerException("Missing ; after constant", this.lexSymbol);
        }
        InterfaceMethodDeclaratorRestNode interfaceMethodDeclRest = this.parseInterfaceMethodDeclaratorRest();
        if (interfaceMethodDeclRest != null) {
            return new InterfaceMethodOrFieldRestNode(s, interfaceMethodDeclRest);
        }
        return null;
    }
    /*
     * ConstantDeclaratorRest { "," ConstantDeclarator }
     */
    private ConstantDeclaratorsRestNode parseConstantDeclaratorsRest() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        ConstantDeclaratorRestNode constantDeclaratorRest = this.parseConstantDeclaratorRest();
        if (constantDeclaratorRest == null) {
            return null;
        }
        LinkedList<ConstantDeclaratorNode> lConst = new LinkedList<>();
        while (this.accept(Sym.COMMA)) {
            ConstantDeclaratorNode constant = this.parseConstantDeclarator();
            if (constant != null) {
                lConst.add(constant);
            } else {
                throw new CompilerException("Missing constant after ,", this.lexSymbol);
            }
        }
        return new ConstantDeclaratorsRestNode(s, constantDeclaratorRest, lConst);

    }
    /*
     * Rule: FormalParameters {"[""]"} ["throws" QualifiedIdentifierList] ;
     */
    private InterfaceMethodDeclaratorRestNode parseInterfaceMethodDeclaratorRest() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        FormalParametersNode formalParamaters = this.parseFormalParameters();
        if (formalParamaters != null) {
            int cnt = 0;
            while (this.accept(Sym.EDGECLAMPS_OPEN)) {
                if (this.accept(Sym.EDGECLAMPS_CLOSE)) {
                    cnt++;
                }
            }
            if (this.accept(Sym.THROWS)) {
                QualifiedIdentifierListNode qList = this.parseQualifiedIdentifierList();
                if (qList != null) {
                    if (this.accept(Sym.SEMICOLON)) {
                        return new InterfaceMethodDeclaratorRestNode(s, formalParamaters, cnt, qList);
                    }
                    throw new CompilerException("Missing ; after identifier ", this.lexSymbol);

                }
                throw new CompilerException("Missing identifier after throws ", this.lexSymbol);
            }

            if (this.accept(Sym.SEMICOLON)) {
                return new InterfaceMethodDeclaratorRestNode(s, formalParamaters, cnt);
            }
            if (cnt > 0) {
                throw new CompilerException("Missing ; after [] ", this.lexSymbol);
            }
        }
        return null;
    }
    /*
     * Rule: {"[" "]"} = VariableInitializer
     */
    private ConstantDeclaratorRestNode parseConstantDeclaratorRest() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        int cnt = 0;
        while (this.accept(Sym.EDGECLAMPS_OPEN)) {
            if (this.accept(Sym.EDGECLAMPS_CLOSE)) {
                cnt++;
            } else {
                throw new CompilerException("Missing ] after [", this.lexSymbol);
            }
        }
        if (this.accept(Sym.OP_EQ)) {
            VariableInitializerNode varInit = this.parseVariableInitializer();
            if (varInit != null) {
                return new ConstantDeclaratorRestNode(s, cnt, varInit);
            }
            throw new CompilerException("Missing variable initalisation after = ", this.lexSymbol);
        }
        if (cnt > 0) {
            throw new CompilerException("Missing = after [] ", this.lexSymbol);
        }
        return null;
    }

    /*
     * Rule: Identifier ConstantDeclaratorRest
     */
    private ConstantDeclaratorNode parseConstantDeclarator() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.IDENTIFIER)) {
            String ident = this.lastTokenText;
            ConstantDeclaratorRestNode constantDeclRest = this.parseConstantDeclaratorRest();
            if (constantDeclRest != null) {
                return new ConstantDeclaratorNode(s, ident, constantDeclRest);
            }
            throw new CompilerException("Missing constant after identifier", this.lexSymbol);

        }
        return null;
    }
    /*
     * Rule: ArrayInitializer | Expression
     */
    private VariableInitializerNode parseVariableInitializer() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        ArrayInitializerNode arrayInit = this.parseArrayInitializer();
        if (arrayInit != null) {
            return new VariableInitializerNode(s, arrayInit);
        }
        ExpressionNode expression = this.parseExpression();
        if (expression != null) {
            return new VariableInitializerNode(s, expression);
        }
        return null;
    }
    /*
     * Rule: { VariableModifier } Type FormalParameterDeclsRest
     */
    private FormalParameterDeclsNode parseFormalParameterDecls() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        VariableModifierNode varMods = this.parseVariableModifier();
        LinkedList<VariableModifierNode> lVarMod = new LinkedList<>();
        while (varMods != null) {
            lVarMod.add(varMods);
            varMods = this.parseVariableModifier();
        }
        TypeNode type = this.parseType();
        if (type != null) {
            FormalParameterDeclsRestNode formParameterDecls = this.parseFormalParameterDeclsRest();
            if (formParameterDecls != null) {
                if (lVarMod.size() > 0) {
                    return new FormalParameterDeclsNode(s, lVarMod, type, formParameterDecls);
                }
                return new FormalParameterDeclsNode(s, type, formParameterDecls);
            }
            throw new CompilerException("Missing formal parameters declarations after type", this.lexSymbol);
        }
        if (lVarMod.size() > 0) {
            throw new CompilerException("Missing type after variable modifier", this.lexSymbol);
        }
        return null;
    }
    /*
     * Rule: "finale" | Annotation
     */
    private VariableModifierNode parseVariableModifier() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.FINAL)) {
            throw new CompilerException("Variable modifier not supported yet", s);
            //return new VariableModifierNode(s, true);
        }

        AnnotationNode annotation = this.parseAnnotation();
        if (annotation != null) {
            throw new CompilerException("Variable modifier not supported yet", s);
            //return new VariableModifierNode(annotation);
        }
        return null;
    }

    /*
     * Rule: VariableDeclaratorId [ "," FormalParameterDecls ]
     *      | "..." VariableDeclaratorId
     */
    private FormalParameterDeclsRestNode parseFormalParameterDeclsRest() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        VariableDeclaratorIdNode varDecId = this.parseVariableDeclaratorId();
        if (varDecId != null) {
            if (this.accept(Sym.COMMA)) {
                FormalParameterDeclsNode formalParameterDecls = this.parseFormalParameterDecls();
                if (formalParameterDecls != null) {
                    return new FormalParameterDeclsRestNode(s, varDecId, formalParameterDecls);
                }
                throw new CompilerException("Missing fromal parameter declarations after ,", this.lexSymbol);
            }
            return new FormalParameterDeclsRestNode(s, varDecId);
        }
        if (this.accept(Sym.TRIPLEDOT)) {
            VariableDeclaratorIdNode varDecTripleId = this.parseVariableDeclaratorId();
            if (varDecTripleId != null) {
                return new FormalParameterDeclsRestNode(s, true, varDecTripleId);
            }
            throw new CompilerException("Missing vaiable declarator id after ...", this.lexSymbol);
        }
        return null;
    }
    
    /*
     * Rule: Identifier {"[" "]"}
     */
    private VariableDeclaratorIdNode parseVariableDeclaratorId() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.IDENTIFIER)) {
            int cnt = 0;
            String ident = this.lastTokenText;
            while (this.accept(Sym.EDGECLAMPS_OPEN)) {
                if (this.accept(Sym.EDGECLAMPS_CLOSE)) {
                    cnt++;
                } else {
                    throw new CompilerException("Missing ] after [", this.lexSymbol);
                }
            }
            return new VariableDeclaratorIdNode(s, ident, cnt);
        }
        return null;
    }
    /*
     * Rule: VariableDeclarator { "," VariableDeclarator }
     */
    private VariableDeclaratorsNode parseVariableDeclarators() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        VariableDeclaratorNode varDec = this.parseVariableDeclarator();
        if (varDec != null) {
            LinkedList<VariableDeclaratorNode> vList = new LinkedList<>();
            while (this.accept(Sym.COMMA)) {
                VariableDeclaratorNode v = this.parseVariableDeclarator();
                if (v != null) {
                    vList.add(v);
                } else {
                    throw new CompilerException("Missing variable declarator after ,", this.lexSymbol);
                }
            }
            if (vList.size() > 0) {
                return new VariableDeclaratorsNode(s, varDec, vList);
            }
            return new VariableDeclaratorsNode(s, varDec);
        }
        return null;
    }

    /*
     * Rule: { [ VariableInitializer { "," VariableInitializer } [","] ] }
     */
    private ArrayInitializerNode parseArrayInitializer() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.CURLY_OPEN)) {
            VariableInitializerNode varInit = this.parseVariableInitializer();
            if (varInit != null) {
                LinkedList<VariableInitializerNode> vList = new LinkedList<>();
                boolean hasComma = false;
                while (this.accept(Sym.COMMA)) {
                    VariableInitializerNode v = this.parseVariableInitializer();
                    if (v != null) {
                        vList.add(v);
                        continue;
                    }

                    if (this.accept(Sym.CURLY_CLOSE)) {
                        hasComma = true;
                        break;
                    } else {
                        throw new CompilerException("Missing variable initializer after ,", this.lexSymbol);
                    }
                }
                if (vList.size() > 0) {
                    if (!hasComma) {
                        if (this.accept(Sym.CURLY_CLOSE)) {
                            return new ArrayInitializerNode(s, varInit, vList);
                        }
                    } else {
                        return new ArrayInitializerNode(s, varInit, vList);
                    }
                    // throw new CompilerException("Missing } after variable initializer", this.lexSymbol);
                }
                if (vList.size() == 0) {
                    if (!hasComma) {
                        if (this.accept(Sym.CURLY_CLOSE)) {
                            return new ArrayInitializerNode(s, varInit);
                        }
                        throw new CompilerException("Missing } after variable init", this.lexSymbol);
                    } else {
                        return new ArrayInitializerNode(s, varInit, hasComma);
                    }
                }

            }
            if (this.accept(Sym.COMMA)) {
                if (this.accept(Sym.CURLY_CLOSE)) {
                    return new ArrayInitializerNode(s);
                }
            }
            if (this.accept(Sym.CURLY_CLOSE)) {
                return new ArrayInitializerNode(s);
            }
            throw new CompilerException("Missing } after {", this.lexSymbol);
        }
        return null;
    }
    /*
     *  Expression1 [AssignmentOperator Expression1]
     */
    private ExpressionNode parseExpression() throws CompilerException, IOException {
        Symbol s = lexSymbol;
        Expression1Node exp1 = this.parseExpression1();
        if (exp1 != null) {
            AssignmentOperatorNode assignOp = this.parseAssignmentOperator();
            if (assignOp != null) {
                Expression1Node assignExp = this.parseExpression1();
                if (assignExp != null) {
                    return new ExpressionNode(exp1, assignOp, assignExp);
                }
                throw new CompilerException("Missing expression after assignment operator", this.lexSymbol);
            }
            return new ExpressionNode(s, exp1);
        }
        return null;
    }

    /*
     * private ExpressionNode parseExpression() throws IOException,
     * CompilerException { Expression1Node exp1 = this.parseExpression1();
     * if(exp1 != null){ AssignmentOperatorNode assignmentOp =
     * this.parseAssignmentOperator(); if(assignmentOp != null){ Expression1Node
     * e1 = this.parseExpression1(); if(e1 != null){ return new
     * ExpressionNode(exp1, assignmentOp, e1); } throw new
     * CompilerException("Missing expression after assignment operator",
     * "Message from parseExpression",2); } return new ExpressionNode(exp1); }
     * return null; }
     */
//    private ExpressionNode parseExpression() throws IOException, CompilerException {
//        PrefixOpNode prefix = this.parsePrefixOp();
//        if (prefix != null) {
//            ExpressionNode exp = this.parseExpression();
//            if (exp != null) {
//                ExpressionHelperNode expHelp = this.parseExpressionHelper();
//                return new ExpressionNode(prefix, exp, expHelp);
//            }
//            throw new CompilerException("Missing expression after prefix",this.lexSymbol);
//        }
//
//        /*
//         * boolean isTypeExp1 = new RuleSwitcher(lexSymbol, lex,
//         * Sym.IDENTIFIER,Sym.OP_LESS,Sym.IDENTIFIER,Sym.COMMA).test(); boolean
//         * isTypeExp2 = new RuleSwitcher(lexSymbol, lex,
//         * Sym.IDENTIFIER,Sym.OP_LESS,Sym.IDENTIFIER,Sym.OP_GREATER).test();
//         * if(isTypeExp1 || isTypeExp2 ){ TypeNode type = this.parseType(); if
//         * (type != null) { ExpressionNode exp = this.parseExpression(); if (exp
//         * != null) { ExpressionHelperNode expHelp =
//         * this.parseExpressionHelper();
//         *
//         * return new ExpressionNode(type, exp, expHelp); } throw new
//         * CompilerException("Missing expression after type", "Message from
//         * parseExpressionNode", 2); } }
//         */
//
//
//        PrimaryNode primary = this.parsePrimary();
//        if (primary != null) {
//            SelectorNode sIter = this.parseSelector();
//            LinkedList<SelectorNode> sList = new LinkedList<>();
//            while (sIter != null) {
//                sList.add(sIter);
//                sIter = this.parseSelector();
//            }
//            PostfixOpNode pIter = this.parsePostfixOp();
//            LinkedList<PostfixOpNode> pList = new LinkedList<>();
//
//            while (pIter != null) {
//                pList.add(pIter);
//                pIter = this.parsePostfixOp();
//            }
//
//
//            Expression2RestNode exp2Rest = this.parseExpression2Rest();
//            Expression1RestNode exp1Rest = this.parseExpression1Rest();
//            AssignmentOperatorNode assign = this.parseAssignmentOperator();
//            if (assign == null) {
//                ExpressionHelperNode expHelp = this.parseExpressionHelper();
//                return new ExpressionNode(primary, sList, pList, exp2Rest, exp1Rest, expHelp);
//            }
//            ExpressionNode expAssign = this.parseExpression();
//            if (expAssign != null) {
//                Expression1RestNode exp1RestExpAssign = this.parseExpression1Rest();
//                ExpressionHelperNode expHelp = this.parseExpressionHelper();
//                return new ExpressionNode(primary, sList, pList, exp2Rest, exp1Rest, assign, expAssign, exp1RestExpAssign, expHelp);
//
//            }
//            throw new CompilerException("Missing expression after assign operator",this.lexSymbol);
//        }
//        return null;
//    }
    
    /*
     *  Rule: { BlockStatements }
     */
    private BlockStatementsNode parseBlockStatements() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        BlockStatementNode blockStatement = this.parseBlockStatement();
        LinkedList<BlockStatementNode> bList = new LinkedList<>();
        while (blockStatement != null) {
            bList.add(blockStatement);
            blockStatement = this.parseBlockStatement();
        }
        if (bList.size() > 0) {
            return new BlockStatementsNode(s, bList);
        }
        return new BlockStatementsNode(s);
    }

    /*
     *  
     *   
     * JavaRule:
     *   LocalVariableDeclarationStatement
     *   ClassOrInterfaceDeclaration
     *   [Identifier :] Statement
     * 
     * Rule: LocalVariableDeclarationStatement
     *        | ClassOrInterfaceDeclaration
     *        | Statement
     *
     */
    private BlockStatementNode parseBlockStatement() throws IOException, CompilerException {
        /*
         * Linksfaktorisierung mit ein wenig heckmäck. Nicht schön da die VariableModiferNode
         * in parseClassOrInterfaceDeclartion kopiert wird.
         */
        Symbol s = this.lexSymbol;
        VariableModifierNode varMod = this.parseVariableModifier();
        LinkedList<VariableModifierNode> vList = new LinkedList<>();
        while (varMod != null) {
            vList.add(varMod);
            varMod = this.parseVariableModifier();
        }
        if (!this.lex.lookahead(Sym.CLASS)) {
            LocalVariableDeclarationsStatementNode localVarDecStatement = this.parseLocalVariableDeclarationsStatement(vList);
            if (localVarDecStatement != null) {
                return new BlockStatementNode(s, localVarDecStatement);
            }
        }

        ClassOrInterfaceDeclaratioNode classOrInterfaceDec =
                this.parseClassOrInterfaceDeclaration(vList);
        if (classOrInterfaceDec != null) {
            throw new CompilerException("Inner class or interface declaration not allowed in TeaJay", s);
            //return new BlockStatementNode(s, classOrInterfaceDec);
        }

        StatementNode statement = this.parseStatement();
        if (statement != null) {
            return new BlockStatementNode(s, statement);
        }
        return null;
    }

    /*
     * Rule: Block |
     *       ";" |
     *        IdentifierDoublePointStatement |
     *        StatementExpressionStatement | 
     *        IfStatement |
     *        AssertStatement assert Expression [: Expression] ; |
     *        SwitchStatement switch ParExpression { SwitchBlockStatementGroups }|
     *        WhileStatement while ParExpression Statement |
     *        DoStatement do Statement while ParExpression ; |
     *        ForStatement for ( ForControl ) Statement |
     *        BreakStatement    break  ; |
     *        ContinueStatement continue  ; |
     *        ReturnStatement return [Expression] ; |
     *        ThrowStatement throw Expression ; |
     *        SynchronizedStatement synchronized ParExpression Block |
     *        TryStatement |
     *        TypeSwitchStatememt
     */
    private StatementNode parseStatement() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        BlockNode block = this.parseBlock();
        if (block != null) {
            return new StatementNode(s, block);
        }
        if (this.accept(Sym.SEMICOLON)) {
            return new StatementNode(s, true);
        }
        boolean isIdentColonStatement = this.lex.lookahead(Sym.IDENTIFIER, Sym.COLON);//new RuleSwitcher(lexSymbol, lex, Sym.IDENTIFIER, Sym.COLON).test();
        if (isIdentColonStatement) {
            throw new CompilerException("Labels not supported", this.lexSymbol);
            
//            IdentDoublePointStatementNode identDoublePointStatement = this.parseIdentDoublePointStatement();
//            if (identDoublePointStatement != null) {
//                return new StatementNode(s, identDoublePointStatement);
//            }
        }

        IfStatementNode ifStatement = this.parseIfStatement();
        if (ifStatement != null) {
            return new StatementNode(s, ifStatement);
        }
        AssertStatementNode assertStatement = this.parseAssertStatement();
        if (assertStatement != null) {
            if (this.accept(Sym.SEMICOLON)) {
                return new StatementNode(s, assertStatement);
            }
            throw new CompilerException("Missing ; after assert statement", this.lexSymbol);
        }
        SwitchStatementNode switchStatement = this.parseSwitchStatement();
        if (switchStatement != null) {
            throw new CompilerException("Switch statement in TeaJay not supported, use if statement",s);
            //return new StatementNode(s, switchStatement);
        }
        WhileStatementNode whileStatement = this.parseWhileStatement();
        if (whileStatement != null) {
            return new StatementNode(s, whileStatement);
        }
        DoStatementNode doStatement = this.parseDoStatement();
        if (doStatement != null) {
            return new StatementNode(s, doStatement);
        }
        ForStatementNode forStatement = this.parseForStatement();
        if (forStatement != null) {
            return new StatementNode(s, forStatement);
        }
        BreakStatementNode breakStatement = this.parseBreakStatement();
        if (breakStatement != null) {
            return new StatementNode(s, breakStatement);
        }
        ContinueStatementNode continueStatement = this.parseContinueStatement();
        if (continueStatement != null) {
            return new StatementNode(s, continueStatement);
        }
        ReturnStatementNode returnStatement = this.parseReturnStatement();
        if (returnStatement != null) {
            return new StatementNode(s, returnStatement);
        }
        ThrowStatementNode throwStatement = this.parseThrowStatement();
        if (throwStatement != null) {
            return new StatementNode(s, throwStatement);
        }
        SynchronizedStatementNode synchronizedStatement = this.parseSynchronizedStatement();
        if (synchronizedStatement != null) {
            throw new CompilerException("Synchronized statement not supoorted yet", s);
            //return new StatementNode(s, synchronizedStatement);
        }
        TryStatementNode tryStatement = this.parseTryStatement();
        if (tryStatement != null) {
            return new StatementNode(s, tryStatement);
        }

        StatementExpressionNode statementExpression = this.parseStatementExpression();
        if (statementExpression != null) {
            if (this.accept(Sym.SEMICOLON)) {
                return new StatementNode(s, statementExpression);
            }
            throw new CompilerException("Missing ; after statement", this.lexSymbol);
        }
        TypeSwitchStatementNode typeSwitchStatement = this.parseTypeSwitchStatement();
        if (typeSwitchStatement != null) {
            return new StatementNode(s, typeSwitchStatement);
        }
        return null;
    }
    /*
     * Rule: { VariableModifier } Type VariableDeclarators ";"
     */
    private LocalVariableDeclarationsStatementNode parseLocalVariableDeclarationsStatement(LinkedList<VariableModifierNode> vList) throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.lexSymbol.sym != Sym.CLOSUREDEC) {
           
            boolean isQualifiedVarDec = this.lex.lookahead(new TypeRule());
            if (!isQualifiedVarDec) {
                return null;
            }
        }

        TypeNode type = this.parseType();
        if (type != null) {

            VariableDeclaratorsNode varDecs = this.parseVariableDeclarators();
            if (varDecs != null) {

                if (this.accept(Sym.SEMICOLON)) {
                    return new LocalVariableDeclarationsStatementNode(s, vList, type, varDecs);
                }
                throw new CompilerException("Missing ; after variable declarations", this.lexSymbol);
            }
            throw new CompilerException("Missing variable declaration after type", this.lexSymbol);
        }
        if (vList.size() > 0) {
            throw new CompilerException("Missing type after modifier", this.lexSymbol);
        }
        return null;
    }

    private IdentDoublePointStatementNode parseIdentDoublePointStatement() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.IDENTIFIER)) {
            String ident = this.lastTokenText;
            if (this.accept(Sym.COLON)) {
                StatementNode statement = this.parseStatement();
                if (statement != null) {
                    return new IdentDoublePointStatementNode(s, ident, statement);
                }
                throw new CompilerException("Missing statement after :", this.lexSymbol);
            }
            throw new CompilerException("Missing : after identifier", this.lexSymbol);
        }
        return null;
    }
    /*
     * Rule: Expression ";"
     */
    private StatementExpressionNode parseStatementExpression() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        ExpressionNode expression = this.parseExpression();
        if (expression != null) {
            return new StatementExpressionNode(s, expression);
        }
        return null;
    }
    /*
     * Rule: if Block | else (Block | if Block)
     */
    private IfStatementNode parseIfStatement() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.IF)) {
            ParExpressionNode parExpression = this.parseParExpression();
            if (parExpression != null) {
                BlockNode block = this.parseBlock();
                if (block != null) {
                    if (this.accept(Sym.ELSE)) {
                        BlockNode eBlock = this.parseBlock();
                        if (eBlock != null) {
                            return new IfStatementNode(s, parExpression, block, eBlock);
                        }
                        /* Fix to support the } else if (...) { Syntax.
                         * This Code creates a dummy Block containing only the next if statement.
                         * For the Grammar:
                         * ... } "else" Block | } ... "else" IfStatement
                         */
                        IfStatementNode elseif = this.parseIfStatement();
                        if (elseif != null) {
                            StatementNode elseifStatement = new StatementNode(s, elseif);
                            BlockStatementNode dummyBlockStatement = new BlockStatementNode(s, elseifStatement);
                            LinkedList<BlockStatementNode> lst = new LinkedList<>();
                            lst.add(dummyBlockStatement);
                            BlockStatementsNode dummyStatements = new BlockStatementsNode(s, lst);
                            BlockNode dummyBlock = new BlockNode(s, dummyStatements);
                            return new IfStatementNode(s, parExpression, block, dummyBlock);
                        }
                        throw new CompilerException("Missing block after else", this.lexSymbol);
                    }
                    return new IfStatementNode(s, parExpression, block);
                }
                throw new CompilerException("Missing block after ( expression )", this.lexSymbol);
            }
            throw new CompilerException("Missing ( expression ) after if", this.lexSymbol);
        }
        return null;
    }
    /*
     * Rule:  try ( Block | ResourceSpecification Block ) (Catches [ Finally] | Finally)
     *          
     */
    private TryStatementNode parseTryStatement() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.TRY)) {
            BlockNode block = this.parseBlock();
            if (block != null) {
                CatchesNode catches = this.parseCatches();
                if (catches != null) {
                    FinallyNode f = this.parseFinally();
                    if (f != null) {
                        return new TryStatementNode(s, block, catches, f);
                    }
                    return new TryStatementNode(s, block, catches);
                }
                FinallyNode f = this.parseFinally();
                if (f != null) {
                    return new TryStatementNode(s, block, f);
                }
                throw new CompilerException("Missing catches or finally after try", this.lexSymbol);
            }
            ResourceSpecificationNode resourceSpec = this.parseResourceSpecification();
            if (resourceSpec != null) {
                block = this.parseBlock();
                if (block != null) {
                    CatchesNode catches = this.parseCatches();
                    if (catches != null) {
                        FinallyNode f = this.parseFinally();
                        if (f != null) {
                            throw new CompilerException("finally not supported yet");
                            //return new TryStatementNode(s, resourceSpec, catches, f);
                        }
                        throw new CompilerException("Try with resource not supported yet");
                        //return new TryStatementNode(s, resourceSpec, catches);
                    }
                    FinallyNode f = this.parseFinally();
                    if (f != null) {
                        throw new CompilerException("finally not supported yet");
                        //return new TryStatementNode(s, resourceSpec, f);
                    }
                    return new TryStatementNode(s, resourceSpec);
                }
                throw new CompilerException("Missing block after resource specification", this.lexSymbol);
            }
            throw new CompilerException("Missing block or resource specification", this.lexSymbol);

        }
        return null;
    }
    /*
     * Rule: "synchronized" ParExpression Block
     */
    private SynchronizedStatementNode parseSynchronizedStatement() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.SYNCHRONIZED)) {
            ParExpressionNode parExpression = this.parseParExpression();
            if (parExpression != null) {
                BlockNode block = this.parseBlock();
                if (block != null) {
                    return new SynchronizedStatementNode(s, parExpression, block);
                }
                throw new CompilerException("Missing block after synchronized( expression )", this.lexSymbol);
            }
            throw new CompilerException("Missing ( expression ) after synchronized", this.lexSymbol);
        }
        return null;
    }
    /*
     *  Rule: "throw" Expression ";"
     */
    private ThrowStatementNode parseThrowStatement() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.THROW)) {
            ExpressionNode expression = this.parseExpression();
            if (expression != null) {
                if (this.accept(Sym.SEMICOLON)) {
                    return new ThrowStatementNode(s, expression);
                }
                throw new CompilerException("Missing ; after throw expression", this.lexSymbol);
            }
            throw new CompilerException("Missing expression after throw", this.lexSymbol);
        }
        return null;
    }
    /*
     * Rule: "return" [Expression]
     */
    private ReturnStatementNode parseReturnStatement() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.RETURN)) {
            ExpressionNode expression = this.parseExpression();
            if (expression != null) {
                if (this.accept(Sym.SEMICOLON)) {
                    return new ReturnStatementNode(s, expression);
                }
                throw new CompilerException("Missing ; after return expression", this.lexSymbol);
            }
            if (this.accept(Sym.SEMICOLON)) {
                return new ReturnStatementNode(s);
            }
            throw new CompilerException("Missing ; after return", this.lexSymbol);
        }
        return null;
    }
    /*
     * continue [identifier]";"
     */
    private ContinueStatementNode parseContinueStatement() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.CONTINUE)) {

            if (this.accept(Sym.IDENTIFIER)) {
                String ident = this.lastTokenText;
                if (this.accept(Sym.SEMICOLON)) {
                    throw new CompilerException("Labeled continue statement not supported",s);
                    //return new ContinueStatementNode(s, ident);
                }
                throw new CompilerException("Missing ; after continue identifier", this.lexSymbol);
            }
            if (this.accept(Sym.SEMICOLON)) {
                return new ContinueStatementNode(s);
            }
            throw new CompilerException("Missing ; after continue", this.lexSymbol);
        }
        return null;
    }

    /*
     * break [identifier]";"
     */
    private BreakStatementNode parseBreakStatement() throws IOException, CompilerException {
        if (this.accept(Sym.BREAK)) {
            Symbol s = this.lexSymbol;
            if (this.accept(Sym.IDENTIFIER)) {
                String ident = this.lastTokenText;
                if (this.accept(Sym.SEMICOLON)) {
                    throw new CompilerException("Labeled break statement not supported",s);
                    //return new BreakStatementNode(s, ident);
                }
                throw new CompilerException("Missing ; after break identifier", this.lexSymbol);
            }
            if (this.accept(Sym.SEMICOLON)) {
                return new BreakStatementNode(s);
            }
            throw new CompilerException("Missing ; after break", this.lexSymbol);
        }
        return null;
    }

    /*
     *  "for" "(" ForControl")" Block
     */
    private ForStatementNode parseForStatement() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.FOR)) {
            if (this.accept(Sym.BRACKET_OPEN)) {
                ForControlNode forControl = this.parseForControl();
                if (forControl != null) {
                    if (this.accept(Sym.BRACKET_CLOSE)) {
                        BlockNode block = this.parseBlock();
                        if (block != null) {
                            return new ForStatementNode(s, forControl, block);
                        }
                        throw new CompilerException("Missing statement after for control", this.lexSymbol);
                    }
                    throw new CompilerException("Missing ) after for control", this.lexSymbol);
                }
                throw new CompilerException("Missing for control after for(", this.lexSymbol);
            }
            throw new CompilerException("Missing ( after for", this.lexSymbol);
        }
        return null;
    }
    /*
     * Rule: "do" block "while" ParExpression ";"
     */
    private DoStatementNode parseDoStatement() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.DO)) {
            BlockNode block = this.parseBlock();
            if (block != null) {
                if (this.accept(Sym.WHILE)) {
                    ParExpressionNode parExpression = this.parseParExpression();
                    if (parExpression != null) {
                        if (this.accept(Sym.SEMICOLON)) {
                            return new DoStatementNode(s, block, parExpression);
                        }
                        throw new CompilerException("Missing ; after (expression) ", this.lexSymbol);
                    }
                    throw new CompilerException("Missing ( expression ) after while", this.lexSymbol);
                }
                throw new CompilerException("Missing while after statements", this.lexSymbol);
            }
            throw new CompilerException("Missing statement after do", this.lexSymbol);
        }
        return null;
    }
    /*
     * Rule "while" ParExpression Block
     */
    private WhileStatementNode parseWhileStatement() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.WHILE)) {
            ParExpressionNode parExpression = this.parseParExpression();
            if (parExpression != null) {
                BlockNode block = this.parseBlock();
                if (block != null) {
                    return new WhileStatementNode(s, parExpression, block);
                }
                throw new CompilerException("Missing block after while( expression )", this.lexSymbol);
            }
            throw new CompilerException("Missing ( expression ) after while", this.lexSymbol);
        }
        return null;
    }
    /*
     * Rule: "assert" Expression [":" Expression] ;
     */
    private AssertStatementNode parseAssertStatement() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.ASSERT)) {
            ExpressionNode expression = this.parseExpression();
            if (expression != null) {
                if (this.accept(Sym.COLON)) {
                    ExpressionNode e = this.parseExpression();
                    if (e != null) {
                        
                        return new AssertStatementNode(s, expression, e);
                    }
                    throw new CompilerException("Missing expression after :", this.lexSymbol);
                }
                
                return new AssertStatementNode(s, expression);
            }
            throw new CompilerException("Missing expression after assert", this.lexSymbol);
        }
        return null;
    }
    /*
     * Rule "switch" ParExpression "{" {switchBlockStatementGroups} "}"
     */
    private SwitchStatementNode parseSwitchStatement() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.SWITCH)) {
            ParExpressionNode parExpression = this.parseParExpression();
            if (parExpression != null) {
                if (this.accept(Sym.CURLY_OPEN)) {
                    SwitchBlockStatementGroupsNode switchBlockStatementGroups = this.parseSwitchBlockStatementGroups();
                    LinkedList<SwitchBlockStatementGroupsNode> list = new LinkedList<>();
                    while (switchBlockStatementGroups != null) {
                        list.add(switchBlockStatementGroups);
                        switchBlockStatementGroups = this.parseSwitchBlockStatementGroups();
                    }
                    if (this.accept(Sym.CURLY_CLOSE)) {
                        return new SwitchStatementNode(s, parExpression, list);
                    }
                    throw new CompilerException("Missing } after { in switch statement", this.lexSymbol);


                }
                throw new CompilerException("Missing {  after swicth expression", this.lexSymbol);
            }

            throw new CompilerException("Missing (expression) after swicth", this.lexSymbol);
        }
        return null;
    }
    /*
     * Rule: "(" Expression ")"
     */
    private ParExpressionNode parseParExpression() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.BRACKET_OPEN)) {
            ExpressionNode exp = this.parseExpression();
            if (exp != null) {
                if (this.accept(Sym.BRACKET_CLOSE)) {
                    return new ParExpressionNode(s, exp);
                }
                throw new CompilerException("Missing ) after ( expression ", this.lexSymbol);
            }
            throw new CompilerException("Missing expression after (", this.lexSymbol);

        }
        return null;
    }
    /*
     * Rule: 
     */
    private SwitchBlockStatementGroupsNode parseSwitchBlockStatementGroups() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        SwitchBlockStatementGroupNode iter = this.parseSwitchBlockStatementGroup();
        LinkedList<SwitchBlockStatementGroupNode> lSwitch = new LinkedList<>();
        while (iter != null) {
            lSwitch.add(iter);
            iter = this.parseSwitchBlockStatementGroup();
        }
        if (lSwitch.size() > 0) {
            return new SwitchBlockStatementGroupsNode(s, lSwitch);
        }
        return null;
    }

    /*
     *  CatchClause {CatchClause}
     */
    private CatchesNode parseCatches() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        CatchClauseNode catchClause = this.parseCatchClause();
        if (catchClause != null) {
            LinkedList<CatchClauseNode> lCatch = new LinkedList<>();
            CatchClauseNode iter = this.parseCatchClause();
            while (iter != null) {
                lCatch.add(iter);
                iter = this.parseCatchClause();
            }
            if (lCatch.size() > 0) {

                return new CatchesNode(s, catchClause, lCatch);
            }
            return new CatchesNode(s, catchClause);
        }
        return null;

    }
    /*
     *  Rule: "(" Resources [";"] ")"
     */
    private ResourceSpecificationNode parseResourceSpecification() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.BRACKET_OPEN)) {
            ResourcesNode resource = this.parseResources();
            if (resource != null) {
                if (this.accept(Sym.SEMICOLON)) {
                    if (this.accept(Sym.BRACKET_CLOSE)) {
                        return new ResourceSpecificationNode(s, resource, true);
                    }
                    throw new CompilerException("Missing ) after ( resource ", this.lexSymbol);
                }
                if (this.accept(Sym.BRACKET_CLOSE)) {
                    return new ResourceSpecificationNode(s, resource);
                }
                throw new CompilerException("Missing ) after ( resource ", this.lexSymbol);
            }
            throw new CompilerException("Missing resource after (", this.lexSymbol);
        }
        return null;
    }
    /*
     * Rule "finally Block
     */
    private FinallyNode parseFinally() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.FINALLY)) {
            BlockNode block = this.parseBlock();
            if (block != null) {
                return new FinallyNode(s, block);
            }
            throw new CompilerException("Missing block after finally", this.lexSymbol);
        }
        return null;
    }
    /*
     * Rule: ForVarControl
     *        | ForInit ; [Expression] ; [ForUpdate]
     *  
     * The second alternative rule is used for a better error message. TeaJay don't support for-loops
     * like for(int i = 0; i < 100; i++)
     */
    private ForControlNode parseForControl() throws IOException, CompilerException {
        // boolean isExpression = new RuleSwitcher(lexSymbol, lex, Sym.IDENTIFIER, Sym.OP_EQ).test();
        Symbol s = this.lexSymbol;
        boolean isForVarControlNode = this.lex.lookahead(new ForVarControlRule());
        if (isForVarControlNode) {
            ForVarControlNode forVarControl = this.parseForVarControl();

            if (forVarControl != null) {
                return new ForControlNode(s, forVarControl);
            }
        }
        ForInitNode forInit = this.parseForInit();
        if (forInit != null) {
            if (this.accept(Sym.SEMICOLON)) {
                ExpressionNode expression = this.parseExpression();
                if (expression != null) {
                    if (this.accept(Sym.SEMICOLON)) {
                        ForUpdateNode forUpdate = this.parseForUpdate();
                        if (forUpdate != null) {
                            return new ForControlNode(s, forInit, expression, forUpdate);
                        }
                        return new ForControlNode(s, forInit, expression);
                    }
                    throw new CompilerException("Missing ; after expression", this.lexSymbol);
                }
                if (this.accept(Sym.SEMICOLON)) {
                    ForUpdateNode forUpdate = this.parseForUpdate();
                    if (forUpdate != null) {
                        return new ForControlNode(s, forInit, forUpdate);
                    }
                    return new ForControlNode(s, forInit);
                }
                throw new CompilerException("Missing ; after empty expression", this.lexSymbol);
            }
            throw new CompilerException("Missing ; afte for init", this.lexSymbol);
        }
        if (this.accept(Sym.SEMICOLON)) {
            ExpressionNode expression = this.parseExpression();
            if (!this.accept(Sym.SEMICOLON)) {
                throw new CompilerException("Missing ; after expression in for loop", this.lexSymbol);
            }
            ForUpdateNode forUpdate = this.parseForUpdate();

            return new ForControlNode(s, null, expression, forUpdate);

        }
        return null;
    }
    /*
     * Rule: "catch" "(" {VariableModifier } CatchType Identifier ")" Block
     *
     */
    private CatchClauseNode parseCatchClause() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.CATCH)) {
            if (this.accept(Sym.BRACKET_OPEN)) {
                VariableModifierNode varMod = this.parseVariableModifier();
                LinkedList<VariableModifierNode> varList = new LinkedList<>();
                while (varMod != null) {
                    varList.add(varMod);
                    varMod = this.parseVariableModifier();
                }
                CatchTypeNode catchType = this.parseCatchType();
                if (catchType != null) {
                    if (this.accept(Sym.IDENTIFIER)) {
                        String ident = this.lastTokenText;
                        if (this.accept(Sym.BRACKET_CLOSE)) {
                            BlockNode block = this.parseBlock();
                            if (block != null) {
                                if (varList.size() > 0) {
                                    return new CatchClauseNode(s, varList, catchType, ident, block);
                                }
                                return new CatchClauseNode(s, catchType, ident, block);
                            }
                            throw new CompilerException("Missing block after catch(clause)", this.lexSymbol);
                        }
                        throw new CompilerException("Missing ) after identifier ", this.lexSymbol);
                    }
                    throw new CompilerException("Missing idetifier after catch", this.lexSymbol);
                }
                throw new CompilerException("Missing catch type in catch ", this.lexSymbol);
            }
            throw new CompilerException("Missing ( after catch ", this.lexSymbol);
        }
        return null;
    }
    /*
     * Rule: Identifier { "|" Identifier }
     * Multicatches not supported
     */
    private CatchTypeNode parseCatchType() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        QualifiedIdentifierNode ident = this.parseQualifiedIdentfier();
        if (ident != null) {

            LinkedList<QualifiedIdentifierNode> identList = new LinkedList<>();
            QualifiedIdentifierNode iter;
            while (this.accept(Sym.OP_OR)) {
                iter = this.parseQualifiedIdentfier();
                if (iter != null) {
                    identList.add(iter);
                } else {
                    throw new CompilerException("Missing identifier after |", this.lexSymbol);
                }
            }
            if (identList.size() > 0) {
                throw new CompilerException("Multicatch not supported yet", this.lexSymbol);
                //return new CatchTypeNode(s, ident, identList);
            }

            return new CatchTypeNode(s, ident, null);

        }
        return null;
    }
    /*
     * Rule: Resource { ";" Resource }
     */
    private ResourcesNode parseResources() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        ResourceNode resource = this.parseResource();
        LinkedList<ResourceNode> rList = new LinkedList<>();
        ResourceNode r;

        //Semantic only: semicolon is not a  ressource
        while (this.accept(Sym.SEMICOLON)) {
            r = this.parseResource();
            if (r != null) {
                rList.add(r);
            } else {
                break;
            }
        }
        if (resource != null) {
            if (rList.size() > 0) {
                return new ResourcesNode(s, resource, rList);
            }
            return new ResourcesNode(s, resource, null);
        }
        return null;
    }
    /*
     * Rule: {VariableModifier} ReferenceType VariableDeclaratorId "=" Expression
     */
    private ResourceNode parseResource() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        VariableModifierNode varMod = this.parseVariableModifier();
        LinkedList<VariableModifierNode> varList = new LinkedList<>();
        while (varMod != null) {
            varList.add(varMod);
            varMod = this.parseVariableModifier();
        }
        ReferenceTypeNode referenceType = this.parseReferenceType();
        if (referenceType != null) {
            VariableDeclaratorIdNode varDecId = this.parseVariableDeclaratorId();
            if (varDecId != null) {
                if (this.accept(Sym.OP_EQ)) {
                    ExpressionNode expression = this.parseExpression();
                    if (expression != null) {
                        if (varList.size() > 0) {
                            return new ResourceNode(s, varList, referenceType, varDecId, expression);
                        }
                        return new ResourceNode(s, null, referenceType, varDecId, expression);
                    }
                    throw new CompilerException("Missing expression after =", this.lexSymbol);
                }
                throw new CompilerException("Missing = after variale declarator id", this.lexSymbol);
            }
            throw new CompilerException("Missing variale declarator id after reference type", this.lexSymbol);
        }

        if (varList.size() > 0) {
            throw new CompilerException("Missing reference type after variable modifier", this.lexSymbol);
        }
        return null;
    }

    /*
     * Rule: SwitchLabels BlockStatements
     */
    private SwitchBlockStatementGroupNode parseSwitchBlockStatementGroup() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        SwitchLabelsNode switchLabels = this.parseSwitchLabels();
        if (switchLabels != null) {
            BlockStatementsNode blockStatements = this.parseBlockStatements();
            if (blockStatements != null) {
                return new SwitchBlockStatementGroupNode(s, switchLabels, blockStatements);
            }
            throw new CompilerException("Missing block statement after switch labels", this.lexSymbol);
        }
        return null;
    }
    /*
     * Rule: SwitchLabel { SwitchLabel }
     */
    private SwitchLabelsNode parseSwitchLabels() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        SwitchLabelNode switchLabel = this.parseSwitchLabel();
        if (switchLabel != null) {
            SwitchLabelNode iter = this.parseSwitchLabel();
            LinkedList<SwitchLabelNode> sLabelList = new LinkedList<>();
            while (iter != null) {
                sLabelList.add(iter);
                iter = this.parseSwitchLabel();
            }
            if (sLabelList.size() > 0) {
                return new SwitchLabelsNode(s, iter, sLabelList);
            }
            return new SwitchLabelsNode(s, iter, null);
        }
        return null;
    }
    /*
     * Rule: "case" Expression ":"
     *       "case" EnumConstantName ":"
     *       | "default" ":" 
     */
    private SwitchLabelNode parseSwitchLabel() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.CASE)) {
            ExpressionNode expression = this.parseExpression();
            if (expression != null) {
                if (this.accept(Sym.COLON)) {
                    return new SwitchLabelNode(s, expression);
                }
                throw new CompilerException("Missing : after expression", this.lexSymbol);
            }
            EnumConstantNameNode enumConstName = this.parseEnumConstantName();
            if (enumConstName != null) {
                if (this.accept(Sym.COLON)) {
                    return new SwitchLabelNode(s, enumConstName);
                }
                throw new CompilerException("Missing : after enum constant name", this.lexSymbol);
            }
            throw new CompilerException("Missing expressio or enum constant name after case", this.lexSymbol);
        }
        if (this.accept(Sym.DEFAULT)) {
            if (this.accept(Sym.COLON)) {
                return new SwitchLabelNode(s, true);
            }
            throw new CompilerException("Missing : after default", this.lexSymbol);
        }
        return null;
    }
    /*
     * Rule: Identifier
     */
    private EnumConstantNameNode parseEnumConstantName() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.IDENTIFIER)) {
            String ident = this.lastTokenText;
            return new EnumConstantNameNode(s, ident);
        }
        return null;
    }
    /*
     * Rule: { VariableModifier } Type VariableDeclaratorId ForVarControlRest 
     */
    private ForVarControlNode parseForVarControl() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        VariableModifierNode iter = this.parseVariableModifier();
        LinkedList<VariableModifierNode> vList = new LinkedList<>();
        while (iter != null) {
            vList.add(iter);
            iter = this.parseVariableModifier();
        }
        TypeNode type = this.parseType();
        if (type != null) {
            VariableDeclaratorIdNode varDecId = this.parseVariableDeclaratorId();
            if (varDecId != null) {
                ForVarControlRestNode forVarControlRest = this.parseForVarControlRest();
                if (forVarControlRest != null) {
                    if (vList.size() > 0) {
                        return new ForVarControlNode(s, vList, type, varDecId, forVarControlRest);
                    }
                    return new ForVarControlNode(s, null, type, varDecId, forVarControlRest);
                }
                throw new CompilerException("Missing for variable control rest after variable declarator id", this.lexSymbol);
            }
            throw new CompilerException("Missing variable delcarator id after type", this.lexSymbol);
        }
        if (vList.size() > 0) {
            throw new CompilerException("Missing type after variable modifier", this.lexSymbol);
        }
        return null;
    }
    /*
     * Rule: StatementExpression { "," StatementExpression }
     */
    private ForInitNode parseForInit() throws IOException, CompilerException {
        Symbol sym = this.lexSymbol;
        StatementExpressionNode statementExpression = this.parseStatementExpression();
        if (statementExpression != null) {
            LinkedList<StatementExpressionNode> statementList = new LinkedList<>();
            while (this.accept(Sym.COMMA)) {
                StatementExpressionNode s = this.parseStatementExpression();
                if (s != null) {
                    statementList.add(s);
                } else {
                    throw new CompilerException("Missing statement expression after ,", this.lexSymbol);
                }
            }
            if (statementList.size() > 0) {
                return new ForInitNode(sym, statementExpression, statementList);
            }
            return new ForInitNode(sym, statementExpression, null);
        }
        return null;
    }
    /*
     * Rule: StatementExpression { "," StatementExpression }
     */
    private ForUpdateNode parseForUpdate() throws IOException, CompilerException {
        Symbol sym = this.lexSymbol;
        StatementExpressionNode statementExpression = this.parseStatementExpression();
        if (statementExpression != null) {
            LinkedList<StatementExpressionNode> statementList = new LinkedList<>();
            while (this.accept(Sym.COMMA)) {
                StatementExpressionNode s = this.parseStatementExpression();
                if (s != null) {
                    statementList.add(s);
                } else {
                    throw new CompilerException("Missing statement expression after ,", this.lexSymbol);
                }
            }
            if (statementList.size() > 0) {
                return new ForUpdateNode(sym, statementExpression, statementList);
            }
            return new ForUpdateNode(sym, statementExpression, null);
        }
        return null;
    }
    /*
     * Rule: ":" Expression |
     *        ForVariableDeclaratorsRest ; [Expression] ; [ForUpdate]
     *  The second rule is not supported
     */
    private ForVarControlRestNode parseForVarControlRest() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        boolean isColon = this.lex.lookahead(Sym.COLON);//new RuleSwitcher(lexSymbol, lex, Sym.COLON).test();
        if (!isColon) {
                if(false){
                    ForVariableDeclaratorsRestNode forVarDecRest = this.parseForVariableDelcaratorRest();
                if (forVarDecRest != null) {
                    if (this.accept(Sym.SEMICOLON)) {
                        ExpressionNode expression = this.parseExpression();
                        if (expression != null) {
                            if (this.accept(Sym.SEMICOLON)) {
                                ForUpdateNode forUpdate = this.parseForUpdate();
                                if (forUpdate != null) {
                                    return new ForVarControlRestNode(s, forVarDecRest, expression, forUpdate);
                                }
                                return new ForVarControlRestNode(s, forVarDecRest, expression);
                            }
                        }
                        if (this.accept(Sym.SEMICOLON)) {
                            ForUpdateNode forUpdate = this.parseForUpdate();
                            if (forUpdate != null) {
                                return new ForVarControlRestNode(s, forVarDecRest, forUpdate);
                            }
                            return new ForVarControlRestNode(s, forVarDecRest);
                        }
                    }
                    throw new CompilerException("Missing ; after for variable declarator", this.lexSymbol);
                }
            }else{
                    throw new CompilerException("TeaJay don't support for-loop use while",s);
            }
        }
        if (this.accept(Sym.COLON)) {
            ExpressionNode expression = this.parseExpression();
            if (expression != null) {
                return new ForVarControlRestNode(s, expression);
            }
            throw new CompilerException("Missing expression after :", this.lexSymbol);
        }
        return null;
    }
    /*
     * Rule: [ = VariableInitializer ] { , VariableDeclarator }
     */
    private ForVariableDeclaratorsRestNode parseForVariableDelcaratorRest() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.OP_EQ)) {
            VariableInitializerNode varInit = this.parseVariableInitializer();
            if (varInit != null) {
                LinkedList<VariableDeclaratorNode> vList = new LinkedList<>();
                while (this.accept(Sym.COMMA)) {
                    VariableDeclaratorNode varDec = this.parseVariableDeclarator();
                    if (varDec != null) {
                        vList.add(varDec);
                    } else {
                        throw new CompilerException("Missing variable declarator after ,", this.lexSymbol);
                    }
                }
                if (vList.size() > 0) {
                    return new ForVariableDeclaratorsRestNode(s, varInit, vList);
                }
                return new ForVariableDeclaratorsRestNode(s, varInit);
            }
            throw new CompilerException("Missing variable initializer after =", this.lexSymbol);
        }
        LinkedList<VariableDeclaratorNode> vList = new LinkedList<>();
        while (this.accept(Sym.COMMA)) {
            VariableDeclaratorNode varDec = this.parseVariableDeclarator();
            if (varDec != null) {
                vList.add(varDec);
            } else {
                throw new CompilerException("Missing variable declarator after ,", this.lexSymbol);
            }
        }
        if (vList.size() > 0) {
            return new ForVariableDeclaratorsRestNode(s, vList);
        }
        return new ForVariableDeclaratorsRestNode(s);
    }
    /*
     *  "=" | "+=" | "-=" | "*=" | "/="
     */
    private AssignmentOperatorNode parseAssignmentOperator() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.OP_EQ)) {
            return new AssignmentOperatorNode(s, AssignmentOperatorNode.Operators.SET);
        }
        if (this.accept(Sym.OP_PLUS_SET)) {
            return new AssignmentOperatorNode(s, AssignmentOperatorNode.Operators.ADD_SET);
        }
        if (this.accept(Sym.OP_SUB_SET)) {
            return new AssignmentOperatorNode(s, AssignmentOperatorNode.Operators.SUB_SET);
        }
        if (this.accept(Sym.OP_MULT_SET)) {
            return new AssignmentOperatorNode(s, AssignmentOperatorNode.Operators.MULT_SET);
        }
        if (this.accept(Sym.OP_DIV_SET)) {
            return new AssignmentOperatorNode(s, AssignmentOperatorNode.Operators.DIV_SET);
        }
        if (this.accept(Sym.OP_AND_SET)) {
            return new AssignmentOperatorNode(s, AssignmentOperatorNode.Operators.AND_SET);
        }
        if (this.accept(Sym.OP_OR_SET)) {
            return new AssignmentOperatorNode(s, AssignmentOperatorNode.Operators.OR_SET);
        }
        if (this.accept(Sym.OP_XOR_SET)) {
            return new AssignmentOperatorNode(s, AssignmentOperatorNode.Operators.INV_SET);
        }
        if (this.accept(Sym.OP_MOD_SET)) {
            return new AssignmentOperatorNode(s, AssignmentOperatorNode.Operators.MOD_SET);
        }
        if (this.accept(Sym.OP_LEFT_SHIFT_SET)) {
            return new AssignmentOperatorNode(s, AssignmentOperatorNode.Operators.LEFT_SHIFT_SET);
        }
        if (this.accept(Sym.OP_RIGHT_SHIFT_SET)) {
            return new AssignmentOperatorNode(s, AssignmentOperatorNode.Operators.RIGHT_SHIFT_SET);
        }
        if (this.accept(Sym.OP_RIGHT_RIGHT_SHIFT_SET)) {
            return new AssignmentOperatorNode(s, AssignmentOperatorNode.Operators.DOUBLE_RIGHT_SHIFT_SET);
        }

        return null;
    }
    /*
     * Rule: Expression3 [Expression2Rest]
     */
    private Expression2Node parseExpression2() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        Expression3Node exp3 = this.parseExpression3();
        if (exp3 != null) {
            Expression2RestNode exp2Rest = this.parseExpression2Rest();
            if (exp2Rest != null) {
                return new Expression2Node(s, exp3, exp2Rest);
            }
            return new Expression2Node(s, exp3);
        }
        return null;
    }
    /*
     * Rule: "?" Expression ":" Expression
     */
    private Expression1RestNode parseExpression1Rest() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.QUESTIONMARK)) {
            ExpressionNode expression = this.parseExpression();
            if (expression != null) {
                if (this.accept(Sym.COLON)) {
                    ExpressionNode exp1 = this.parseExpression();
                    if (exp1 != null) {
                        return new Expression1RestNode(s, expression, exp1);
                    }
                    throw new CompilerException("Missing expression after :", this.lexSymbol);
                }
                throw new CompilerException("Missing : after expression ", this.lexSymbol);
            }
            throw new CompilerException("Missing expression after ?", this.lexSymbol);
        }
        return null;
    }

    /*
     *
     *ET1 = expression3 [expression2Rest] [expression1Rest] [AssignmentOperator expression1] ET1 
     *      | epsilon;
     * 
     */
    private Et1Node parseEt1() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        Expression3Node exp3 = this.parseExpression3();
        if (exp3 == null) {
            return new Et1Node();
        }
        Expression2RestNode exp2Rest = this.parseExpression2Rest();
        Expression1RestNode exp1Rest = this.parseExpression1Rest();
        AssignmentOperatorNode assignOp = this.parseAssignmentOperator();
        Expression1Node exp1 = null;
        if (assignOp != null) {
            exp1 = this.parseExpression1();
            if (exp1 == null) {
                throw new CompilerException("Missing expression after assign operator", lexSymbol);
            }
        }
        Et1Node et1 = this.parseEt1();
        return new Et1Node(s, exp3, exp2Rest, exp1Rest, assignOp, exp1, et1);

    }

    /**
     * PrefixOp expression3| ( PrefixOp expression3 [expression2Rest]
     * [expression1Rest] [AssignmentOperator expression1] ET1 | Primary
     * {Selector} {PostfixOp} [expression2Rest] [expression1Rest]
     * [AssignmentOperator expression1] ET1 expression3 | Type ET1 ) expression3
     * | Primary {Selector} {PostfixOp} .
     *
     * @return
     * @throws IOException
     * @throws CompilerException
     */
    private Expression3Node parseExpression3() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        PrefixOpNode prefixOp = this.parsePrefixOp();
        if (prefixOp != null) {
            Expression3Node exp = this.parseExpression3();
            if (exp != null) {
                Expression2RestNode exp2Rest = this.parseExpression2Rest();
                Expression1RestNode exp1Rest = this.parseExpression1Rest();
                AssignmentOperatorNode assignOp = this.parseAssignmentOperator();
                Expression1Node assignExp1 = this.parseExpression1();
                if (assignOp != null) {

                    if (assignExp1 == null) {
                        throw new CompilerException("Missing expression after assignment operator", this.lexSymbol);
                    }
                }
                Et1Node et1Expression = null;//this.parseEt1();
                Expression3Node exp3 = this.parseExpression3();
                return new Expression3Node(s, prefixOp, exp, exp2Rest, exp1Rest, assignOp, assignExp1, et1Expression, exp3);
            }
            throw new CompilerException("Missing expression after prefix", this.lexSymbol);
        }


        //boolean isType = this.lex.lookahead(new CastRule());
        //if(!isType){
            /*
         * Primary {Selector} {PostfixOp} [expression2Rest] [expression1Rest] 
         *                  [AssignmentOperator expression1] ET1 expression3
         * | Primary {Selector} {PostfixOp}
         */

        PrimaryNode primary = this.parsePrimary();
        if (primary != null) {
            LinkedList<SelectorNode> lSelector = new LinkedList<>();
            SelectorNode selector = this.parseSelector();
            while (selector != null) {
                lSelector.add(selector);
                selector = this.parseSelector();
            }
            LinkedList<PostfixOpNode> lPostfixOp = new LinkedList<>();
            PostfixOpNode postfixOp = this.parsePostfixOp();
            while (postfixOp != null) {
                lPostfixOp.add(postfixOp);
                postfixOp = this.parsePostfixOp();
            }
            Expression2RestNode exp2Rest = this.parseExpression2Rest();
            Expression1RestNode exp1Rest = this.parseExpression1Rest();
            AssignmentOperatorNode assignmentOperator = this.parseAssignmentOperator();
            Expression1Node exp1 = null;
            if (assignmentOperator != null) {
                exp1 = this.parseExpression1();
                if (exp1 == null) {
                    throw new CompilerException("Missing expression after assignment operator", this.lexSymbol);
                }
            }
            Et1Node et1 = null;//this.parseEt1();
            Expression3Node exp3 = null;//this.parseExpression3();
            return new Expression3Node(s, primary, lSelector, lPostfixOp, exp2Rest, exp1Rest, assignmentOperator, exp1, et1, exp3);
        }


        return null;
    }
    /*
     * Rule: InfixOp Expression3 | instanceof Type
     */
    private Expression2RestNode parseExpression2Rest() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        LinkedList<Expression2RestNode.InfixOpExpression3Node> iList = new LinkedList<>();
        InfixOpNode infix = this.parseInfixOp();
        while (infix != null) {
            Expression3Node exp3 = this.parseExpression3();
            if (exp3 != null) {
                iList.add(new Expression2RestNode.InfixOpExpression3Node(this.lexSymbol, infix, exp3));
                infix = this.parseInfixOp();
            } else {
                throw new CompilerException("Missing expression after infix", this.lexSymbol);
            }
        }
        if (iList.size() > 0) {

            return new Expression2RestNode(s, iList);
        }
        if (this.accept(Sym.INSTANCEOF)) {
            TypeNode type = this.parseType();
            if (type != null) {
                return new Expression2RestNode(s, type);
            }
            throw new CompilerException("Missing type after instance", this.lexSymbol);
        }
        return null;
    }

    private InfixOpNode parseInfixOp() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.OR)) {
            return new InfixOpNode(s, InfixOpNode.InfixOps.OR);
        }
        if (this.accept(Sym.AND)) {
            return new InfixOpNode(s, InfixOpNode.InfixOps.AND);
        }
        if (this.accept(Sym.OP_OR)) {
            return new InfixOpNode(s, InfixOpNode.InfixOps.BIT_OR);
        }
        if (this.accept(Sym.OP_XOR)) {
            return new InfixOpNode(s, InfixOpNode.InfixOps.BIT_XOR);
        }
        if (this.accept(Sym.OP_AND)) {
            return new InfixOpNode(s, InfixOpNode.InfixOps.BIT_AND);
        }
        if (this.accept(Sym.OP_EQEQ)) {
            return new InfixOpNode(s, InfixOpNode.InfixOps.EQL);
        }
        if (this.accept(Sym.OP_NEQ)) {
            return new InfixOpNode(s, InfixOpNode.InfixOps.NEQL);
        }
        if (this.accept(Sym.OP_LESS)) {
            return new InfixOpNode(s, InfixOpNode.InfixOps.LESS);
        }
        if (this.accept(Sym.OP_GREATER)) {
            return new InfixOpNode(s, InfixOpNode.InfixOps.GREATER);
        }
        if (this.accept(Sym.OP_LEQ)) {
            return new InfixOpNode(s, InfixOpNode.InfixOps.LESS_EQUAL);
        }
        if (this.accept(Sym.OP_GEQ)) {
            return new InfixOpNode(s, InfixOpNode.InfixOps.GREATER_EQUAL);
        }
        if (this.accept(Sym.OP_LEFT_SHIFT)) {
            return new InfixOpNode(s, InfixOpNode.InfixOps.LEFT_SHIFT);
        }
        if (this.accept(Sym.OP_RIGHT_SHIFT)) {
            return new InfixOpNode(s, InfixOpNode.InfixOps.RIGHT_SHIFT);
        }
        if (this.accept(Sym.OP_RIGHT_RIGHT_SHIFT)) {
            return new InfixOpNode(s, InfixOpNode.InfixOps.DOUBLE_RIGHT_SHIFT);
        }
        if (this.accept(Sym.OP_PLUS)) {
            return new InfixOpNode(s, InfixOpNode.InfixOps.ADD);
        }
        if (this.accept(Sym.OP_SUB)) {
            return new InfixOpNode(s, InfixOpNode.InfixOps.SUB);
        }
        if (this.accept(Sym.OP_MULT)) {
            return new InfixOpNode(s, InfixOpNode.InfixOps.MULT);
        }
        if (this.accept(Sym.OP_DIV)) {
            return new InfixOpNode(s, InfixOpNode.InfixOps.DIV);
        }
        if (this.accept(Sym.OP_MOD)) {
            return new InfixOpNode(s, InfixOpNode.InfixOps.MOD);
        }
        return null;
    }

    private PrefixOpNode parsePrefixOp() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.OP_PLUS_PLUS)) {
            throw new CompilerException("Operator ++ not supported", s);
            //return new PrefixOpNode(s, PrefixOpNode.PrefixOps.ADD_ADD);
        }
        if (this.accept(Sym.OP_SUB_SUB)) {
            throw new CompilerException("Operator -- not supported", s);
            // return new PrefixOpNode(s, PrefixOpNode.PrefixOps.SUB_SUB);
        }
        if (this.accept(Sym.OP_NOT)) {

            return new PrefixOpNode(s, PrefixOpNode.PrefixOps.NOT);
        }
        if (this.accept(Sym.OP_NEG)) {
            throw new CompilerException("Operator ~ not supported", s);
            //return new PrefixOpNode(s, PrefixOpNode.PrefixOps.INVERT);
        }
        if (this.accept(Sym.OP_PLUS)) {
            return new PrefixOpNode(s, PrefixOpNode.PrefixOps.ADD);
        }
        if (this.accept(Sym.OP_SUB)) {
            return new PrefixOpNode(s, PrefixOpNode.PrefixOps.SUB);
        }
        return null;
    }
    /*
     * Rule:    Literal 
     *          | ( ParExpression | "(" Type ")" Expression ) 
     *          | ClosureDef ClosureCreator | "this" [Arguments]
     *          | "super" SuperSuffix 
     *          | "new" Creator 
     *          | "void" "." "class"
     *          | "Identifier" { "." Identifier } [IdentifierSuffix]
     *          | BasicType { "[" "]"}.class
     */
    private PrimaryNode parsePrimary() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        LiteralNode literal = this.parseLiteral();
        if (literal != null) {
            return new PrimaryNode(s, literal);
        }
        //Check for cast Expression;
        if (this.lex.lookahead(new CastRule())) {
            if (this.accept(Sym.BRACKET_OPEN)) {
                TypeNode type = this.parseType();
                if (type != null) {
                    if (this.accept(Sym.BRACKET_CLOSE)) {
                        ExpressionNode exp = this.parseExpression();
                        if (exp != null) {
                            return new PrimaryNode(s, exp, type);
                        } else {
                            //-------------------------------------------
                            //XXX XXX: Partially fix wrong cast detection
                            //-------------------------------------------
                            ReferenceTypeNode referenceType = type.getReferenceType();
                            if (referenceType != null && referenceType.getTypeArguments() == null) {
                                LinkedList<String> idents = new LinkedList<>();
                                for (DotIdentifierTypeArgumentsHelperNode di : referenceType.getDotIdentfierTypeArgumenstList()) {
                                    if (di.getTypeArguments() != null) {
                                        throw new CompilerException("Missing expression after cast", lexSymbol);
                                    }
                                    idents.add(di.getIdentifier());
                                }
                                return new PrimaryNode(s, referenceType.getName(), idents);
                            }
                            //END XXX XXX
                        }
                        throw new CompilerException("Missing expression after cast", lexSymbol);
                    }
                    throw new CompilerException("Missing ) after ( in cast expression", lexSymbol);
                }
                throw new CompilerException("Missing type after ( in cast expression", lexSymbol);
            }
        } else {
            ParExpressionNode parExpression = this.parseParExpression();
            if (parExpression != null) {
                return new PrimaryNode(s, parExpression);
            }
        }
        /*TeaJay Closure def*/

        if (this.accept(Sym.CLOSUREDEF)) {
            ClosureCreatorNode closureCreator = this.parseClosureCreator();
            if (closureCreator != null) {
                return new PrimaryNode(s, closureCreator);
            }
            throw new CompilerException("Missing closure definiation after closure ", s);
        }
        if (this.accept(Sym.THIS)) {
            ArgumentsNode arg = this.parseArguments();
            if (arg != null) {
                return new PrimaryNode(s, arg);
            }
            return new PrimaryNode(s);
        }
        if (this.accept(Sym.SUPER)) {
            if (this.lex.lookahead(Sym.DOT, Sym.OP_LESS)) {
                return new PrimaryNode(s, PrimaryNode.ThisOrSuper.Super);
            }
            SuperSuffixNode superSuffix = this.parseSuperSuffix();
            if (superSuffix != null) {
                return new PrimaryNode(s, superSuffix);
            }
            throw new CompilerException("Missing super suffix after super", this.lexSymbol);
        }
        if (this.accept(Sym.NEW)) {
            CreatorNode creator = this.parseCreator();
            if (creator != null) {
                return new PrimaryNode(s, creator);
            }
            throw new CompilerException("Missing creator after new", this.lexSymbol);
        }
        //Not supported
        NonWildcardTypeArgumentsNode nonWildcardTypeArgs = this.parseNonWildcardTypeArguments();
        if (nonWildcardTypeArgs != null) {
            ExplicitGenericInvocationSuffixNode expGenInvocSuffix = this.parseExplicitGenericInvocationSuffix();
            if (expGenInvocSuffix != null) {
                return new PrimaryNode(s, nonWildcardTypeArgs, expGenInvocSuffix);
            }
            if (this.accept(Sym.THIS)) {
                ArgumentsNode args = this.parseArguments();
                if (args != null) {
                    return new PrimaryNode(s, nonWildcardTypeArgs, args);
                }
                throw new CompilerException("Missing arguments after non wildcard type arguments", this.lexSymbol);
            }
            throw new CompilerException("Missing explicit gerneric invocation suffix or this arguments after non wildcard type arguments", this.lexSymbol);
        }

        if (this.accept(Sym.IDENTIFIER)) {
            String firstIdent = this.lastTokenText;
            LinkedList<String> iList = new LinkedList<>();
            if (this.lex.lookahead(Sym.DOT, Sym.IDENTIFIER)) {
                while (this.accept(Sym.DOT)) {
                    if (this.accept(Sym.IDENTIFIER)) {
                        String ident = this.lastTokenText;
                        iList.add(ident);
                    }
                    if (!this.lex.lookahead(Sym.DOT, Sym.IDENTIFIER)) {
                        break;
                    }
                }
            }
            if (this.lex.lookahead(Sym.DOT, Sym.SUPER, Sym.DOT)) {
                return new PrimaryNode(s, firstIdent, iList, null);
            }
            IdentifierSuffixNode identSuffix = this.parseIdentifierSuffix();
            return new PrimaryNode(s, firstIdent, iList, identSuffix);

        }

        BasicTypeNode basicType = this.parseBasicType();
        if (basicType != null) {
            int cnt = 0;
            while (this.accept(Sym.EDGECLAMPS_OPEN)) {
                if (this.accept(Sym.EDGECLAMPS_CLOSE)) {
                    cnt++;
                } else {
                    throw new CompilerException("Missing ] after [", this.lexSymbol);
                }
            }
            if (this.accept(Sym.DOT)) {
                if (this.accept(Sym.CLASS)) {
                    return new PrimaryNode(s, basicType, cnt, true);
                }
                throw new CompilerException("Missing class after basic type . ", this.lexSymbol);
            }
            return new PrimaryNode(s, basicType, cnt);
        }
        if (this.accept(Sym.VOID)) {
            if (this.accept(Sym.DOT)) {
                if (this.accept(Sym.CLASS)) {
                    return new PrimaryNode(s, true);
                }
                throw new CompilerException("Missing .class after void", this.lexSymbol);
            }
        }
        return null;
    }

    private SelectorNode parseSelector() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.DOT)) {

            if (this.accept(Sym.IDENTIFIER)) {
                String ident = this.lastTokenText;
                ArgumentsNode args = this.parseArguments();
                if (args != null) {
                    return new SelectorNode(s, ident, args);
                }
                return new SelectorNode(s, ident);
            }
            ExplicitGenericInvocationNode explicitInvoc = this.parseExplicitGenericInvocation();
            if (explicitInvoc != null) {
                return new SelectorNode(s, explicitInvoc);
            }
            if (this.accept(Sym.THIS)) {
                return new SelectorNode(s, SelectorNode.ThisOrSuper.This);
            }
            if (this.accept(Sym.SUPER)) {
                if (this.lex.lookahead(Sym.DOT, Sym.OP_LESS)) {
                    return new SelectorNode(s, SelectorNode.ThisOrSuper.Super);
                }
                SuperSuffixNode superSuffix = this.parseSuperSuffix();
                if (superSuffix != null) {
                    return new SelectorNode(s, superSuffix);
                }

                throw new CompilerException("Missing super suffix after super", this.lexSymbol);
            }
            if (this.accept(Sym.NEW)) {
                NonWildcardTypeArgumentsNode nWildcarsTypeArgs = this.parseNonWildcardTypeArguments();
                if (nWildcarsTypeArgs != null) {
                    InnerCreatorNode innerCreator = this.parseInnerCreator();
                    if (innerCreator != null) {
                        return new SelectorNode(s, nWildcarsTypeArgs, innerCreator);
                    }
                    throw new CompilerException("Missing inner creator after new", this.lexSymbol);
                }
                InnerCreatorNode innerCreator = this.parseInnerCreator();
                if (innerCreator != null) {
                    return new SelectorNode(s, innerCreator);
                }
                throw new CompilerException("Missing inner creator after new", this.lexSymbol);
            }
            throw new CompilerException("Missing this, super... after .", this.lexSymbol);
        }
        if (this.accept(Sym.EDGECLAMPS_OPEN)) {
            ExpressionNode exp = this.parseExpression();
            if (exp != null) {
                if (this.accept(Sym.EDGECLAMPS_CLOSE)) {
                    return new SelectorNode(s, exp);
                }
                throw new CompilerException("Missing ] after [ expression", this.lexSymbol);
            }
            throw new CompilerException("Missing expression after [", this.lexSymbol);
        }
        ArgumentsNode arguments = parseArguments();
        if (arguments != null) {
            return new SelectorNode(s, arguments);
        }
        return null;
    }

    private PostfixOpNode parsePostfixOp() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.OP_PLUS_PLUS)) {
            throw new CompilerException("Operator ++ not supported", s);
            //return new PostfixOpNode(s, PostfixOpNode.PostfixOps.ADD_ADD);
        }
        if (this.accept(Sym.OP_SUB_SUB)) {
            throw new CompilerException("Operator -- not supported", s);

            // return new PostfixOpNode(s, PostfixOpNode.PostfixOps.SUB_SUB);
        }
        return null;
    }
    //EDIT

    private LiteralNode parseLiteral() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.INT_L)) {
            return new LiteralNode(s, LiteralType.INT, this.lastTokenText);
        }
        if (this.accept(Sym.LONG_L)) {
            return new LiteralNode(s, LiteralType.LONG, this.lastTokenText);
        }
        if (this.accept(Sym.FLOAT_L)) {
            return new LiteralNode(s, LiteralType.FLOAT, this.lastTokenText);
        }
        if (this.accept(Sym.DOUBLE_L)) {
            return new LiteralNode(s, LiteralType.DOUBLE, this.lastTokenText);
        }
        if (this.accept(Sym.CHAR_L)) {
            return new LiteralNode(s, LiteralType.CHAR, this.lastTokenText);
        }
        if (this.accept(Sym.STRING_L)) {
            return new LiteralNode(s, LiteralType.STRING, this.lastTokenText);
        }
        if (this.accept(Sym.BOOL_L)) {
            return new LiteralNode(s, LiteralType.BOOLEAN, this.lastTokenText);
        }
        if (this.accept(Sym.NULL_L)) {
            return new LiteralNode(s, LiteralType.NULL, "null");
        }
        return null;
    }
    //End EDIT
    /*
     * Rule : "[" Expression  "]" "[" Expression  "]"
     *        | Arguments |
     */
    private IdentifierSuffixNode parseIdentifierSuffix() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.EDGECLAMPS_OPEN)) {
            int cnt = 0;
            if (this.accept(Sym.EDGECLAMPS_CLOSE)) {
                cnt++;
            }
            while (this.accept(Sym.EDGECLAMPS_OPEN)) {
                if (this.accept(Sym.EDGECLAMPS_CLOSE)) {
                    cnt++;
                } else {
                    throw new CompilerException("Missing ] after [", this.lexSymbol);
                }
            }
            if (this.accept(Sym.DOT)) {
                if (this.accept(Sym.CLASS)) {
                    return new IdentifierSuffixNode(s, cnt, IdentifierSuffixNode.Suffixes.CLASS);
                }
                throw new CompilerException("Missing .class after []", this.lexSymbol);
            }
            ExpressionNode exp = this.parseExpression();
            if (exp != null) {
                if (this.accept(Sym.EDGECLAMPS_CLOSE)) {
                    LinkedList<IdentifierSuffixNode.EdgeClampsExpression> eList = new LinkedList<>();
                    eList.add(new IdentifierSuffixNode.EdgeClampsExpression(this.lexSymbol, exp));
                    while (this.accept(Sym.EDGECLAMPS_OPEN)) {
                        exp = this.parseExpression();
                        if (exp != null) {
                            if (this.accept(Sym.EDGECLAMPS_CLOSE)) {
                                eList.add(new IdentifierSuffixNode.EdgeClampsExpression(this.lexSymbol, exp));
                            } else {
                                throw new CompilerException("Missing ] after [", this.lexSymbol);
                            }
                        } else {
                            throw new CompilerException("Missing exp ] after [", this.lexSymbol);
                        }
                    }
                    return new IdentifierSuffixNode(s, eList);
                }
                throw new CompilerException("Missing ] after [ exp ", this.lexSymbol);
            }
            throw new CompilerException("Missing expression after [,", this.lexSymbol);
        }
        ArgumentsNode args = this.parseArguments();
        if (args != null) {
            return new IdentifierSuffixNode(s, args);
        }
        if (this.accept(Sym.DOT)) {
            if (this.accept(Sym.CLASS)) {
                return new IdentifierSuffixNode(s, IdentifierSuffixNode.Suffixes.CLASS);
            }
            if (this.accept(Sym.THIS)) {
                throw new CompilerException("Inner this access not allowed in TeaJay",s);
                //return new IdentifierSuffixNode(s, IdentifierSuffixNode.Suffixes.THIS);
            }
            if (this.accept(Sym.SUPER)) {
                args = this.parseArguments();
                if (args != null) {
                    throw new CompilerException("Inner super access not allowed in TeaJay",s);
                    //return new IdentifierSuffixNode(s, IdentifierSuffixNode.Suffixes.SUPER, args);
                }
                throw new CompilerException("Missing arguments after super", this.lexSymbol);
            }
            if (this.accept(Sym.NEW)) {
                NonWildcardTypeArgumentsNode nWTypeArgs = this.parseNonWildcardTypeArguments();
                if (nWTypeArgs != null) {
                    InnerCreatorNode innerCreator = this.parseInnerCreator();
                    if (innerCreator != null) {
                        throw new CompilerException("Inner creation of type is not allowed", s);
                        //return new IdentifierSuffixNode(s, IdentifierSuffixNode.Suffixes.NEW, nWTypeArgs, innerCreator);
                    }
                    throw new CompilerException("Missing inner creator after new non wildcard type arguments", this.lexSymbol);
                }
                InnerCreatorNode innerCreator = this.parseInnerCreator();
                if (innerCreator != null) {
                    throw new CompilerException("Inner creation of type is not allowed", s);
                    //return new IdentifierSuffixNode(s, IdentifierSuffixNode.Suffixes.NEW, innerCreator);
                }
                throw new CompilerException("Missing inner creator after new ", this.lexSymbol);
            }
            ExplicitGenericInvocationNode eGenInv = this.parseExplicitGenericInvocation();
            if (eGenInv != null) {
                return new IdentifierSuffixNode(s, eGenInv);
            }
            throw new CompilerException("Missing class or this, super arguments, or this or explicit generic invokation or new after dot", this.lexSymbol);
        }
        return null;
    }

    private ArgumentsNode parseArguments() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.BRACKET_OPEN)) {
            ExpressionNode exp = this.parseExpression();
            if (exp != null) {
                LinkedList<ExpressionNode> eList = new LinkedList<>();
                ExpressionNode e;
                while (this.accept(Sym.COMMA)) {
                    e = this.parseExpression();
                    if (e != null) {
                        eList.add(e);
                    } else {
                        throw new CompilerException("Missing expression after ,", this.lexSymbol);
                    }
                }
                if (this.accept(Sym.BRACKET_CLOSE)) {
                    if (eList.size() > 0) {

                        return new ArgumentsNode(s, exp, eList);
                    }
                    return new ArgumentsNode(s, exp, null);
                }
                throw new CompilerException("Missing ) after ( expression ", this.lexSymbol);
            }
            if (this.accept(Sym.BRACKET_CLOSE)) {
                return new ArgumentsNode(s);
            }
            throw new CompilerException("Missing ) after ( ", this.lexSymbol);
        }
        return null;
    }

    private ExplicitGenericInvocationSuffixNode parseExplicitGenericInvocationSuffix() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.SUPER)) {
            SuperSuffixNode superSuffix = this.parseSuperSuffix();
            if (superSuffix != null) {
                return new ExplicitGenericInvocationSuffixNode(s, superSuffix);
            }
            throw new CompilerException("Missing super suffix after super", this.lexSymbol);
        }
        if (this.accept(Sym.IDENTIFIER)) {
            String ident = this.lastTokenText;
            ArgumentsNode args = this.parseArguments();
            if (args != null) {
                return new ExplicitGenericInvocationSuffixNode(s, ident, args);
            }
            throw new CompilerException("Missing arguments after identifier", this.lexSymbol);
        }
        return null;
    }
    /*
     * Rule: NonWildcardTypeArguments CreatedName ClassCreatorRest
     *       CreatedName ( ClassCreatorRest | ArrayCreatorRest )
     */
    private CreatorNode parseCreator() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        NonWildcardTypeArgumentsNode nonWildcardTypeArgs = this.parseNonWildcardTypeArguments();
        if (nonWildcardTypeArgs != null) {
            CreatedNameNode createdName = this.parseCreatedName();
            if (createdName != null) {
                ClassCreatorRestNode classCreatorRest = this.parseClassCreatorRest();
                if (classCreatorRest != null) {
                    return new CreatorNode(s, nonWildcardTypeArgs, createdName, classCreatorRest);
                }
                throw new CompilerException("Missing class creator after created name", this.lexSymbol);
            }
            throw new CompilerException("Missing created name after non wildcard type arguments", this.lexSymbol);
        }
        CreatedNameNode createdName = this.parseCreatedName();
        if (createdName != null) {
            ClassCreatorRestNode classCreatorRest = this.parseClassCreatorRest();
            if (classCreatorRest != null) {
                return new CreatorNode(s, createdName, classCreatorRest);
            }
            ArrayCreatorRestNode arrayCreatorRest = this.parseArrayCreatorRest();
            if (arrayCreatorRest != null) {
                throw new CompilerException("Array creation not supported in Teajay use List",s);
                //return new CreatorNode(s, createdName, arrayCreatorRest);
            }
            throw new CompilerException("Missing class creator or array creator after created name", this.lexSymbol);
        }

        return null;
    }
    /*
     * Rule: Arguments | "." Identifier Arguments
     */
    private SuperSuffixNode parseSuperSuffix() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        ArgumentsNode args = this.parseArguments();
        if (args != null) {
            return new SuperSuffixNode(s, args);
        }
        if (this.accept(Sym.DOT)) {

            if (this.accept(Sym.IDENTIFIER)) {
                String ident = this.lastTokenText;
                ArgumentsNode iargs = this.parseArguments();
                if (iargs != null) {
                    return new SuperSuffixNode(s, ident, iargs);
                }
                return new SuperSuffixNode(s, ident);
            }
            throw new CompilerException("Missing identifier after .", this.lexSymbol);
        }
        return null;
    }
    /*
     * Rule: NonWildcardTypeArguments ExplicitGenericInvocationSuffix
     */
    private ExplicitGenericInvocationNode parseExplicitGenericInvocation() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        NonWildcardTypeArgumentsNode nonWildCardTypeArgs = this.parseNonWildcardTypeArguments();
        if (nonWildCardTypeArgs != null) {
            ExplicitGenericInvocationSuffixNode explicitGenInvocSuffix = this.parseExplicitGenericInvocationSuffix();
            if (explicitGenInvocSuffix != null) {
                return new ExplicitGenericInvocationNode(s, nonWildCardTypeArgs, explicitGenInvocSuffix);
            }
            throw new CompilerException("Missing explicit generic invocation suffix after non wildcard type arguments", this.lexSymbol);
        }
        return null;
    }

    /*
     * Rule: Identifier [NonWildcardTypeArgumentsOrDiamond] ClassCreatorRest
     */
    private InnerCreatorNode parseInnerCreator() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.IDENTIFIER)) {
            String ident = this.lastTokenText;
            NonWildcardTypeArgumentsOrDiamond nonWildcardtypeArgsOrDiamiond = this.parseNonWildcardTypeArgumentsOrDiamond();
            if (nonWildcardtypeArgsOrDiamiond != null) {
                ClassCreatorRestNode cCreatorRest = this.parseClassCreatorRest();
                if (cCreatorRest != null) {
                    return new InnerCreatorNode(s, ident, nonWildcardtypeArgsOrDiamiond, cCreatorRest);
                }
                throw new CompilerException("Missing class creator after non wildcard type or < > ", this.lexSymbol);
            }
            ClassCreatorRestNode cCreatorRest = this.parseClassCreatorRest();
            if (cCreatorRest != null) {
                return new InnerCreatorNode(s, ident, cCreatorRest);
            }
            throw new CompilerException("Missing class creator after identifier ", this.lexSymbol);
        }
        return null;
    }
    /*
     * Rule: Identifier [TypeArgumentsOrDiamond] { . Identifier [TypeArgumentsOrDiamond] }
     *       | BaseType 
     *  The second rule is in TeaJay not supported
     */
    private CreatedNameNode parseCreatedName() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.IDENTIFIER)) {
            String ident = this.lastTokenText;
            TypeArgumentsOrDiamondNode tArgsOrDia = this.parseTypeArgumentsOrDiamond();
            if (tArgsOrDia != null) {

                LinkedList<CreatedNameNode.IdentTypeArgumentsOrDiamondNode> l = new LinkedList<>();
                while (this.accept(Sym.DOT)) {
                    if (this.accept(Sym.IDENTIFIER)) {
                        String identifier = this.lastTokenText;
                        TypeArgumentsOrDiamondNode t = this.parseTypeArgumentsOrDiamond();
                        if (t != null) {
                            l.add(new CreatedNameNode.IdentTypeArgumentsOrDiamondNode(this.lexSymbol, identifier, t));
                        } else {
                            l.add(new CreatedNameNode.IdentTypeArgumentsOrDiamondNode(this.lexSymbol, identifier));
                        }
                    } else {
                        throw new CompilerException("Missing identifier after .", this.lexSymbol);
                    }
                }
                if (l.size() > 0) {
                    return new CreatedNameNode(s, ident, tArgsOrDia, l);
                }
                return new CreatedNameNode(s, ident, tArgsOrDia, null);
            }
            LinkedList<CreatedNameNode.IdentTypeArgumentsOrDiamondNode> l = new LinkedList<>();
            while (this.accept(Sym.DOT)) {
                if (this.accept(Sym.IDENTIFIER)) {
                    String identifier = this.lastTokenText;
                    TypeArgumentsOrDiamondNode t = this.parseTypeArgumentsOrDiamond();
                    if (t != null) {
                        l.add(new CreatedNameNode.IdentTypeArgumentsOrDiamondNode(this.lexSymbol, identifier, t));
                    } else {
                        l.add(new CreatedNameNode.IdentTypeArgumentsOrDiamondNode(this.lexSymbol, identifier));
                    }
                } else {
                    throw new CompilerException("Missing identifier after .", this.lexSymbol);
                }
            }
            if (l.size() > 0) {
                return new CreatedNameNode(s, ident, l);
            }
            return new CreatedNameNode(s, ident, null);
        }
        //Not in the java rules
        BasicTypeNode baseType = this.parseBasicType();
        if (baseType != null) {
            return new CreatedNameNode(s, baseType);
        }
        return null;
    }

    /*
     * Rule: Arguments ClassBody
     */
    private ClassCreatorRestNode parseClassCreatorRest() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        ArgumentsNode args = this.parseArguments();
        if (args != null) {
            ClassBodyNode classBody = this.parseClassBody();
            if (classBody != null) {
                throw new CompilerException("Anonymous type implementation not supported",s);
                //return new ClassCreatorRestNode(s, args, classBody);
            }
            return new ClassCreatorRestNode(s, args);
        }
        return null;
    }

    /*
     * Rule "["
     *         ( "]" {"["  "]"} ArrayInitializer  |  Expression "]" {"[" Expression "]"} {"[" "]"} )
     *      "]"
     */
    private ArrayCreatorRestNode parseArrayCreatorRest() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.EDGECLAMPS_OPEN)) {
            if (this.accept(Sym.EDGECLAMPS_CLOSE)) {
                int cnt = 0;
                while (this.accept(Sym.EDGECLAMPS_OPEN)) {
                    if (this.accept(Sym.EDGECLAMPS_CLOSE)) {
                        cnt++;
                    } else {
                        throw new CompilerException("Missing ] after [", this.lexSymbol);
                    }
                }
                ArrayInitializerNode arrayInit = this.parseArrayInitializer();
                if (arrayInit != null) {
                    return new ArrayCreatorRestNode(s, cnt, arrayInit);
                }
                throw new CompilerException("Missing array init after []", this.lexSymbol);

            }
            ExpressionNode exp = this.parseExpression();
            if (exp != null) {
                if (this.accept(Sym.EDGECLAMPS_CLOSE)) {
                    LinkedList<ExpressionNode> expList = new LinkedList<>();
                    int cnt = 0;
                    while (this.accept(Sym.EDGECLAMPS_OPEN)) {
                        ExpressionNode e = this.parseExpression();
                        if (e != null) {
                            if (this.accept(Sym.EDGECLAMPS_CLOSE)) {
                                expList.add(e);
                                continue;
                            }
                        }
                        if (this.accept(Sym.EDGECLAMPS_CLOSE)) {
                            cnt++;
                            break;
                        }
                        throw new CompilerException("Missing expression or ] after[", this.lexSymbol);
                    }
                    while (this.accept(Sym.EDGECLAMPS_OPEN)) {
                        if (this.accept(Sym.EDGECLAMPS_CLOSE)) {
                            cnt++;
                        } else {
                            throw new CompilerException("Missing ] after [", this.lexSymbol);
                        }
                    }
                    return new ArrayCreatorRestNode(s, exp, expList, cnt);
                }
                throw new CompilerException("Missing ] after [ expression", this.lexSymbol);
            }


            throw new CompilerException("Missing ] after [", this.lexSymbol);
        }
        return null;
    }

    /*
     * Rule: EnumConstant {, EnumConstant}
     */
    private EnumConstantsNode parseEnumConstantsParse() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        EnumConstantNode enumConstant = this.parseEnumConstant();
        if (enumConstant != null) {
            if (this.accept(Sym.COMMA)) {
                LinkedList<EnumConstantNode> eList = new LinkedList<>();
                EnumConstantNode eIter = this.parseEnumConstant();
                /*while (this.accept(Sym.COMMA)) {
                 if (eIter != null) {
                 eList.add(eIter);
                 eIter = this.parseEnumConstant();
                 } else {
                 break;
                 }
                 }*/
                while (eIter != null) {
                    eList.add(eIter);
                    if (this.accept(Sym.COMMA)) {
                        eIter = this.parseEnumConstant();
                    } else {
                        break;
                    }
                }
                if (eList.size() > 0) {
                    return new EnumConstantsNode(s, enumConstant, eList);
                }
                return new EnumConstantsNode(s, enumConstant, null);
            }
            return new EnumConstantsNode(s, enumConstant, null);
        }
        return null;
    }

    /*
     * Rule: ";" ClassBodyDeclaration
     */
    
    private EnumBodyDeclarationsNode parseEnumBodyDeclarations() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.SEMICOLON)) {
            /*ClassBodyDeclarationNode cIter = this.parseClassBodyDeclaration();
            LinkedList<ClassBodyDeclarationNode> cList = new LinkedList<>();
            while (cIter != null) {
                cList.add(cIter);
                cIter = this.parseClassBodyDeclaration();
            }
            return new EnumBodyDeclarationsNode(s, cList);*/
        }
        return null;
    }

    /*
     * Rule: [ Annotations ] Identifier [Arguments] [ClassBody]  
     */
    private EnumConstantNode parseEnumConstant() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        AnnotationsNode annotations = this.parseAnnotations();
        if (annotations != null) {
            if (this.accept(Sym.IDENTIFIER)) {
                String ident = this.lastTokenText;
                ArgumentsNode args = this.parseArguments();
                if (args != null) {
                    ClassBodyNode classBody = this.parseClassBody();
                    if (classBody != null) {
                        return new EnumConstantNode(s, annotations, ident, args, classBody);
                    }
                    return new EnumConstantNode(s, annotations, ident, args);
                }
                ClassBodyNode classBody = this.parseClassBody();
                if (classBody != null) {
                    return new EnumConstantNode(s, annotations, ident, null, classBody);
                }
                return new EnumConstantNode(s, annotations, ident);
            }
        }
        if (this.accept(Sym.IDENTIFIER)) {
            String ident = this.lastTokenText;
            ArgumentsNode args = this.parseArguments();
            if (args != null) {
                ClassBodyNode classBody = this.parseClassBody();
                if (classBody != null) {
                    return new EnumConstantNode(s, ident, args, classBody);
                }
                return new EnumConstantNode(s, ident, args);
            }
            ClassBodyNode classBody = this.parseClassBody();
            if (classBody != null) {
                return new EnumConstantNode(s, ident, null, classBody);
            }
            return new EnumConstantNode(s, ident);
        }

        return null;
    }
    /*
     * Rule: AnnotationTypeElementDeclaration {AnnotationTypeElementDeclaration}
     */
    private AnnotationTypeElementDeclarationsNode parseAnnotationTypeElementDeclarations() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        AnnotationTypeElementDeclarationNode a = this.parseAnnotationTypeElementDeclaration();
        if (a != null) {
            AnnotationTypeElementDeclarationNode aIter = this.parseAnnotationTypeElementDeclaration();
            LinkedList<AnnotationTypeElementDeclarationNode> aList = new LinkedList<>();
            while (aIter != null) {
                aList.add(aIter);
                aIter = this.parseAnnotationTypeElementDeclaration();
            }
            if (aList.size() > 0) {
                return new AnnotationTypeElementDeclarationsNode(s, a, aList);
            }
            return new AnnotationTypeElementDeclarationsNode(s, a);
        }
        return null;
    }
    /*
     * [ {Modifier} ] AnnotationTypeElementRest 
     */
    private AnnotationTypeElementDeclarationNode parseAnnotationTypeElementDeclaration() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        ModifierNode mIter = this.parseModifier();
        LinkedList<ModifierNode> mList = new LinkedList<>();
        while (mIter != null) {
            mList.add(mIter);
            mIter = this.parseModifier();
        }
        AnnotationTypeElementRestNode aTRest = this.parseAnnotationTypeElementRest();
        if (aTRest != null) {
            if (mList.size() > 0) {
                return new AnnotationTypeElementDeclarationNode(s, mList, aTRest);
            }
            return new AnnotationTypeElementDeclarationNode(s, mList, null);
        }
        if (mList.size() > 0) {
            throw new CompilerException("Missing annotation type element rest", this.lexSymbol);
        }
        return null;
    }
    /*
     *  Type Identifier AnnotationMethodOrConstantRest ; |
     *   ClassDeclaration |
     *   InterfaceDeclaration |
     *   EnumDeclaration |
     *   AnnotationTypeDeclaration
     */
    private AnnotationTypeElementRestNode parseAnnotationTypeElementRest() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        TypeNode type = this.parseType();
        if (type != null) {

            if (this.accept(Sym.IDENTIFIER)) {
                String ident = this.lastTokenText;
                AnnotationMethodOrConstantRestNode amcr = this.parseAnnotationMethodOrConstantRest();
                if (amcr != null) {
                    if (this.accept(Sym.SEMICOLON)) {
                        return new AnnotationTypeElementRestNode(s, type, ident, amcr);
                    }
                    throw new CompilerException("Missing ; after annotation method or constante", this.lexSymbol);
                }
                throw new CompilerException("Missing annotation method or constante after identifier", this.lexSymbol);
            }
            throw new CompilerException("Missing identifier after type", this.lexSymbol);
        }
        ClassDeclarationNode classDecl = this.parseClassDeclaration();
        if (classDecl != null) {
            return new AnnotationTypeElementRestNode(s, classDecl);
        }
        InterfaceDeclarationNode interfaceDecl = this.parseInterfaceDeclaration();
        if (interfaceDecl != null) {
            return new AnnotationTypeElementRestNode(s, interfaceDecl);
        }
        EnumDeclarationNode enumDecl = this.parseEnumDeclaration();
        if (enumDecl != null) {
            return new AnnotationTypeElementRestNode(s, enumDecl);
        }
        AnnotationTypeDeclarationNode atd = this.parseAnnotationTypeDeclaration();
        if (atd != null) {
            return new AnnotationTypeElementRestNode(s, atd);
        }
        return null;
    }
    
    /*
    * Rule: AnnotationMethodRest | ConstantDeclaratorsRest
    */
    private AnnotationMethodOrConstantRestNode parseAnnotationMethodOrConstantRest() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        AnnotationMethodRestNode amr = this.parseAnnotationMethodRest();
        if (amr != null) {
            return new AnnotationMethodOrConstantRestNode(s, amr);
        }
        ConstantDeclaratorsRestNode constDeclsRest = this.parseConstantDeclaratorsRest();
        if (constDeclsRest != null) {
            return new AnnotationMethodOrConstantRestNode(s, constDeclsRest);
        }
        return null;
    }

    /*
     * Rule: "(" ")" ["[" "]"] [default ElementValue]
     */
    private AnnotationMethodRestNode parseAnnotationMethodRest() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.BRACKET_OPEN)) {
            if (this.accept(Sym.BRACKET_CLOSE)) {
                if (this.accept(Sym.EDGECLAMPS_OPEN)) {
                    if (this.accept(Sym.EDGECLAMPS_CLOSE)) {
                        if (this.accept(Sym.DEFAULT)) {
                            ElementValueNode eVal = this.parseElementValue();
                            if (eVal != null) {
                                return new AnnotationMethodRestNode(s, true, eVal);
                            }
                            throw new CompilerException("Missing element value after default", this.lexSymbol);
                        }
                        return new AnnotationMethodRestNode(s, true);
                    }
                    throw new CompilerException("Missing ] after [", this.lexSymbol);
                }
                if (this.accept(Sym.DEFAULT)) {
                    ElementValueNode eVal = this.parseElementValue();
                    if (eVal != null) {
                        return new AnnotationMethodRestNode(s, true, eVal);
                    }
                    throw new CompilerException("Missing element value after default", this.lexSymbol);
                }
                return new AnnotationMethodRestNode(s);
            }
            throw new CompilerException("Missing ) after (", this.lexSymbol);
        }
        return null;
    }

    private ExpressionHelperNode parseExpressionHelper() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        ExpressionNode exp = this.parseExpression();
        if (exp != null) {
            ExpressionHelperNode expHelper = this.parseExpressionHelper();
            return new ExpressionHelperNode(s, exp, expHelper);

        }
        return new ExpressionHelperNode(s);
    }

    /* 
     * Matchen des Konkrettesten 
     * 
     * BaseString func(int i ) { switch (i ) case 0: return null; case 1: retun this.str = new MyString(); ...}
     
     * case Array<Number> { }   (AUSBLICK: )
     * 
     * case Array<Object>  { }  (AUSBLICK: )
     * 
     
     * 
     * <T> Array<T> func() { return new Array<T>(100); }
     * Object val = func();
     * typeswitch(  ){
     * 
     * 
     * 
     * case Object {
     *              
     *          }
     *      
     * case BaseString { }
     * 
     * case String { }
     * 
     
     * default  { }
     *  
     * }
     * 
     * if((val) instanceof Number) { 
     *           
     * }
     * else if((val) instanceof Integer) {
     *      CODEGEN: LocalTable.get(val).setConcretType(Integer);
     *   
     *   //val =>integer;
     *   val.methodedieIntegerbesitzaberNumbernicht();
     *   val =  new Integer(10);
     * }
     *          
     * }
     * 
     * 
     * Rule: "typeswitch" "(" identifier ")" "{" TypeSwitchBlocks "}"
     */
    private TypeSwitchStatementNode parseTypeSwitchStatement() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.TYPESWITCH)) {
            //  ParExpressionNode parExp = this.parseParExpression();
            if (this.accept(Sym.BRACKET_OPEN)) {
                if (this.accept(Sym.IDENTIFIER)) {
                    String ident = this.lastTokenText;

                    if (this.accept(Sym.BRACKET_CLOSE)) {
                        if (this.accept(Sym.CURLY_OPEN)) {

                            TypeSwitchBlocksNode typeSwitchBlocks = this.parseTypeSwitchBlocks();
                            if (typeSwitchBlocks == null) {
                                throw new CompilerException("Empty typeswitch", lexSymbol);
                            }

                            if (this.accept(Sym.CURLY_CLOSE)) {
                                return new TypeSwitchStatementNode(s, ident, typeSwitchBlocks);
                            }
                            throw new CompilerException("Missing } after typeswitch (identifier ) { ...", lexSymbol);
                        }
                        throw new CompilerException("Missing { after typeswitch (identifier) ", lexSymbol);
                    }
                    throw new CompilerException("Missing ) after typeswitch (identifier ", lexSymbol);
                }
                throw new CompilerException("Missing identifier after typeswitch", s);
            }
            throw new CompilerException("Missing ( identifier ) after typeswitch", s);
        }

        return null;
    }

    /*
     * Rule: {TypeSwitchBlock} "default" block
     */
    private TypeSwitchBlocksNode parseTypeSwitchBlocks() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        List<TypeSwitchBlockNode> list = new LinkedList<>();
        TypeSwitchBlockNode node = this.parseTypeSwitchBlock();
        while (node != null) {
            list.add(node);
            node = this.parseTypeSwitchBlock();
        }
        if (this.accept(Sym.DEFAULT)) {
            BlockNode block = this.parseBlock();
            if (block != null) {
                return new TypeSwitchBlocksNode(s, list, block);
            }
            throw new CompilerException("Missing block after default in typeswitch", s);
        } else {
            throw new CompilerException("missing default case in typeswitch", s);
        }
//        if (list.isEmpty()) {
//            return null;
//        }
//        return new TypeSwitchBlocksNode(s, list);
    }

    /*
     * Rule: TypeSwitchLabel Block
     */
    private TypeSwitchBlockNode parseTypeSwitchBlock() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        TypeSwitchLabelNode label = this.parseTypeSwitchLabel();
        if (label != null) {
            BlockNode block = this.parseBlock();
            if (block != null) {
                return new TypeSwitchBlockNode(s, label, block);
            }
            throw new CompilerException("Missing block after type in typeswitch", this.lexSymbol);
        }
        return null;
    }
    /*
     * Rule: "case" Type 
     */
    private TypeSwitchLabelNode parseTypeSwitchLabel() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.CASE)) {
            TypeNode type = this.parseType();
            if (type != null) {
                return new TypeSwitchLabelNode(s, type);
            } else {

                throw new CompilerException("Missing type after case in typeswitch", s);
            }
        }
        return null;
    }

    /*
     *  "Closure" [ "<" Type ">" ] ClosureTypeFormalParam
     */
    private ClosureTypeDecNode parseClosureTypeDec() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.CLOSUREDEC)) {

            if (this.accept(Sym.OP_LESS)) {
                TypeNode type = this.parseType();
                if (type != null) {
                    s = this.lexSymbol;
                    if (this.accept(Sym.OP_GREATER)) {
                        ClosureTypeFormalParamNode formalParam = this.parseClosureTypeFormalParam();
                        return new ClosureTypeDecNode(s, type, formalParam);
                    }
                    throw new CompilerException("Missing > after < in closure declaration ", s);
                }
                throw new CompilerException("Missing return type in closure declaration ", s);
            }
            ClosureTypeFormalParamNode formalParam = this.parseClosureTypeFormalParam();
            if (formalParam == null) {
                throw new CompilerException("Missing formal parameters for Closure-type", s);
            }
            return new ClosureTypeDecNode(s, null, formalParam);
        }
        return null;
    }
    
    /*
     * Rule: "(" [Type { "," Type} ] ")"
     */
    
    private ClosureTypeFormalParamNode parseClosureTypeFormalParam() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.BRACKET_OPEN)) {
            List<TypeNode> l = new LinkedList<>();
            TypeNode iter = this.parseType();
            while (iter != null) {
                l.add(iter);
                if (this.accept(Sym.COMMA)) {
                    iter = this.parseType();
                } else {
                    break;
                }
            }
            if (this.accept(Sym.BRACKET_CLOSE)) {
                return new ClosureTypeFormalParamNode(s, l);
            }
        }
        return null;
    }
    /*
     *  Rule: ClosureEnv ClosureRet FormalParameters Block
     */
    private ClosureCreatorNode parseClosureCreator() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        ClosureEnvNode env = this.parseClosureEnv();
        ClosureReturnNode ret = this.parseClosureRet();
        FormalParametersNode parameters = this.parseFormalParameters();
        if (parameters == null) {
            throw new CompilerException("Missing formal parameters", s);
        }
        BlockNode block = this.parseBlock();
        if (block != null) {
            ArgumentsNode args = this.parseArguments();
            return new ClosureCreatorNode(s, env, ret, parameters, block, args);
        }

        return null;
    }
    /*
     * Rule: "[" ClosureEnvList "]"
     */
    private ClosureEnvNode parseClosureEnv() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.EDGECLAMPS_OPEN)) {
            ClosureEnvListNode l = this.parseClosureEnvList();
            if (this.accept(Sym.EDGECLAMPS_CLOSE)) {
                return new ClosureEnvNode(s, l);
            }
            throw new CompilerException("Missing ] after [ in closure environment defination", lexSymbol);
        }
        return null;
    }
    /*
     * Rule: "<" Type ">"
     */
    private ClosureReturnNode parseClosureRet() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.OP_LESS)) {
            TypeNode type = this.parseType();
            if (this.accept(Sym.OP_GREATER)) {
                return new ClosureReturnNode(s, type);
            }
            throw new CompilerException("Missing > after < in closure defination", s);
        }
        return null;
    }
    /*
     * Rule: { ClosureEnvItem ; }
     */
    private ClosureEnvListNode parseClosureEnvList() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        List<ClosureEnvItemNode> l = new LinkedList<>();
        ClosureEnvItemNode iter = this.parseClosureEnvItem();
        while (iter != null) {
            l.add(iter);
            if (!this.accept(Sym.SEMICOLON)) {
                break;
            }
            iter = this.parseClosureEnvItem();
        }
        return new ClosureEnvListNode(s, l);
    }
    /*
     * Rule: Type Identifier "=" Expression
     */
    private ClosureEnvItemNode parseClosureEnvItem() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        TypeNode type = this.parseType();
        if (type != null) {


            if (this.accept(Sym.IDENTIFIER)) {
                String ident = this.lastTokenText;
                Symbol sym1 = this.lexSymbol;
                if (this.accept(Sym.OP_EQ)) {
                    Symbol sym2 = this.lexSymbol;
                    ExpressionNode exp = this.parseExpression();
                    if (exp != null) {
                        return new ClosureEnvItemNode(s, type, ident, exp);
                    }
                    throw new CompilerException("Missing expression after =", sym2);
                }
                throw new CompilerException("Missing = after identifier in closure environment", sym1);
            }
            throw new CompilerException("Missing identifier after type in closure environment", s);
        }

        return null;
    }
    /*
     * Rule: {Modifier} "typedef" identifier [TypeParameters] 
     *       ["extends" Type] ["interfaces TypeList] TjTypeDefBody 
     */
    private TjTypeDefNode parseTjTypeDef() throws CompilerException, IOException {
        Symbol s = this.lexSymbol;
        LinkedList<ModifierNode> modifierList = new LinkedList<>();
        
        ModifierNode modifier = this.parseModifier();
        while(modifier != null){
            modifierList.add(modifier);
            modifier = this.parseModifier();
        }
        if (!this.accept(Sym.TYPEDEF)) {
            return null;
        }

        if (!this.accept(Sym.IDENTIFIER)) {
            throw new CompilerException("Missing identifier after class declaration", this.lexSymbol);
        }
        String typeName = this.lastTokenText;

        TypeParametersNode typeParameters = this.parseTypeParameters();
        TypeNode type = null;
        if (this.accept(Sym.EXTENDS)) {
            type = this.parseType();
            if (type == null) {
                throw new CompilerException("Missing type after extends declaration", this.lexSymbol);
            }
        }
        TypeListNode typeList = null;
        if (this.accept(Sym.INTERFACES)) {
            typeList = this.parseTypeList();
            if (typeList == null) {
                throw new CompilerException("Missing interface type after interface declaration", this.lexSymbol);
            }
        }
        TjTypeDefBodyNode typeDefBody = this.parseTjTypeDefBody();
        if (typeDefBody != null) {
            return new TjTypeDefNode(s, modifierList, typeName, typeParameters, type, typeList, typeDefBody);
        }
        throw new CompilerException("Missing body after type declaration", this.lexSymbol);
    }
    /*
     * "{" {TjTypeDefBodyDeclaration } "}" 
     */
    private TjTypeDefBodyNode parseTjTypeDefBody() throws CompilerException, IOException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.CURLY_OPEN)) {
            TjTypeDefBodyDeclarationNode tjTypeDefBodyDecl;
            ArrayList<TjTypeDefBodyDeclarationNode> tjTypeDefBodyList = new ArrayList<>();
            tjTypeDefBodyDecl = this.parseTjTypeDefBodyDeclaration();
            while (tjTypeDefBodyDecl != null) {
                tjTypeDefBodyList.add(tjTypeDefBodyDecl);
                tjTypeDefBodyDecl = this.parseTjTypeDefBodyDeclaration();
            }
            if (this.accept(Sym.CURLY_CLOSE)) {
                return new TjTypeDefBodyNode(s, tjTypeDefBodyList);
            } else {
                throw new CompilerException("Missing } after { ", this.lexSymbol);
            }
        }
        return null;
    }
    /*
     *  Rule: [";"] | {Modifier} TjTypeDefMemberDecl 
     */
    private TjTypeDefBodyDeclarationNode parseTjTypeDefBodyDeclaration() throws CompilerException, IOException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.SEMICOLON)) {
            return new TjTypeDefBodyDeclarationNode(s, true);
        }
        ModifierNode modifier = this.parseModifier();
        ArrayList<ModifierNode> modsList = new ArrayList<>();
        while (modifier != null) {
            modsList.add(modifier);
            modifier = this.parseModifier();
        }
        TjTypeDefMemberDeclNode tjTypeDefMemberDecl = this.parseTjTypeDefMemberDecl();
        if (tjTypeDefMemberDecl != null) {
            return new TjTypeDefBodyDeclarationNode(s, modsList, tjTypeDefMemberDecl);
        }
        return null;

    }
    /*
     * Rule: TjTypeDefConstructorDecl  | Type Identifier InterfaceMethodDeclaratorRest
     *       | "void" Identifier VoidInterfaceDeclaratorRest | InterfaceGenericMethodDecl
     */
    private TjTypeDefMemberDeclNode parseTjTypeDefMemberDecl() throws CompilerException, IOException {
        boolean isConstructor = this.lex.lookahead(Sym.IDENTIFIER, Sym.BRACKET_OPEN);
        Symbol s = this.lexSymbol;

        if (isConstructor) {
            TjTypeDefConstructorDeclNode construcor = this.parseTjTypeDefConstructorDecl();
            if (construcor != null) {
                return new TjTypeDefMemberDeclNode(s, construcor);
            }
        }

        TypeNode type = this.parseType();
        if (type != null) {
            if (this.accept(Sym.IDENTIFIER)) {
                String ident = this.lastTokenText;
                InterfaceMethodDeclaratorRestNode interfaceMethodDeclaratorRest = this.parseInterfaceMethodDeclaratorRest();
                if (interfaceMethodDeclaratorRest != null) {
                    return new TjTypeDefMemberDeclNode(s, type, ident, interfaceMethodDeclaratorRest);
                }
                throw new CompilerException("Missing method declaration after " + ident, lexSymbol);
            }
        }

        if (this.accept(Sym.VOID)) {

            if (this.accept(Sym.IDENTIFIER)) {
                String ident = this.lastTokenText;
                VoidInterfaceMethodDeclaratorRestNode voidInterfaceMethodDeclaratorRestNode = this.parseVoidInterfaceMethodDeclaratorRest();
                if (voidInterfaceMethodDeclaratorRestNode != null) {
                    return new TjTypeDefMemberDeclNode(s, ident, voidInterfaceMethodDeclaratorRestNode);
                }
                throw new CompilerException("Missing void method declaration in interface", this.lexSymbol);
            }
            throw new CompilerException("Missing identifier after void", this.lexSymbol);
        }

        InterfaceGenericMethodDeclNode interfaceGenericMethodDeclNode = this.parseInterfaceGenericMethodDecl();
        if (interfaceGenericMethodDeclNode != null) {
            return new TjTypeDefMemberDeclNode(s, interfaceGenericMethodDeclNode);
        }

        return null;
    }
    /*
     * Rule: Identifier InterfaceMethodDeclaratorRest
     */
    private TjTypeDefConstructorDeclNode parseTjTypeDefConstructorDecl() throws CompilerException, IOException {
        Symbol s = this.lexSymbol;
        if (this.accept(Sym.IDENTIFIER)) {
            String ident = this.lastTokenText;
            InterfaceMethodDeclaratorRestNode interfaceMethodDeclaratorRest = this.parseInterfaceMethodDeclaratorRest();
            if (interfaceMethodDeclaratorRest != null) {
                return new TjTypeDefConstructorDeclNode(s, ident, interfaceMethodDeclaratorRest);
            }
            throw new CompilerException("Missing formal parameters after method declaration", lexSymbol);
        }
        return null;
    }
    /*
     * Rule "typeimpl" Identifier [TypeParameters] "implements" Type ClassBody
     */
    private TjImplDeclarationNode parseTjImplDeclaration() throws IOException, CompilerException {
        Symbol s = this.lexSymbol;
        if (!this.accept(Sym.TYPEIMPL)) {
            return null;
        }

        if (!this.accept(Sym.IDENTIFIER)) {
            throw new CompilerException("Missing identifier after typeimpl", this.lexSymbol);
        }
        String implName = this.lastTokenText;

        TypeParametersNode typeParameters = this.parseTypeParameters();
        TypeNode implementsTjType = null;
        if (this.accept(Sym.IMPLEMENTS)) {
            implementsTjType = this.parseType();
            if (implementsTjType == null) {
                throw new CompilerException("Missing type after implements declaration", this.lexSymbol);
            }
            ClassBodyNode classBody = this.parseClassBody();
            if (classBody != null) {
                return new TjImplDeclarationNode(s, implName, typeParameters, implementsTjType, classBody);
            }
            throw new CompilerException("Missing body in typeimpl", s);
        }
        throw new CompilerException("Missing implements after typeimpl", s);
    }
}
