/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package parser;

import java.io.IOException;
import lexer.LexerWrapper;
import lexer.Rule;
import lexer.Sym;
import lexer.Symbol;

/**
 *
 * @author Michael Burzan
 */
public class TypeRule implements Rule {

    private Symbol lexSymbol;

//    private boolean checkClosureType(LexerWrapper lexer) throws IOException, GenericException{
//        if(this.accept(lexer, Sym.CLOSUREDEC)){
//            return true;
//        }
//        return false;
//    }
    private class GenericException extends Exception {
    }
    private class ClosureDec extends RuntimeException{
        
    }

    @Override
    public boolean checkRule(LexerWrapper lexer) {
        boolean isRule = false;
        try {
            lexSymbol = lexer.getCurrentToken();
            
            isRule = checkType(lexer);
            if (isRule && this.accept(lexer, Sym.IDENTIFIER)) {
                return true;
            }

        } catch (IOException ex) {
            //IGNORE EXCEPTION
        } catch (GenericException e) {
            return false;
        }catch(ClosureDec e){
            return true;
        }
        return false;
    }

    private boolean checkDotIdentifier(LexerWrapper lex) throws IOException, GenericException {
        boolean isDotIdent = false;
        while (this.accept(lex, Sym.DOT)) {
            if (!this.accept(lex, Sym.IDENTIFIER)) {
                throw new GenericException();
            }
            checkGenericTypeRule(lex);
            isDotIdent = true;
        }
        return isDotIdent;
    }

    private boolean checkRefType(LexerWrapper lexer) throws IOException, GenericException {
        if (this.accept(lexer, Sym.IDENTIFIER)) {
            this.checkGenericTypeRule(lexer);
            this.checkDotIdentifier(lexer);


            while (this.accept(lexer, Sym.EDGECLAMPS_OPEN)) {
                if (!this.accept(lexer, Sym.EDGECLAMPS_CLOSE)) {
                    return false;
                }
            }
            return true;
        }
        if (this.checkBaseType(lexer)) {
            while (this.accept(lexer, Sym.EDGECLAMPS_OPEN)) {
                if (!this.accept(lexer, Sym.EDGECLAMPS_CLOSE)) {
                    return false;
                }
            }
            return true;
        }
        return false;
    }

    private boolean checkType(LexerWrapper lexer) throws IOException, GenericException {
        if (this.checkRefType(lexer)) {
            return true;
        }
//        if(this.checkClosureType(lexer)){
//            return true;
//        }
        return false;
    }

    private boolean checkBaseType(LexerWrapper lexer) throws IOException {
        if (this.accept(lexer, Sym.INT)) {
            return true;
        }
        if (this.accept(lexer, Sym.FLOAT)) {
            return true;
        }
        if (this.accept(lexer, Sym.DOUBLE)) {
            return true;
        }
        if (this.accept(lexer, Sym.LONG)) {
            return true;
        }
        if (this.accept(lexer, Sym.SHORT)) {
            return true;
        }
        if (this.accept(lexer, Sym.CHAR)) {
            return true;
        }
        if (this.accept(lexer, Sym.BOOLEAN)) {
            return true;
        }
        if (this.accept(lexer, Sym.BYTE)) {
            return true;
        }
        return false;
    }

    private boolean checkGenericTypeRule(LexerWrapper lexer) throws IOException, GenericException {

        if (this.accept(lexer, Sym.OP_LESS)) {
            lexer.setTemplateState();
            if(this.accept(lexer, Sym.CLOSUREDEC)){
                throw new ClosureDec();
            }
            if(this.accept(lexer, Sym.QUESTIONMARK)){
                throw new ClosureDec();
            }
            if (checkGenericParameter(lexer)) {

                while (this.accept(lexer, Sym.COMMA)) {
                    if (!checkGenericParameter(lexer)) {
                        lexer.resetTemplateState();
                        throw new GenericException();
                    }
                }
                if (this.accept(lexer, Sym.OP_GREATER)) {
                    lexer.resetTemplateState();
                    return true;
                }
            }
            lexer.resetTemplateState();
            throw new GenericException();
        }

        return false;
    }

    private boolean checkGenericParameter(LexerWrapper lexer) throws IOException, GenericException {
        if (this.accept(lexer, Sym.QUESTIONMARK)) {
            if (this.accept(lexer, Sym.SUPER) || this.accept(lexer, Sym.EXTENDS)) {
                if (this.checkRefType(lexer)) {
                    return true;
                }
                return false;
            }

            return true;
        }
        if (this.checkRefType(lexer)) {
            return true;
        }
        return false;
    }

    private boolean accept(LexerWrapper lexer, int symbol) throws IOException {
        if (symbol == this.lexSymbol.sym) {
            this.lexSymbol = lexer.yylex();
            return true;
        }
        return false;

    }

    /*  @Override
     public boolean checkRule(LexerWrapper lex) {
     boolean isRule = false;
     try {
     lexSymbol = lex.yylex();
     isRule = checkRuleType(lex);
          

     } catch (IOException ex) {
     //IGNORE EXCEPTION
     }
     return isRule;

     }

     private boolean checkRuleType(LexerWrapper lex) throws IOException {
       
        
     //RULE "<" Template ">" "." ident "<" Template ">" {"[" "]"} ident
     if (checkGenericTypeRule(lex)) {
     if (checkDotIdentifier(lex)) {

     while (this.accept(lex, Sym.EDGECLAMPS_OPEN)) {
     if (!this.accept(lex, Sym.EDGECLAMPS_CLOSE)) {
     return false;
     }
     }
     if (this.accept(lex, Sym.IDENTIFIER)) {
     return true;
     }
     return false;
     }
     }
        
     // RULE : = {"."ident"<"Templat">"} {"[" "]"} ident
     if (checkDotIdentifier(lex)) {

     while (this.accept(lex, Sym.EDGECLAMPS_OPEN)) {
     if (!this.accept(lex, Sym.EDGECLAMPS_CLOSE)) {
     return false;
     }
     }
     if (this.accept(lex, Sym.IDENTIFIER)) {
     return true;
     }
     return false;
     }
        
     //RULE: =  {"[" "]"} ident
     while (this.accept(lex, Sym.EDGECLAMPS_OPEN)) {
     if (!this.accept(lex, Sym.EDGECLAMPS_CLOSE)) {
     return false;
     }
     }
     if (this.accept(lex, Sym.IDENTIFIER)) {
     return true;
     }
     return false;
     }

     private boolean checkGenericTypeRule(LexerWrapper lexer) throws IOException {
                    
     if (this.accept(lexer, Sym.OP_LESS)) {
     lexer.setTemplateState();
     if (checkGenericParameter(lexer)) {

     while (this.accept(lexer, Sym.COMMA)) {
     if (!checkGenericParameter(lexer)) {
     return false;
     }
     }
     if (this.accept(lexer, Sym.OP_GREATER)) {
     lexer.resetTemplateState();
     return true;
     }
     }
     }
     return false;
     }

     private boolean checkGenericParameter(LexerWrapper lexer) throws IOException {
     if (this.accept(lexer, Sym.QUESTIONMARK)) {
     if (this.accept(lexer, Sym.SUPER) || this.accept(lexer, Sym.EXTENDS)) {
     if (this.checkRefType(lexer)) {
     return true;
     }
     return false;
     }

     return true;
     }
     if (this.checkRefType(lexer)) {
     return true;
     }
     return false;
     }

     private boolean accept(LexerWrapper lexer, int symbol) throws IOException {
     if (symbol == this.lexSymbol.sym) {
     this.lexSymbol = lexer.yylex();
     return true;
     }
     return false;

     }

     private boolean checkDotIdentifier(LexerWrapper lex) throws IOException {
     boolean isDotIdent = false;
     while (this.accept(lex, Sym.DOT)) {
     if (!this.accept(lex, Sym.IDENTIFIER)) {
     return false;
     }
     checkGenericTypeRule(lex);
     isDotIdent = true;
     }
     return isDotIdent;
     }

     private boolean checkRefType(LexerWrapper lexer) throws IOException {
     if(this.accept(lexer, Sym.IDENTIFIER)){
     this.checkGenericTypeRule(lexer);
            
     while(this.accept(lexer, Sym.EDGECLAMPS_OPEN)){
     if(!this.accept(lexer, Sym.EDGECLAMPS_CLOSE)){
     return false;
     }
     }
     return true;
     }
     if(this.checkBaseType(lexer)){
     while(this.accept(lexer, Sym.EDGECLAMPS_OPEN)){
     if(!this.accept(lexer, Sym.EDGECLAMPS_CLOSE)){
     return false;
     }
     }
     return true;
     }
     return false;
     }

     private boolean checkBaseType(LexerWrapper lexer) throws IOException {
     if(this.accept(lexer, Sym.INT)){
     return true;
     }
     if(this.accept(lexer, Sym.FLOAT)){
     return true;
     }
     if(this.accept(lexer, Sym.DOUBLE)){
     return true;
     }
     if(this.accept(lexer, Sym.LONG)){
     return true;
     }
     if(this.accept(lexer, Sym.SHORT)){
     return true;
     }
     if(this.accept(lexer, Sym.CHAR)){
     return true;
     }
     if(this.accept(lexer, Sym.BOOLEAN)){
     return true;
     }
     if(this.accept(lexer, Sym.BYTE)){
     return true;
     }
     return false;
     }*/
}
