/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage0;

import abstractsyntaxtree.CompilationUnitHelperNode;
import abstractsyntaxtree.CompilationUnitNode;
import abstractsyntaxtree.ImportDeclarationNode;
import abstractsyntaxtree.ModifierNode;
import abstractsyntaxtree.TjTypeDefBodyNode;
import abstractsyntaxtree.TjTypeDefNode;
import abstractsyntaxtree.TypeDeclarationNode;
import abstractsyntaxtree.TypeListNode;
import abstractsyntaxtree.TypeNode;
import abstractsyntaxtree.TypeParametersNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractCompilationUnitVisitor;
import codegeneration.abstractvisitors.AbstractTjTypeDefVisitor;
import codegeneration.environment.Stage0Environment;
import codegeneration.javavisitor.compilationunit.CompilationUnitHelperVisitor;
import codegeneration.javavisitor.compilationunit.ImportDeclarationVisitor;
import exceptions.CompilerException;
import java.util.ArrayList;
import java.util.Collection;
import minicompiler.SourceFileNameChecker;
import teajay.bytecode.ClassAccessFlag;
import teajay.bytecode.ClassFile;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.util.QIDTools;
import teajay.util.io.ClassFinder;
import teajay.util.io.TJSourceFinder;
import teajay.util.tjattribute.TeaJayAttribute;

/**
 *
 * @author Michael Burzan get the package node visit the typedeclaration node
 * visit typedef node: get the name of typedef generate full qualifierdname and
 * create the class file for a typedef
 */
public class Stage0Visitor extends AbstractCompilationUnitVisitor {

    CompilationUnitHelperVisitor compilationUnitHelperVisitor = new CompilationUnitHelperVisitor();
    private final Stage0Environment stage0Env;
    private final Collection<ClassAccessFlag> modis = new ArrayList<>();

    public Stage0Visitor(Stage0Environment env) {
        this.stage0Env = env;
    }

    @Override
    public void begin(CompilationUnitNode node) throws CompilerException {
    }

    @Override
    public void visit(CompilationUnitNode node) throws CompilerException {
    }

    @Override
    public void end(CompilationUnitNode node) throws CompilerException {
        String qid = stage0Env.getClassFile().getThis().getClassName();
        String fileName = stage0Env.getAst().value1;
        SourceFileNameChecker.checkSourceFile(fileName, qid, node);
    }

    @Override
    public void begin(TypeDeclarationNode node) throws CompilerException {
    }

    @Override
    public Visitor visit(TypeDeclarationNode node) throws CompilerException {
        return new Stage0TypeDeclarationVisitor(compilationUnitHelperVisitor.getPackageName(), this.stage0Env);
    }

    @Override
    public void end(TypeDeclarationNode node) throws CompilerException {
    }

    @Override
    public void begin(ImportDeclarationNode node) throws CompilerException {
    }

    @Override
    public Visitor visit(ImportDeclarationNode node) throws CompilerException {
        return new ImportDeclarationVisitor(node.hasWildcard(), node.isStatic(), this.stage0Env);
    }

    @Override
    public void end(ImportDeclarationNode node) throws CompilerException {
    }

    @Override
    public void begin(CompilationUnitHelperNode node) throws CompilerException {
    }

    @Override
    public Visitor visit(CompilationUnitHelperNode node) throws CompilerException {
        return this.compilationUnitHelperVisitor;
    }

    @Override
    public void end(CompilationUnitHelperNode node) throws CompilerException {
        // TJSourceFinder finder = TJSourceFinder.instance();
        // stage0Env.addOwnPackageToWorkingQueue(finder.findAll(compilationUnitHelperVisitor.getPackageName()));
    }

    @Override
    public void begin(TjTypeDefNode node) throws CompilerException {
    }

    @Override
    public Visitor visit(TjTypeDefNode node) throws CompilerException {
        return new AbstractTjTypeDefVisitor() {
            @Override
            public void begin(ModifierNode node) throws CompilerException {
                switch (node.getModifier()) {
                    case PUBLIC:
                        modis.add(ClassAccessFlag.PUBLIC);
                        break;
                    case PROTECTED:
                        throw new CompilerException(node.getModifier() + "not allowed", node.getSymbol());
                    case PRIVATE:
                        throw new CompilerException(node.getModifier() + "not allowed", node.getSymbol());
                    case STATIC:
                        throw new CompilerException(node.getModifier() + "not allowed", node.getSymbol());
                    case ABSTRACT:
                        throw new CompilerException(node.getModifier() + "not allowed", node.getSymbol());
                    case FINAL:
                        modis.add(ClassAccessFlag.FINAL);
                        break;
                    case NATIVE:
                        throw new CompilerException(node.getModifier() + "not allowed", node.getSymbol());
                    case SYNCHRONIZED:
                        throw new CompilerException(node.getModifier() + "not allowed", node.getSymbol());
                    case TRANSIENT:
                        throw new CompilerException(node.getModifier() + "not allowed", node.getSymbol());
                    case VOLATILE:
                        throw new CompilerException(node.getModifier() + "not allowed", node.getSymbol());
                    case STRICTFP:
                        throw new CompilerException(node.getModifier() + "not allowed", node.getSymbol());
                    case SYNTHETIC:
                        throw new CompilerException(node.getModifier() + "not allowed", node.getSymbol());
                    default:
                        throw new AssertionError(node.getModifier().name());

                }
            }

            @Override
            public void begin(TypeParametersNode node) throws CompilerException {
            }

            @Override
            public void begin(TypeNode node) throws CompilerException {
            }

            @Override
            public void begin(TypeListNode node) throws CompilerException {
            }

            @Override
            public void begin(TjTypeDefBodyNode node) throws CompilerException {
            }

            @Override
            public Visitor visit(TypeParametersNode node) throws CompilerException {
                return null;
            }

            @Override
            public Visitor visit(TypeNode node) throws CompilerException {
                return null;
            }

            @Override
            public Visitor visit(TypeListNode node) throws CompilerException {
                return null;
            }

            @Override
            public Visitor visit(TjTypeDefBodyNode node) throws CompilerException {
                return null;
            }

            @Override
            public Visitor visit(ModifierNode node) throws CompilerException {
                return null;
            }

            @Override
            public void end(TypeParametersNode node) throws CompilerException {
            }

            @Override
            public void end(TypeNode node) throws CompilerException {
            }

            @Override
            public void end(TypeListNode node) throws CompilerException {
            }

            @Override
            public void end(TjTypeDefBodyNode node) throws CompilerException {
            }

            @Override
            public void end(ModifierNode node) throws CompilerException {
            }
        };

    }

    @Override
    public void end(TjTypeDefNode node) throws CompilerException {
        String typeName = node.getIdentifier();
        try {
            modis.add(ClassAccessFlag.ABSTRACT);
            modis.add(ClassAccessFlag.SUPER);
            boolean containsFinal = modis.contains(ClassAccessFlag.FINAL);
            modis.remove(ClassAccessFlag.FINAL);
            ClassFile file;
            String packageName = compilationUnitHelperVisitor.getPackageName();

            if (packageName != null) {
                file = new ClassFile(packageName + "/" + typeName, modis.toArray(new ClassAccessFlag[0]));
            } else {
                file = new ClassFile(typeName, modis.toArray(new ClassAccessFlag[0]));
            }
            if (containsFinal) {
                file.addAttribute(new TeaJayAttribute(TeaJayAttribute.Type.Typedef, TeaJayAttribute.AccessType.Final));
            } else {
                file.addAttribute(new TeaJayAttribute(TeaJayAttribute.Type.Typedef));
            }

            ClassFinder.instance().addClassFile(file);
            stage0Env.setClassFile(file);


        } catch (ByteCodeException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }

    }
}
