/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage1;

import abstractsyntaxtree.ReferenceTypeNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractBoundVisitor;
import codegeneration.environment.Stage1Environment;
import exceptions.CompilerException;
import java.util.ArrayList;
import java.util.List;
import minicompiler.WorkingNode;
import teajay.util.io.ClassFinder;
import teajay.util.io.Input;
import teajay.util.io.TJSourceFinder;

/**
 *
 * @author Michael Burzan
 */
class Stage1BoundVisitor extends AbstractBoundVisitor {

    private final Stage1Environment stage1Env;
    private Stage1GenericReferenceTypeVisitor genVisitor;
    private List<String> identifierList = new ArrayList<>();

    public Stage1BoundVisitor(Stage1Environment stage1Env) {
        this.stage1Env = stage1Env;
    }

    @Override
    public void begin(ReferenceTypeNode node) throws CompilerException {
        this.identifierList.add(node.getName());
    }

    @Override
    public Visitor visit(ReferenceTypeNode node) throws CompilerException {

        return genVisitor = new Stage1GenericReferenceTypeVisitor(stage1Env, identifierList);
    }

    @Override
    public void end(ReferenceTypeNode node) throws CompilerException {
        StringBuilder identifier = new StringBuilder();

        for (int i = 0; i < identifierList.size(); i++) {
            identifier.append(identifierList.get(i));
            if (i != (identifierList.size() - 1)) {
                identifier.append("/");
            }
        }

        String ident = identifier.toString();
        if (!stage1Env.isGenericParameter(ident)) {
            if (stage1Env.getFacade().getFullQualifiedId(ident) == null) {
                if (ClassFinder.instance().find(ident) == null) {
                    String packageName = stage1Env.getPackageName();
                    if(packageName == null){
                        packageName = "";
                    }
                    Input find = TJSourceFinder.instance().find(packageName+"/" + ident);
                    if (find != null) {
                        stage1Env.addWorkingNode(new WorkingNode(stage1Env.getFacade(),packageName+"/"+ident, find));

                    } else {
                        find = TJSourceFinder.instance().find(ident);
                        if(find != null){
                            stage1Env.addWorkingNode(new WorkingNode(stage1Env.getFacade(),ident, find));
                        }else{
                            throw new CompilerException("Type " + ident + " not exist", node.getSymbol());
                        }
                        
                    }
                }
            }
        }
    }
}
