/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage2;

import abstractsyntaxtree.FieldDeclaratorsRestNode;
import abstractsyntaxtree.MethodeDeclaratorRestNode;
import abstractsyntaxtree.ModifierNode;
import static abstractsyntaxtree.ModifierNode.Modifier.PROTECTED;
import static abstractsyntaxtree.ModifierNode.Modifier.SYNCHRONIZED;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractMethodeOrFieldRestVisitor;
import codegeneration.environment.Stage2Environment;
import exceptions.CompilerException;
import java.util.Collection;
import java.util.List;
import stage2.utils.MethodHeadBuilder;
import teajay.bytecode.ClassFile;
import teajay.bytecode.attributes.signature.Signature;
import teajay.bytecode.cpool.FieldDescriptor;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.bytecode.fields.FieldAccessFlag;
import teajay.bytecode.fields.FieldInfo;
import teajay.bytecode.methods.MethodInfo;
import teajay.generics.parser.Type;
import teajay.generics.writer.SignatureEncoder;

/**
 *
 * @author Michael Burzan
 */
class Stage2TjImplMethodeOrFieldRestVisitor extends AbstractMethodeOrFieldRestVisitor {

    private final Stage2Environment stage2Env;
    private final List<ModifierNode.Modifier> flagList;
    private final Collection<List<MethodInfo>> methods;
    private final String fieldOrMethodName;
    private final Type type;
    private Stage2FieldDeclaratorsRestVisitor stage2FieldDeclarator;
    private Stage2MethodHeadVisitor stage2MethodDeclaratorRest;

    public Stage2TjImplMethodeOrFieldRestVisitor(Stage2Environment stage2Env,
            List<ModifierNode.Modifier> flagList, Collection<List<MethodInfo>> methods,
            String fieldOrMethodName, Type type) {
        this.stage2Env = stage2Env;
        this.flagList = flagList;
        this.methods = methods;
        this.fieldOrMethodName = fieldOrMethodName;
        this.type = type;
    }

    @Override
    public void begin(FieldDeclaratorsRestNode node) throws CompilerException {
    }

    @Override
    public void begin(MethodeDeclaratorRestNode node) throws CompilerException {
    }

    @Override
    public Visitor visit(FieldDeclaratorsRestNode node) throws CompilerException {
        return stage2FieldDeclarator = new Stage2FieldDeclaratorsRestVisitor(stage2Env);
    }

    @Override
    public Visitor visit(MethodeDeclaratorRestNode node) throws CompilerException {
        return stage2MethodDeclaratorRest = new Stage2MethodHeadVisitor(stage2Env);
    }

    @Override
    public void end(FieldDeclaratorsRestNode node) throws CompilerException {
        FieldDescriptor fieldDescriptor = stage2Env.getTypeUtils().makeFieldDescriptor(type);
        try {
            FieldInfo field;
            checkExistFieldInClassFile(this.stage2Env.getClassFile(), this.fieldOrMethodName, node);
            if (this.flagList.isEmpty()) {
                field = new FieldInfo(fieldOrMethodName, fieldDescriptor, FieldAccessFlag.PRIVATE);
            } else {
                int i = 0;
                FieldAccessFlag[] accessFlags = new FieldAccessFlag[flagList.size()];
                for (ModifierNode.Modifier modi : flagList) {
                    switch (modi) {
                        case PUBLIC:
                            break;
                        case PROTECTED:
                            break;
                        case PRIVATE:
                            accessFlags[i] = FieldAccessFlag.PRIVATE;
                            i++;
                            break;
                        case STATIC:
                            accessFlags[i] = FieldAccessFlag.STATIC;
                            i++;
                            break;
                        case ABSTRACT:
                            break;
                        case FINAL:
                            break;
                        case NATIVE:
                            break;
                        case SYNCHRONIZED:
                            break;
                        case TRANSIENT:
                            break;
                        case VOLATILE:
                            break;
                        case STRICTFP:
                            break;
                        default:
                            throw new AssertionError(modi.name());

                    }
                }
                
                field = new FieldInfo(fieldOrMethodName, fieldDescriptor, accessFlags);
            }
            //add signature to field, because generic field declaration is possible
            field.addAttribute(new Signature(SignatureEncoder.encode(type)));
            //add field to class file
            this.stage2Env.getClassFile().addField(field);
        } catch (ByteCodeException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }
    }

    @Override
    public void end(MethodeDeclaratorRestNode node) throws CompilerException {
        //Build Methodhead
        MethodHeadBuilder.buildMethodHeadTypeImpl(node, stage2Env, type, fieldOrMethodName, this.stage2MethodDeclaratorRest.getMethodHead(), flagList);
    }

    /**
     * 
     * @param classFile current class file
     * @param fieldOrMethodName field or method name
     * @param node
     * @throws CompilerException 
     */
    private void checkExistFieldInClassFile(ClassFile classFile, String fieldOrMethodName,
            FieldDeclaratorsRestNode node) throws CompilerException {
       
        Collection<FieldInfo> fields = classFile.getFields();
        for (FieldInfo field : fields) {
            if (field.getFieldName().equals(fieldOrMethodName)) {
                throw new CompilerException("Field " + fieldOrMethodName + "is already defined", node.getSymbol());
            }
        }
    }
}
