/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage2;

import abstractsyntaxtree.InterfaceMethodDeclaratorRestNode;
import abstractsyntaxtree.TypeNode;
import abstractsyntaxtree.TypeParametersNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractInterfaceGenericMethodDeclVisitor;
import codegeneration.environment.Stage2Environment;
import datastructs.Pair;
import exceptions.CompilerException;
import java.util.List;
import teajay.bytecode.methods.MethodAccessFlag;
import teajay.generics.parser.Type;

/**
 *
 * @author Michael Burzan
 */
class Stage2TjTypeDefInterfaceGenericMethodDeclVisitor extends AbstractInterfaceGenericMethodDeclVisitor {
    private final Stage2Environment stage2Env;
    private final String methodName;
    private final boolean isVoid;
    private Stage2TypeParametersVisitor stage2TypeParametersVisitor;
    private Stage2TypeVisitor stage2Type;
    private Stage2MethodHeadVisitor stage2InterfaceMethodDeclarator;
    private Type returnType;
    private List<Type.TypeParamType> typeParams;
    private final List<MethodAccessFlag> methodAccessFlag;

    Stage2TjTypeDefInterfaceGenericMethodDeclVisitor(Stage2Environment stage2Env, String identifier, boolean aVoid, List<MethodAccessFlag> methodAccessFlag) {
        this.stage2Env = stage2Env;
        this.methodName = identifier;
        this.isVoid = aVoid;
        this.methodAccessFlag = methodAccessFlag;
    }

    @Override
    public void begin(TypeParametersNode node) throws CompilerException {
        
    }

    @Override
    public void begin(TypeNode node) throws CompilerException {
        
    }

    @Override
    public void begin(InterfaceMethodDeclaratorRestNode node) throws CompilerException {
        
    }

    @Override
    public Visitor visit(TypeParametersNode node) throws CompilerException {
        return stage2TypeParametersVisitor = new Stage2TypeParametersVisitor(stage2Env);
    }

    @Override
    public Visitor visit(TypeNode node) throws CompilerException {
        return stage2Type = new Stage2TypeVisitor(stage2Env);
    }

    @Override
    public Visitor visit(InterfaceMethodDeclaratorRestNode node) throws CompilerException {
        return stage2InterfaceMethodDeclarator =  new Stage2MethodHeadVisitor(stage2Env);
    }

    @Override
    public void end(TypeParametersNode node) throws CompilerException {
        this.typeParams = stage2TypeParametersVisitor.getTypeParams();
    }

    @Override
    public void end(TypeNode node) throws CompilerException {
        this.returnType = stage2Type.getType();
    }

    @Override
    public void end(InterfaceMethodDeclaratorRestNode node) throws CompilerException {
        if(isVoid){
            this.returnType = new Type.SimpleType("void");
        }
    }
    /**
     * 
     * @return Pair: The first attribute is a List of Pair with Type and String 
     * this represents the method parameter. The second attribute is a List of String this
     * represents the throws List
     * @throws CompilerException 
     */
    public Pair<List<Pair<Type, String>>, List<String>> getMethodHead() throws CompilerException {
        return stage2InterfaceMethodDeclarator.getMethodHead();
    }
    /**
     * Example: <T,Y> methodName(T a,Y b)
     * @return generic Types 
     */
    public List<Type.TypeParamType> getTypeParams() {
        return typeParams;
    }
    
    public Type getReturnType() {
        return returnType;
    }

    public String getMethodName() {
        return methodName;
    }

    boolean isVoid() {
        return isVoid;
    }
    
    
}
