/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage2;

import abstractsyntaxtree.BasicTypeNode;
import abstractsyntaxtree.ClosureTypeDecNode;
import abstractsyntaxtree.EdgeClampsOpenCloseHelperNode;
import abstractsyntaxtree.ReferenceTypeNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractTypeVisitor;
import codegeneration.environment.Stage2Environment;
import exceptions.CompilerException;
import java.util.ArrayList;
import java.util.List;
import teajay.generics.check.TypeUtils;
import teajay.generics.parser.Type;

/**
 *
 * @author Michael Burzan
 * Visit reference type an build the Type Class instance
 */
class Stage2TypeVisitor extends AbstractTypeVisitor {

    private final Stage2Environment stage2Env;
    private List<String> identifierList = new ArrayList<>();
    private Stage2ReferenceTypeVisitor stage2ReferenceType;
    private Type type;
    private Stage2ClosureTypeDecVisitor closureTypeDec;

    public Stage2TypeVisitor(Stage2Environment stage2Env) {
        this.stage2Env = stage2Env;
    }

    @Override
    public void begin(BasicTypeNode node) throws CompilerException {
        this.type = new Type.SimpleType(node.getType().toString());
    }

    @Override
    public void visit(BasicTypeNode node) throws CompilerException {
    }

    @Override
    public void end(BasicTypeNode node) throws CompilerException {
    }

    @Override
    public void begin(ReferenceTypeNode node) throws CompilerException {
        identifierList.add(node.getName());
    }

    @Override
    public Visitor visit(ReferenceTypeNode node) throws CompilerException {

        return stage2ReferenceType = new Stage2ReferenceTypeVisitor(stage2Env, identifierList);
    }

    @Override
    public void end(ReferenceTypeNode node) throws CompilerException {
        StringBuilder identifier = new StringBuilder();
        //Build from identifier.identifier => identifier/identifier
        for (int i = 0; i < identifierList.size(); i++) {
            identifier.append(identifierList.get(i));
            if (i != (identifierList.size() - 1)) {
                identifier.append("/");
            }
        }

        String ident = identifier.toString();
        //get qid from identifier
        String qID = stage2Env.getSymbolFacade().getFullQualifiedId(ident);
        if (qID == null) {
            qID = ident;
        }
        //is type a generic parameter
        if (stage2Env.isTypeVariable(ident)) {
            this.type = new Type.SimpleType(ident);
        } else {
            //otherwise build ClassType
            this.type = new Type.ClassType(null, qID, stage2ReferenceType.getTypeParameters());
            
            TypeUtils.checkTypes(type, stage2Env.getClassFile().getThis().getClassName(), node.getSymbol());
        }
    }

    @Override
    public void begin(EdgeClampsOpenCloseHelperNode node) throws CompilerException {
    }

    @Override
    public void visit(EdgeClampsOpenCloseHelperNode node) throws CompilerException {
    }

    @Override
    public void end(EdgeClampsOpenCloseHelperNode node) throws CompilerException {
    }

    @Override
    public void begin(ClosureTypeDecNode node) throws CompilerException {
    }

    @Override
    public Visitor visit(ClosureTypeDecNode node) throws CompilerException {
        return closureTypeDec = new Stage2ClosureTypeDecVisitor(stage2Env);
    }

    @Override
    public void end(ClosureTypeDecNode node) throws CompilerException {
        this.type = closureTypeDec.getType();
    }

    @Override
    public String getTypeName() throws CompilerException {
        return null;
    }

    public Type getType() throws CompilerException {
        return this.type;
    }
}
