/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage3;

import abstractsyntaxtree.ClassOrInterfaceDeclaratioNode;
import abstractsyntaxtree.LocalVariableDeclarationsStatementNode;
import abstractsyntaxtree.StatementNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractBlockStatementVisitor;
import codegeneration.environment.Stage3Environment;
import exceptions.CompilerException;
import stage3.states.LocalVariableDeclarationState;

/**
 *
 * @author Michael Burzan
 */
public class Stage3BlockStatementVisitor extends AbstractBlockStatementVisitor {
    private final Stage3Environment stage3Env;
    private Stage3StatementVisitor statementVisitor;
    private boolean hasNoStatement = true;
   
   

    public Stage3BlockStatementVisitor(Stage3Environment stage3Env) {
        this.stage3Env = stage3Env;
    }

    @Override
    public void begin(LocalVariableDeclarationsStatementNode node) throws CompilerException {
        hasNoStatement = false;
        stage3Env.writeSuperConstructorCall(node);
        LocalVariableDeclarationState lState = new LocalVariableDeclarationState();
        stage3Env.changeState(lState, node);
    }

    @Override
    public void begin(ClassOrInterfaceDeclaratioNode node) throws CompilerException {
        
    }

    @Override
    public void begin(StatementNode node) throws CompilerException {
        
        hasNoStatement &= node.hasSemicolon();
    }

    @Override
    public Visitor visit(LocalVariableDeclarationsStatementNode node) throws CompilerException {
        
        return new Stage3LocalVariableDeclarationsStatementVisitor(stage3Env);
    }

    @Override
    public Visitor visit(ClassOrInterfaceDeclaratioNode node) throws CompilerException {
        throw new CompilerException("Inner class declaration not allowed", node.getSymbol());
    }

    @Override
    public Visitor visit(StatementNode node) throws CompilerException {
        return statementVisitor = new Stage3StatementVisitor(stage3Env);
    }

    @Override
    public void end(LocalVariableDeclarationsStatementNode node) throws CompilerException {
        stage3Env.prevState();
    }

    @Override
    public void end(ClassOrInterfaceDeclaratioNode node) throws CompilerException {
        
    }

    @Override
    public void end(StatementNode node) throws CompilerException {
        
    }
    /**
     * 
     * @return true if block is empty 
     */
    boolean hasNoStatement() {
        return this.hasNoStatement;
    }
    
}
