/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage3;

import abstractsyntaxtree.BlockNode;
import abstractsyntaxtree.CatchTypeNode;
import abstractsyntaxtree.VariableModifierNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractCatchClauseVisitor;
import codegeneration.environment.Stage3Environment;
import exceptions.CompilerException;
import stage3.states.BlockState;
import stage3.states.EndState;
import stage3.states.TryState;
import teajay.bytecode.attributes.code.AStore;
import teajay.bytecode.attributes.code.JVMInstruction;
import teajay.bytecode.attributes.code.Label;
import teajay.bytecode.attributes.code.VerificationType;
import teajay.bytecode.cpool.ClassInfo;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.generics.parser.Type;

/**
 *
 * @author Michael Burzan
 */
class Stage3CatchClauseVisitor extends AbstractCatchClauseVisitor {

    private final Stage3Environment stage3Env;
    private final String identifier;
    private Stage3CatchTypeVisitor catchType;
    private boolean hasEndState = false;
    

    Stage3CatchClauseVisitor(Stage3Environment stage3Env, String identifier) {
        this.stage3Env = stage3Env;
        this.identifier = identifier;
    }

    @Override
    public void begin(VariableModifierNode node) throws CompilerException {
    }

    @Override
    public void begin(CatchTypeNode node) throws CompilerException {
    }

    @Override
    public void begin(BlockNode node) throws CompilerException {
        stage3Env.changeState(new BlockState(), node);
    }

    @Override
    public Visitor visit(VariableModifierNode node) throws CompilerException {
        return null;
    }

    @Override
    public Visitor visit(CatchTypeNode node) throws CompilerException {
        return catchType = new Stage3CatchTypeVisitor(stage3Env);
    }

    @Override
    public Visitor visit(BlockNode node) throws CompilerException {
        return new Stage3BlockVisitor(stage3Env);
    }

    @Override
    public void end(VariableModifierNode node) throws CompilerException {
    }

    @Override
    public void end(CatchTypeNode node) throws CompilerException {
  
        try {
            
            stage3Env.getLocalTable().addFrame();

            Label label = new Label();
            /*
             * Write instruction to save the exception value into next free register
             * and generate the stack map frame.
             * 
             *It may not be set the variable to the local table  first
             *,since the calculation of the stack map table would be erroneous 
             * 
             */
            stage3Env.getLocalTable().addStackMapFrame(this.stage3Env.getMethodInfo().
                    getCode().getInstructionList().
                    add(label, new AStore(stage3Env.getLocalTable().getNextId())),
                    VerificationType.valueOf(new ClassInfo(((Type.ClassType) catchType.getType()).name)));

            TryState tryState = stage3Env.hasState(TryState.class);
            tryState.setHandleLabel(label);
            tryState.setClassInfo(new ClassInfo(((Type.ClassType) catchType.getType()).name));
            // Save catch variable in LocalTable
            stage3Env.getLocalTable().addEntry(identifier, catchType.getType());
        } catch (ByteCodeException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }
    }

    @Override
    public void end(BlockNode node) throws CompilerException {
        EndState endState = stage3Env.hasState(EndState.class);
        //remove end state from state machine
        if(endState != null){
            stage3Env.prevState();
            hasEndState = true;
        }
        // remove block state from state machine
        stage3Env.prevState();
        stage3Env.getLocalTable().removeFrame();
    }
    /**
     * 
     * @return true if catch block has reached a end statement 
     */
    public boolean hasEndStateReached(){
        return hasEndState;
    }
    
}
