/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage3;

import abstractsyntaxtree.BlockNode;
import abstractsyntaxtree.ParExpressionNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractIfStatementVisitor;
import codegeneration.environment.ExpressionEnvironment;
import codegeneration.environment.Stage3Environment;
import exceptions.CompilerException;
import java.util.LinkedList;
import java.util.List;
import stage3.states.BlockState;
import stage3.states.EndState;
import stage3.states.UnreachableCodeState;
import teajay.bytecode.attributes.code.Goto;
import teajay.bytecode.attributes.code.Label;
import teajay.bytecode.attributes.code.NOP;
import teajay.generics.check.TypeException;
import teajay.generics.parser.Type;

/**
 *
 * @author Michael Burzan
 */
public class Stage3IfStatementVisitor extends AbstractIfStatementVisitor {

    private final Stage3Environment stage3Env;
    final static int UNREACHABLECODE = 2;
    private Label label;
    private Label jmpOverElse;
    private boolean elseBlock;
    private Stage3BlockVisitor blockVisitor;
    private boolean ifBlock = true;
    private int deathCode = 0;
    private boolean nextStatementIsDeathCode;
    private boolean hasIfBlockEndState = false;
    private List<Label> labels;

    public Stage3IfStatementVisitor(Stage3Environment stage3Env) {
        this.stage3Env = stage3Env;
        this.elseBlock = false;
    }

    @Override
    public void begin(ParExpressionNode node) throws CompilerException {
    }

    @Override
    public void begin(BlockNode node) throws CompilerException {
        stage3Env.changeState(new BlockState(), node);
        stage3Env.getLocalTable().addFrame();
        if (elseBlock) {
            /*
             *  if(exp){
             *      ...
             *      goto jmpOverElse;
             *  }else{
             *      ...
             *  }
             *  jmpOverElse:
             *  ...
             */
            if (!hasIfBlockEndState) {
                jmpOverElse = new Label("jmpOverElse: " + stage3Env.getMethodInfo().getName().getValue());
                stage3Env.getExpressionEnvironment().insertCode(new Goto(jmpOverElse));
                BlockState blockState = stage3Env.hasState(BlockState.class);
                blockState.addLabels(getLabel());
                label = jmpOverElse;
            } else {
                /*
                 * if(exp){
                 *  return exp;
                 * }else{
                 *  ...
                 * }
                 * jump over else is unreachable
                 */
                BlockState blockState = stage3Env.hasState(BlockState.class);
                blockState.addLabels(getLabel());
                label = null;
            }

            ifBlock = false;
        } else {
            elseBlock = true;

        }
    }

    @Override
    public Visitor visit(ParExpressionNode node) throws CompilerException {
        return new Stage3ParExpressionVisitor(stage3Env);
    }

    @Override
    public Visitor visit(BlockNode node) throws CompilerException {
        return this.blockVisitor = new Stage3BlockVisitor(stage3Env);
    }

    @Override
    public void end(ParExpressionNode node) throws CompilerException {
        ExpressionEnvironment env = stage3Env.getExpressionEnvironment();
        Type.ClassType bType = new Type.ClassType(null, "java/lang/Boolean", null);
        Type typeOfResult = env.typeOfResult();
        try {
            stage3Env.getTypeUtils().checkAssignability(bType, typeOfResult);
            this.label = env.writeIf();

        } catch (TypeException ex) {
            throw new CompilerException("if statetement expects boolean", node.getSymbol());
        }
    }

    @Override
    public void end(BlockNode node) throws CompilerException {
        EndState endState = stage3Env.hasState(EndState.class);
        UnreachableCodeState unreachableCode = stage3Env.hasState(UnreachableCodeState.class);
        //check for unreachable code
        if (endState != null || unreachableCode != null) {
            deathCode++;
            stage3Env.prevState();
            hasIfBlockEndState = true;
        } else {
            deathCode--;
        }

        if (elseBlock && blockVisitor.isEmptyBlock()) {
            stage3Env.getExpressionEnvironment().insertCode(new NOP());
        }

        BlockState blockState = stage3Env.hasState(BlockState.class);
        labels = blockState.getLabels();
        stage3Env.prevState();



        stage3Env.getLocalTable().removeFrame();
        if (!ifBlock && deathCode == UNREACHABLECODE) {
            nextStatementIsDeathCode = true;
            jmpOverElse = null;
        }

    }

    public List<Label> getLabel() {
        List<Label> list = new LinkedList<>();
        if (label != null) {
            list.add(label);
        }
        for (Label l : labels) {
            if (!list.contains(l)) {
                list.add(l);
            }
        }

        return list;
    }

    public boolean isNextStatementUnreachable() {
        return this.nextStatementIsDeathCode;
    }
}
