/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage3;

import abstractsyntaxtree.BlockNode;
import abstractsyntaxtree.ParExpressionNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractWhileStatementVisitor;
import codegeneration.environment.ExpressionEnvironment;
import codegeneration.environment.Stage3Environment;
import exceptions.CompilerException;
import stage3.states.BlockState;
import stage3.states.EndState;
import stage3.states.LoopState;
import teajay.bytecode.attributes.code.Goto;
import teajay.bytecode.attributes.code.Label;
import teajay.bytecode.attributes.code.NOP;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.generics.check.TypeException;
import teajay.generics.parser.Type;

/**
 *
 * @author Michael Burzan
 * 
 * loop: exp
 * if false jmpOver
 * block
 * goto loop
 * jmpOver
 * 
 * if last statement in Block end statement:
 *
  loop: exp
 * if false jmpOver
 * block
 * end statement
 * jmpOver
 * 
 */
public class Stage3WhileStatementVisitor extends AbstractWhileStatementVisitor {

    private final Stage3Environment stage3Env;
    private Stage3ParExpressionVisitor parExpression;
    private Stage3BlockVisitor block;
    private Label jmpOverLoop;
    private Label loopLabel;

    public Stage3WhileStatementVisitor(Stage3Environment stage3Env) {
        this.stage3Env = stage3Env;
    }

    @Override
    public void begin(ParExpressionNode node) throws CompilerException {
        BlockState state = stage3Env.hasState(BlockState.class);
        //creates the loop label 
        loopLabel = new Label("LoopLabel");
        if (state == null) {
            throw new Error("Fatal error: while statement has no surrounded block state. This is inpossible");
        }
        //bind loop label to the first instruction in par expression
        state.addLabel(loopLabel);

    }

    @Override
    public void begin(BlockNode node) throws CompilerException {
        stage3Env.changeState(new BlockState(), node);
        this.stage3Env.getLocalTable().addFrame();
    }

    @Override
    public Visitor visit(ParExpressionNode node) throws CompilerException {
        return parExpression = new Stage3ParExpressionVisitor(stage3Env);
    }

    @Override
    public Visitor visit(BlockNode node) throws CompilerException {
        return block = new Stage3BlockVisitor(stage3Env);
    }

    @Override
    public void end(ParExpressionNode node) throws CompilerException {

        ExpressionEnvironment env = stage3Env.getExpressionEnvironment();
        //check is type result Boolean
        Type.ClassType bType = new Type.ClassType(null, "java/lang/Boolean", null);
        Type typeOfResult = env.typeOfResult();
        try {
            stage3Env.getTypeUtils().checkAssignability(bType, typeOfResult);
            this.jmpOverLoop = env.writeWhile();
            LoopState loopState = stage3Env.hasState(LoopState.class);
            loopState.setBreakLabel(this.jmpOverLoop);
            loopState.setLoopLabel(this.loopLabel);
        } catch (TypeException ex) {
            throw new CompilerException("while statetement expects boolean", node.getSymbol());
        }
    }

    @Override
    public void end(BlockNode node) throws CompilerException {
        EndState endState = stage3Env.hasState(EndState.class);
        if (endState != null) {
            stage3Env.prevState();
            stage3Env.prevState();
            stage3Env.getLocalTable().removeFrame();
        } else {
            //last statement in block is not break,continue,return or throw
            stage3Env.prevState();
            //wriet jump to loop label
            stage3Env.getExpressionEnvironment().insertCode(new Goto(loopLabel));
            stage3Env.getLocalTable().removeFrame();
        }


    }

    public Label getLabel() {
        return this.jmpOverLoop;
    }
}
