/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage3.states;

import abstractsyntaxtree.SyntaxNode;
import exceptions.CompilerException;

/**
 *
 * @author Michael Burzan
 *
 */
class BeginState implements State {

    BeginState() {
    }

    public State changeState(ConstructorState newState, SyntaxNode node) throws CompilerException {
        throw new CompilerException("Constructor declaration not allowed", node.getSymbol());
    }

    public State changeState(VoidMethodState newState, SyntaxNode node) throws CompilerException {
        throw new CompilerException("Method declaration not allowed", node.getSymbol());
    }

    public State changeState(AssertState state, SyntaxNode node) throws CompilerException {
        throw new CompilerException("Assert statement not allowed", node.getSymbol());
    }

    public State changeState(BlockState state, SyntaxNode node) throws CompilerException {
        throw new CompilerException("Block statement not allowed", node.getSymbol());
    }

    public State changeState(DoState state, SyntaxNode node) throws CompilerException {
        throw new CompilerException("Do statement not allowed", node.getSymbol());
    }

    public State changeState(IdentDoublePointState state, SyntaxNode node) throws CompilerException {
        throw new CompilerException("Identifier label not allowed", node.getSymbol());
    }

    public State changeState(IfState state, SyntaxNode node) throws CompilerException {
        throw new CompilerException("if statement not allowed here", node.getSymbol());
    }

    public State changeState(StatementExpressionState state, SyntaxNode node) throws CompilerException {
        throw new CompilerException("Statement not allowed", node.getSymbol());
    }

    public State changeState(WhileState state, SyntaxNode node) throws CompilerException {
        throw new CompilerException("While not allowed", node.getSymbol());
    }

    public State changeState(ForState state, SyntaxNode node) throws CompilerException {
        throw new CompilerException("For not allowed", node.getSymbol());
    }

    public State changeState(BreakState state, SyntaxNode node) throws CompilerException {
        throw new CompilerException("Break not allowed", node.getSymbol());
    }

    public State changeState(ContinueState state, SyntaxNode node) throws CompilerException {
        throw new CompilerException("Continue not allowed", node.getSymbol());
    }

    public State changeState(ReturnState state, SyntaxNode node) throws CompilerException {
        throw new CompilerException("Return not allowed", node.getSymbol());
    }

    public State changeState(ThrowState state, SyntaxNode node) throws CompilerException {
        throw new CompilerException("Throw not allowed", node.getSymbol());
    }

    public State changeState(SynchronizedState state, SyntaxNode node) throws CompilerException {
        throw new CompilerException("Synchronized not allowed", node.getSymbol());
    }

    public State changeState(TryState state, SyntaxNode node) throws CompilerException {
        throw new CompilerException("Try not allowed", node.getSymbol());
    }

    public State changeState(TypeSwitchState state, SyntaxNode node) throws CompilerException {
        throw new CompilerException("Typeswitch not allowed", node.getSymbol());
    }

    public State changeState(MethodState state, SyntaxNode node) throws CompilerException {
        throw new CompilerException("Method not allowed", node.getSymbol());
    }

    public State changeState(UnreachableCodeState state, SyntaxNode node) throws CompilerException {
        throw new CompilerException("Unreachable code not allowed", node.getSymbol());
    }

    public State changeState(LocalVariableDeclarationState state, SyntaxNode node) throws CompilerException {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    public State getParent() {
        return null;
    }
}

public abstract class AbstractState implements State {

    protected State parent = new BeginState();

    public State changeState(ConstructorState newState, SyntaxNode node) throws CompilerException {
        throw new CompilerException("Constructor declaration not allowed", node.getSymbol());
    }

    public State changeState(VoidMethodState newState, SyntaxNode node) throws CompilerException {
        throw new CompilerException("Method declaration not allowed", node.getSymbol());
    }

    public State changeState(AssertState state, SyntaxNode node) throws CompilerException {
        throw new CompilerException("Assert statement not allowed", node.getSymbol());
    }

    public State changeState(BlockState state, SyntaxNode node) throws CompilerException {
        throw new CompilerException("Block statement not allowed", node.getSymbol());
    }

    public State changeState(DoState state, SyntaxNode node) throws CompilerException {
        throw new CompilerException("Do statement not allowed", node.getSymbol());
    }

    public State changeState(IdentDoublePointState state, SyntaxNode node) throws CompilerException {
        throw new CompilerException("Identifier label not allowed", node.getSymbol());
    }

    public State changeState(IfState state, SyntaxNode node) throws CompilerException {
        throw new CompilerException("if statement not allowed here", node.getSymbol());
    }

    public State changeState(StatementExpressionState state, SyntaxNode node) throws CompilerException {
        throw new CompilerException("Statement not allowed", node.getSymbol());
    }

    public State changeState(WhileState state, SyntaxNode node) throws CompilerException {
        throw new CompilerException("While not allowed", node.getSymbol());
    }

    public State changeState(ForState state, SyntaxNode node) throws CompilerException {
        throw new CompilerException("For not allowed", node.getSymbol());
    }

    public State changeState(BreakState state, SyntaxNode node) throws CompilerException {
        throw new CompilerException("Break not allowed", node.getSymbol());
    }

    public State changeState(ContinueState state, SyntaxNode node) throws CompilerException {
        throw new CompilerException("Continue not allowed", node.getSymbol());
    }

    public State changeState(ReturnState state, SyntaxNode node) throws CompilerException {
        throw new CompilerException("Return not allowed", node.getSymbol());
    }

    public State changeState(ThrowState state, SyntaxNode node) throws CompilerException {
        throw new CompilerException("Throw not allowed", node.getSymbol());
    }

    public State changeState(SynchronizedState state, SyntaxNode node) throws CompilerException {
        throw new CompilerException("Synchronized not allowed", node.getSymbol());
    }

    public State changeState(TryState state, SyntaxNode node) throws CompilerException {
        throw new CompilerException("Try not allowed", node.getSymbol());
    }

    public State changeState(TypeSwitchState state, SyntaxNode node) throws CompilerException {
        throw new CompilerException("Typeswitch not allowed", node.getSymbol());
    }

    @Override
    public State changeState(LocalVariableDeclarationState state, SyntaxNode node) throws CompilerException {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public State changeState(MethodState state, SyntaxNode node) throws CompilerException {
        throw new CompilerException("Method not allowed", node.getSymbol());
    }

    @Override
    public State changeState(UnreachableCodeState state, SyntaxNode node) throws CompilerException {
        throw new CompilerException("Unreachable code not allowed", node.getSymbol());
    }

    @Override
    public State getParent() {
        return this.parent;
    }
}
