/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage3.states;

import abstractsyntaxtree.SyntaxNode;
import exceptions.CompilerException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import teajay.bytecode.attributes.code.Label;

/**
 *
 * @author Michael Burzan
 */
public class BlockState extends AbstractState {

    private Set<Label> labelList = new HashSet<>();

    public BlockState() {
    }

    public void addLabel(Label l) {
        if (l == null) {
            throw new Error("Label does not null");
        }
        if (!labelList.contains(l)) {
            this.labelList.add(l);
        }
    }

    private boolean findInParents(Class<? extends State> state) {
        State iter = this.getParent();
        while (iter != null) {
            if (state.isInstance(iter)) {
                return true;
            }
            iter = iter.getParent();
        }
        return false;
    }

    public List<Label> getLabels() {
        return Arrays.asList(this.labelList.toArray(new Label[0]));
    }

    @Override
    public State changeState(StatementExpressionState state, SyntaxNode node) throws CompilerException {
        state.parent = this;
        return state;
    }

    @Override
    public State changeState(LocalVariableDeclarationState state, SyntaxNode node) throws CompilerException {
        state.parent = this;
        return state;
    }

    @Override
    public State changeState(IfState state, SyntaxNode node) throws CompilerException {
        state.parent = this;
        return state;
    }

    @Override
    public State changeState(BlockState state, SyntaxNode node) throws CompilerException {
        state.parent = this;
        return state;
    }

    @Override
    public State changeState(ReturnState state, SyntaxNode node) throws CompilerException {
        State iter = this.getParent();
        while (iter != null) {
            if (iter instanceof StaticConstructorState) {
                throw new CompilerException("Unnecessary return statement in static constructor", node.getSymbol());
            }
            iter = iter.getParent();
        }

        state.parent = this;
        return state;
    }

    @Override
    public State changeState(WhileState state, SyntaxNode node) throws CompilerException {
        state.parent = this;
        return state;
    }

    @Override
    public State changeState(DoState state, SyntaxNode node) throws CompilerException {
        state.parent = this;
        return state;
    }

    @Override
    public State changeState(ForState state, SyntaxNode node) throws CompilerException {
        state.parent = this;
        return state;
    }

    @Override
    public State changeState(ThrowState state, SyntaxNode node) throws CompilerException {
        state.parent = this;
        return state;
    }

    @Override
    public State changeState(TryState state, SyntaxNode node) throws CompilerException {
        state.parent = this;
        return state;
    }

    @Override
    public State changeState(BreakState state, SyntaxNode node) throws CompilerException {
        if (!findInParents(LoopState.class)) {
            throw new CompilerException("break outside loop");
        }
        state.parent = this;
        return state;
    }

    @Override
    public State changeState(ContinueState state, SyntaxNode node) throws CompilerException {
        if (!findInParents(LoopState.class)) {
            throw new CompilerException("continue outside loop");
        }
        state.parent = this;
        return state;
    }

    @Override
    public State changeState(TypeSwitchState state, SyntaxNode node) throws CompilerException {
        state.parent = this;
        return state;
    }

    @Override
    public State changeState(SynchronizedState state, SyntaxNode node) throws CompilerException {
        state.parent = this;
        return state;
    }

    @Override
    public State changeState(UnreachableCodeState state, SyntaxNode node) throws CompilerException {
        state.parent = this;
        return state;
    }

    public void addLabels(List<Label> labels) {
        this.labelList.addAll(labels);
    }

    public void resetLabelList() {
        this.labelList.clear();
    }
}
