/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package stage3.states;

import abstractsyntaxtree.SyntaxNode;
import codegeneration.environment.Stage3Environment;
import exceptions.CompilerException;
import teajay.bytecode.attributes.code.JVMInstruction;
import teajay.bytecode.attributes.code.Label;
import teajay.bytecode.cpool.ClassInfo;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public class TryState extends AbstractState {
    private Label startPC;
    private JVMInstruction endPC;
    private Label handleLabel;
    private ClassInfo classInfo;
    

    public State changeState(BlockState state, SyntaxNode node) throws CompilerException{
        state.parent = this;
        return state;
    }
    
    public void setStartLabel(Label start){
        this.startPC = start;
    }
    public void setEndLabel(JVMInstruction ins){
        this.endPC = ins;
    }

    public Label getStartPC() {
        return startPC;
    }

    public JVMInstruction getEndPC() {
        return endPC;
    }

    public void setHandleLabel(Label label) {
        this.handleLabel = label;
    }

    public void setClassInfo(ClassInfo classInfo) {
        this.classInfo = classInfo;
    }

    public Label getHandleLabel() {
        return handleLabel;
    }

    public ClassInfo getClassInfo() {
        return classInfo;
    }
    
    public void commit(Stage3Environment stage3Env) {
        stage3Env.getMethodInfo().getCode().addExceptionHandler(getStartPC(),
                    getEndPC(),
                    getHandleLabel(),
                    getClassInfo());
    }
    
    
}
