/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage3.utils;

import abstractsyntaxtree.SyntaxNode;
import codegeneration.environment.Stage3Environment;
import exceptions.CompilerException;
import teajay.bytecode.ClassFile;
import teajay.bytecode.attributes.AttributeInfo;
import teajay.generics.check.TypeException;
import teajay.generics.parser.Type;
import teajay.util.tjattribute.TeaJayAttribute;

/**
 *
 * @author Michael Burzan
 */
public class TypeChecker {
    private final Type type;
    private final Stage3Environment stage3Env;
    private final SyntaxNode node;

    public TypeChecker(Type type, Stage3Environment stage3Env,SyntaxNode node){
        this.type = type;
        this.stage3Env = stage3Env;
        this.node = node;
        
    }
        
    /**
     * check is current type a ClassType and check the signature compatibility with typedef
     * @throws CompilerException 
     */
    public void check() throws CompilerException {
        if(!(this.type instanceof Type.ClassType)){
            throw new CompilerException("Invalid extends type "+this.type, node.getSymbol());
        }
        try{
            stage3Env.getTypeUtils().checkSignatureCompatibility(this.type);
        }catch(TypeException ex){
            throw new CompilerException(this.type,ex.getMessage(),node.getSymbol());
        }
    }
    
    public static void check(Type type,SyntaxNode node,Stage3Environment stage3Env) throws CompilerException {
        if(!(type instanceof Type.ClassType) && !(type instanceof Type.SimpleType) && !(type instanceof Type.ArrayType)){
            throw new CompilerException("Invalid extends type " +type, node.getSymbol());
        }
        try{
            stage3Env.getTypeUtils().checkSignatureCompatibility(type);
        }catch(TypeException ex){
            //ex.printStackTrace(System.err);
            throw new CompilerException(type,ex.getMessage(),node.getSymbol());
        }
    }
    
    
}
