/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package symbol;

import java.util.Arrays;
import java.util.Deque;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.Set;
import teajay.bytecode.cpool.FieldDescriptor;
import teajay.generics.check.TypeUtils;
import teajay.generics.parser.Type;

/**
 *
 * @author Michael (Burzan | Kußmann)
 */
public class LocalEntry implements Comparable<LocalEntry> {

    private final String qID;
    private final Deque<String> typeSwitchNameOfTheType = new LinkedList<>();
    private final Deque<Type> typeSwitchedType = new LinkedList<>();
    private final int index;
    private final Type type;
    private final TypeUtils typeUtils;
    private final Set<LocalVariableState> variableStates = new HashSet<>();

    LocalEntry(int index, String qID, Type type, TypeUtils typeUtils, LocalVariableState ... state) {
        this.index = index;
        this.qID = qID;
        this.type = type;
        this.typeUtils = typeUtils;
        variableStates.addAll(Arrays.asList(state));
    }

    public int getIndex() {
        return index;
    }

    public Type getType() {
        if (typeSwitchedType.isEmpty()) {
            return this.type;
        } else {
            return this.typeSwitchedType.peek();
        }
    }

    public String getQID() {
        if (typeSwitchNameOfTheType.isEmpty()) {
            return qID;
        } else {
            return typeSwitchNameOfTheType.peek();
        }
    }

    public void setTypeSwitchType(Type t) {
        FieldDescriptor descr = typeUtils.makeFieldDescriptor(t);
        this.typeSwitchNameOfTheType.push(descr.getClassInfo().getClassName());
        this.typeSwitchedType.push(t);
    }

    public void resetTypeSwitchType() {
        if (this.typeSwitchNameOfTheType.isEmpty()) {
            throw new IllegalStateException("no type to reset");
        }
        this.typeSwitchNameOfTheType.pop();
        this.typeSwitchedType.pop();
    }
    
    public String getPreviousQID() {
        if (this.typeSwitchNameOfTheType.size() > 1) {
            String current = this.typeSwitchNameOfTheType.pop();
            String last = this.typeSwitchNameOfTheType.peek();
            this.typeSwitchNameOfTheType.push(current);
            return last;
        } else {
            return qID;
        }
    }
    
    public void removeVariableState(LocalVariableState s){
        this.variableStates.remove(s);
    }
    
    public void addVariableState(LocalVariableState s){
        this.variableStates.add(s);
    }
    
    public boolean isVariableState(LocalVariableState ... state){
        return this.variableStates.containsAll(Arrays.asList(state));
    }
    
    
    @Override
    public String toString() {
        return getQID() + ", index = " + index + " (" + getType() + ")";
    }

    @Override
    public int compareTo(LocalEntry o) {
        if (index < o.index) {
            return -1;
        }
        if (index == o.index) {
            return 0;
        }
        return 1;
    }
}