/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package symbol;

import codegeneration.environment.Stage3Environment;
import datastructs.Pair;
import exceptions.CompilerException;
import teajay.runtime.exceptions.ResolveException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;
import teajay.bytecode.ClassFile;
import teajay.bytecode.attributes.signature.Signature;
import teajay.bytecode.cpool.FieldDescriptor;
import teajay.bytecode.cpool.MethodDescriptor;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.bytecode.methods.MethodInfo;
import teajay.generics.parser.Type;
import teajay.generics.writer.SignatureEncoder;
import teajay.methods.MethodFinder;
import teajay.methods.MethodFinder.QueryType;
import teajay.util.io.ClassFileRef;
import teajay.util.io.ClassFinder;
import teajay.util.io.FileIterator;

/**
 *
 * @author cefour
 */
public class MethodResolver {
        private static final MethodFilter bridgeFilter = new MethodFilter() {

            @Override
            public boolean filter(MethodInfo info) {
                return !info.isBridge();
            }
        };
    public static void main(String... args) throws ByteCodeException, IOException, ResolveException, CompilerException {


        MethodInfo findStaticMethod;
        ClassFinder.instance().addPaths(FileIterator.BOOTPATH, FileIterator.CLASSPATH);
        findStaticMethod = existMethod(null, "java/util/List", "get", "int");
        System.out.println(findStaticMethod);

    }
    @Deprecated
    public static MethodInfo findStaticMethod(ClassFile currentClassFile, String qTypeName, String methodName,
            String... inputParameters)
            throws ByteCodeException, IOException, ResolveException, CompilerException {
        ClassFinder finder = ClassFinder.instance();
        ClassFileRef ref = finder.find(qTypeName);

        List<FieldDescriptor> checkParametersList = new ArrayList<>();
        List<ClassFileRef> classRefParameterList = new ArrayList<>();
        for (String s : inputParameters) {
            ClassFileRef pClass = finder.find(s);
            classRefParameterList.add(pClass);
            checkParametersList.add(FieldDescriptor.fromClassFile(pClass.get()));
        }

        MethodInfo info = checkMethodsInClassFile(currentClassFile, ref, methodName, checkParametersList, new StaticMethodFilter());
        if (info == null) {
            return checkIsBestSpecializedMethod(currentClassFile, ref, methodName, checkParametersList, classRefParameterList, new StaticMethodFilter()).value1;
        }
        return info;
    }
    @Deprecated
    public static MethodInfo existMethod(Stage3Environment env, Type callerType, String methodName, FieldDescriptor... parameters)
            throws ByteCodeException, IOException, ResolveException, CompilerException {
        return existMethod(env.getClassFile(), ((Type.ClassType) callerType).name, methodName, parameters);
    }
    @Deprecated
    public static MethodInfo existMethod(ClassFile currentClassFile, String qTypeName, String methodName,
            FieldDescriptor... parameterTypeName)
            throws ByteCodeException, IOException, ResolveException, CompilerException {
        String[] types = new String[parameterTypeName.length];
        for (int i = 0; i < types.length; i++) {
            types[i] = parameterTypeName[i].getClassInfo().getClassName();
        }
        return existMethod(currentClassFile, qTypeName, methodName, types);
    }

    /**
     * @param qTypeName caller object, possible caller object is this or static
     * method
     * @param methodName
     * @param methodParamter
     */
    @Deprecated
    public static MethodInfo existMethod(ClassFile currentClassFile, String qTypeName, String methodName,
            String... parameterTypeName)
            throws ByteCodeException, IOException, ResolveException, CompilerException {
        ClassFinder finder = ClassFinder.instance();
        ClassFileRef findClass = finder.find(qTypeName);

        //Search for exact method description
        List<FieldDescriptor> checkParametersList = new ArrayList<>();
        List<ClassFileRef> classRefParameterList = new ArrayList<>();
        for (String s : parameterTypeName) {
            ClassFileRef pClass = finder.find(s);
            //TODO: Cannot find methods with array signatures
            classRefParameterList.add(pClass);
            checkParametersList.add(FieldDescriptor.fromClassFile(pClass.get()));
        }

        //It the method not found in object, then search Method in derivation of the object
        MethodInfo correctMethod = checkMethodsInClassFile(currentClassFile, findClass, methodName, checkParametersList, bridgeFilter);
        if (correctMethod == null) {
            correctMethod = findMethodInDerivationTree(currentClassFile, findClass, methodName, checkParametersList);
            if (correctMethod == null) {
                return findBestSpecializedMethod(currentClassFile, findClass, methodName, checkParametersList, classRefParameterList);

            }
        }
        return correctMethod;

    }

    private static MethodInfo findMethodInDerivationTree(ClassFile currentClassFile, ClassFileRef fileRef, String methodName,
            List<FieldDescriptor> parameters) throws ByteCodeException, IOException, ResolveException, CompilerException {

        LinkedList<ClassFileRef> superClasses = DerivationTree.instance().getSuperClasses(fileRef);
        for (ClassFileRef ref : superClasses) {
            MethodInfo info = checkMethodsInClassFile(currentClassFile, ref, methodName, parameters, null);

            if (info != null) {
                return info;
            }
        }
        return null;
    }

    private static MethodInfo checkMethodsInClassFile(ClassFile currentClassFile, ClassFileRef ref, String methodName, List<FieldDescriptor> parametersList, MethodFilter methodFilter) throws ByteCodeException, IOException, ResolveException, CompilerException {
        ClassFile file = ref.get();
        List<MethodInfo> methods = file.getMethods(methodName);
        if (methods == null) {
            return null;
        }
        List<MethodInfo> candidates = new ArrayList<>();
        MethodInfo correctMethod = null;
        for (MethodInfo info : methods) {
            if (methodFilter != null) {
                if (!methodFilter.filter(info)) {
                    continue;
                }
            }
            MethodDescriptor desc = info.getDescriptor();

            if (desc.getParameterCount() != parametersList.size()) {
                continue; // TODO mit Michael über String ... sprechen
            }

            boolean flag = true;
            List<FieldDescriptor> parameters = desc.getParameters();
            for (int i = 0; i < parameters.size(); i++) {
                FieldDescriptor paramDesc = parameters.get(i);

                if (isNullObject(parametersList.get(i))) {
                } else if (!paramDesc.equals(parametersList.get(i))) {
                    flag = false;
                    break;
                }


            }
            if (flag == false) {
                continue;
            }
            candidates.add(info);


        }
        if (candidates.size() == 1) {
            return candidates.get(0);
        } else if (candidates.size() > 1) {
            throw new CompilerException("Ambigiuous method reference " + candidates);
        }
        return null;
    }

    /**
     * depth 1 class A { }
     *
     * depth 2 class B extends A{
     *
     * }
     *
     * depth 3 class C extends B {
     *
     * }
     *
     * depth 4 class D extends C {
     *
     * }
     *
     * void method(A a); void method(C c);
     *
     *
     * method(new B()) >> call method(A a); method(new D()) >> call method(C c);
     *
     *
     *
     * @param findClass
     * @param methodName
     * @param checkParametersList
     * @param classRefParameterList
     * @return
     * @throws ByteCodeException
     * @throws IOException
     * @throws ResolveException
     */
    private static MethodInfo findBestSpecializedMethod(ClassFile currentClassFile, ClassFileRef fileRef, String methodName, List<FieldDescriptor> parametersList, List<ClassFileRef> callRefParameterList) throws ByteCodeException, IOException, ResolveException, CompilerException {
        LinkedList<ClassFileRef> allClasses = DerivationTree.instance().getSuperClasses(fileRef);
        LinkedList<Pair<MethodInfo, Integer>> resultList = new LinkedList<>();
        allClasses.addFirst(fileRef);
        for (ClassFileRef ref : allClasses) {
            Pair p = checkIsBestSpecializedMethod(currentClassFile, ref, methodName, parametersList, callRefParameterList, bridgeFilter);
            if (p != null) {
                resultList.add(p);
            }
        }
        int distance = Integer.MAX_VALUE;
        MethodInfo info = null;
        for (Pair<MethodInfo, Integer> p : resultList) {
            if (distance > p.value2) {
                distance = p.value2;
                info = p.value1;
            }
        }


        return info;
    }

    public static MethodInfo findConstructor(Stage3Environment stage3Env, Type.ClassType typeOfClass, FieldDescriptor... params) throws CompilerException {
        try {
            String[] types = new String[params.length];
            for (int i = 0; i < types.length; i++) {
                types[i] = params[i].getClassInfo().getClassName();
            }

            ClassFinder finder = ClassFinder.instance();
            ClassFileRef findClass = finder.find(typeOfClass.name);

            //Search for exact method description
            List<FieldDescriptor> checkParametersList = new ArrayList<>();
            List<ClassFileRef> classRefParameterList = new ArrayList<>();
            for (String s : types) {

                ClassFileRef pClass = finder.find(s);
                //TODO: Cannot find methods with array signatures
                classRefParameterList.add(pClass);
                checkParametersList.add(FieldDescriptor.fromClassFile(pClass.get()));

            }
            Pair<MethodInfo, Integer> bestConstructor = checkIsBestSpecializedMethod(
                    stage3Env.getClassFile(),
                    findClass, "<init>",
                    checkParametersList,
                    classRefParameterList, bridgeFilter);
            if (bestConstructor != null) {
                return bestConstructor.value1;
            }
        } catch (ByteCodeException | IOException | ResolveException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }
        return null;
    }

    private static Pair<MethodInfo, Integer> checkIsBestSpecializedMethod(
            ClassFile currentClassFile, ClassFileRef ref,
            String methodName, List<FieldDescriptor> parametersList,
            List<ClassFileRef> callRefList, MethodFilter methodFilter) throws ByteCodeException, IOException, ResolveException, CompilerException {
        ClassFinder finder = ClassFinder.instance();
        ClassFileRef currFileRef = finder.find(currentClassFile.getThis().getClassName());
        ClassFile file = ref.get();
        List<MethodInfo> methods = file.getMethods(methodName);
        if (methods == null) {
            return null;
        }
        MethodInfo correctMethod = null;
        //List<MethodInfo> possibleBestMethods = new LinkedList<>();
        List<Pair> possibleMethods = new LinkedList<>();
        for (MethodInfo info : methods) {
            if (methodFilter != null) {
                if (!methodFilter.filter(info)) {
                    continue;
                }
            }
            if (info.isPrivate() && !currentClassFile.getThis().getClassName().equals(ref.get().getThis().getClassName())) {
                continue;
            }
            if (info.isProtected() && !DerivationTree.instance().isAssignable(currFileRef, ref)) {
                continue;
            }
            if (info.isBridge()) {
                continue;
            }
            MethodDescriptor desc = info.getDescriptor();

            if (desc.getParameterCount() != parametersList.size()) {
                continue; // TODO mit Michael über call( [Object] ... bla ) sprechen
            }
            boolean methodIsNotPossibleToCall = false;
            List<FieldDescriptor> parameters = desc.getParameters();
            int calculateDepth = 0;
            for (int i = 0; i < parameters.size(); i++) {
                ClassFileRef pcRef = callRefList.get(i);
                ClassFileRef pRef = finder.find(parameters.get(i).getClassInfo().getClassName());
                if (pRef == null) {
                    methodIsNotPossibleToCall = true;
                    break;
                }
                if (pcRef == pRef) {
                    calculateDepth += 0;
                } else if (pcRef.referencesNull()) {
                    calculateDepth += 0;
                } else if (!pcRef.referencesPrimitive() && !pRef.referencesPrimitive()) {
                    if (DerivationTree.instance().isDerivated(pRef, pcRef)) {
                        calculateDepth += DerivationTree.instance().getDepth(pcRef)
                                - DerivationTree.instance().getDepth(pRef);
                    } else { // Parameter of the method can't call with call parameter
                        methodIsNotPossibleToCall = true;
                        break;
                    }
                } else {
                    methodIsNotPossibleToCall = true;
                    break;
                }
            }

            if (!methodIsNotPossibleToCall) {
                possibleMethods.add(new Pair(info, calculateDepth));
            }
        }
        //Best specialized method has lowest distance from derivation node to base node
        int distance = Integer.MAX_VALUE;
        boolean ambigiuous = true;
        Pair result = null;
        for (Pair<MethodInfo, Integer> p : possibleMethods) {
            if (distance > p.value2) {
                distance = p.value2;
                result = p;
                ambigiuous = false;
            } else if (distance == p.value2) {
                ambigiuous = true;
            }
        }
        if (result != null) {

            if (ambigiuous) {
                throw new CompilerException("Ambigiuous method " + result.value1);
            }

            return result;
        }
        return null;
    }

    private static boolean isNullObject(FieldDescriptor obj) {
        return obj.getClassInfo().getClassName().equals("null");


    }

    public static MethodInfo findStaticMethod(Stage3Environment stage3Env, String ident, FieldDescriptor[] callParameters) {
        List<ClassFileRef> staticImports = stage3Env.getSymbolFacade().getStaticImports();
        return null;
    
    }

    private static abstract class MethodFilter {

        public MethodFilter() {
        }

        public abstract boolean filter(MethodInfo info);
    }

    private static class StaticMethodFilter extends MethodFilter {

        @Override
        public boolean filter(MethodInfo info) {
            return info.isStatic();
        }
    }

    private <T> void test(LinkedList<? extends Number> g, T b) {
        MethodInfo info = null;
        LinkedList<Type> l = new LinkedList<>();
        l.add(new Type.ClassType(null, "java/lang/Number", null));
        Type t = new Type.MethodType(l, new Type.SimpleType("void"));
        info.addAttribute(new Signature(SignatureEncoder.encode(t)));
        //info.getAttribute(Signature.NAME) >> MethodResolver
        test(new LinkedList<Integer>(), new String());
    }
}
