/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.generics.check;

import java.util.Iterator;
import teajay.generics.parser.Type;
import static teajay.generics.parser.Type.WildcardType.Kind.EXTENDS;
import static teajay.generics.parser.Type.WildcardType.Kind.SUPER;
import static teajay.generics.parser.Type.WildcardType.Kind.UNBOUNDED;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public class FlatAssignabilityVisitor implements Type.Visitor<Void, Type> {

    private final TypeUtils utils;
    private boolean first;
    private boolean toggled = false;

    public FlatAssignabilityVisitor(TypeUtils utils, boolean first) {
        this.utils = utils;
        this.first = first;
    }

    @Override
    public Void visitSimpleType(Type.SimpleType type, Type p) {
        if (p instanceof Type.SimpleType) {
            if (type.name.equals(((Type.SimpleType) p).name)) {
                return null;
            }
        }
        throw new TypeException("Incompatible types: " + type + " <-> " + p);
    }

    @Override
    public Void visitArrayType(Type.ArrayType type, Type p) {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public Void visitMethodType(Type.MethodType type, Type p) {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public Void visitClassSigType(Type.ClassSigType type, Type p) {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public Void visitClassType(Type.ClassType type, Type p) {
        if (p instanceof Type.ClassType) {
            Type.ClassType ctype = (Type.ClassType) p;
            if (!ctype.name.equals(type.name)) {
                throw new TypeException("Incompatible types: " + type + " <-> " + ctype);
            }
            //first = false;
            if (!toggled && first) {
                first = false;
                toggled = true;
            } else if (toggled) {
                first = true;
            }
            if (type.typeArgs != null && ctype.typeArgs != null
                    && type.typeArgs.size() == ctype.typeArgs.size()) {
                Iterator<Type> ita = type.typeArgs.iterator();
                Iterator<Type> itb = ctype.typeArgs.iterator();
                while (ita.hasNext()) {
                    ita.next().accept(this, itb.next());
                }
            } else if (type.typeArgs != ctype.typeArgs) {
                if ((type.typeArgs != null && ctype.typeArgs != null)
                        || (type.typeArgs == null && !ctype.typeArgs.isEmpty())
                        || (ctype.typeArgs == null && !type.typeArgs.isEmpty())) {
                    throw new TypeException("Same type with different type argument count: " + type + " <-> " + ctype);
                }
            }
                                first = true;
                    toggled = false;
        } else if (p instanceof Type.WildcardType) {
            Type.WildcardType wtype = (Type.WildcardType) p;

            switch (wtype.kind) {
                case EXTENDS:
                    if (first) {
                        utils.checkAssignability(type, wtype.boundType, true);
                    } else {
                        throw new TypeException("Incompatible types: " + type + " <-> " + p);
                    }
                    break;
                case SUPER:
                default:
                    if (first) {
                        if (!type.name.equals("java/lang/Object")) {
                            throw new TypeException("Incompatible types: " + type + " <-> " + p);
                        }
                    } else {
                        throw new TypeException("Incompatible types: " + type + " <-> " + p);
                    }
                //first = false;
            }

        } else if (p instanceof Type.SimpleType && first) {
            Type.SimpleType simpleType = (Type.SimpleType) p;
            if (simpleType.isPrimitiveType()) {
                throw new TypeException("Incompatible types: " + type + " <-> " + p);
            }
            Type.TypeParamType typeParam = utils.getTypeParamType(simpleType.name);
            //TODO: Check other interfaces aswell
            if (typeParam.interfaceBounds != null && !typeParam.interfaceBounds.isEmpty()
                    && typeParam.classBound.isObject()) {
                utils.checkAssignability(type, typeParam.interfaceBounds.get(0), true);
            } else {
                utils.checkAssignability(type, typeParam.classBound, true);
            }
        } else {
            throw new TypeException("Incompatible types: " + type + " <-> " + p);
        }
        return null;
    }

    @Override
    public Void visitTypeParamType(Type.TypeParamType type, Type p) {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public Void visitWildcardType(Type.WildcardType type, Type p) {
        switch (type.kind) {
            case EXTENDS:
                if (p instanceof Type.ClassType) {
                    if (!first) {
                        utils.checkAssignability(type.boundType, p, false);
                    } else {
                        throw new TypeException("Incompatible types: " + type + " <-> " + p);
                    }
                } else if (p instanceof Type.WildcardType) {
                    Type.WildcardType wt = ((Type.WildcardType) p);
                    switch (wt.kind) {
                        case EXTENDS:
                            //first = false;
                            if (first) {
                                throw new TypeException("Incompatible types: " + type + " <-> " + p);
                            }
                            type.boundType.accept(this, wt.boundType);
                            break;
                        default:
                            //first = false;
                            type.boundType.accept(this, TypeUtils.object);
                    }
                } else if (p instanceof Type.SimpleType) {
                    //Type.SimpleType simpleType = (Type.SimpleType) p;
                    //Type right = utils.getUnerasedType(simpleType.name);
                    //utils.checkAssignability(type.boundType, simpleType, first);
                    type.boundType.accept(this, p);
                } else {
                    throw new TypeException("Incompatible types: " + type + " <-> " + p);
                }
                break;
            case SUPER:
                if (p instanceof Type.ClassType) {
                    if (!first) {
                        utils.checkAssignability(p, type.boundType, first);
                    } else {
                        utils.checkAssignability(type.boundType, p, first);
                    }
                    //first = false;
                } else if (p instanceof Type.WildcardType) {
                    Type.WildcardType wt = ((Type.WildcardType) p);
                    switch (wt.kind) {
                        case SUPER:
                            //first = false;
                            wt.boundType.accept(this, type.boundType);
                            break;
                        default:
                            //first = false;
                            TypeUtils.object.accept(this, type.boundType);
                    }
                } else if (p instanceof Type.SimpleType) {
                    utils.checkAssignability(p, type.boundType, first);
                    //throw new TypeException("not supported yet");
                } else {
                    throw new TypeException("Incompatible types: " + type + " <-> " + p);
                }
                break;
            case UNBOUNDED:
                if (first) {
                    throw new TypeException("Incompatible types: " + type + " <-> " + p);
                }
        }
        return null;
    }
}
