/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.generics.check;

import java.util.Iterator;
import teajay.generics.parser.Type;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public class SignatureDescenderVisitor implements Type.Visitor<Type, Type.ClassType> {

    private final TypeUtils utils;
    //final Map<String, Type> tempContext = new HashMap<>();
    final Context tempContext;
    private final ReplaceVisitor rvisitor;

    public SignatureDescenderVisitor(TypeUtils utils, Context context) {
        this.utils = utils;
        this.tempContext = new Context(context);
        this.rvisitor = new ReplaceVisitor(tempContext);
    }
    
    public SignatureDescenderVisitor(TypeUtils utils, Context context, boolean ignoreUnknownTypeVariables) {
        this.utils = utils;
        this.tempContext = new Context(context);
        this.rvisitor = new ReplaceVisitor(tempContext, false, ignoreUnknownTypeVariables);
    }

    @Override
    public Type visitSimpleType(Type.SimpleType type, Type.ClassType endPoint) {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public Type visitArrayType(Type.ArrayType type, Type.ClassType endPoint) {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public Type visitMethodType(Type.MethodType type, Type.ClassType endPoint) {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public Type visitClassSigType(Type.ClassSigType type, Type.ClassType endPoint) {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public Type visitClassType(Type.ClassType type, Type.ClassType endPoint) {
        if (type.outerType != null) {
            Type.ClassType newOuterType = (Type.ClassType) type.outerType.accept(rvisitor, null);
            newOuterType.accept(this, null);
        }
        if (type.name == null) {
            return null;
        } else if (type.name.equals("teajay/lang/Closure")) {
            return type;
        }
        Type.ClassSigType thisSig = utils.getClassSignature(type);
        if (type.typeArgs != null && thisSig.typeParamTypes != null) {
            if (type.typeArgs.size() != thisSig.typeParamTypes.size()) {
                throw new TypeException("Type argument count missmatch: " + type + " <-> " + thisSig);
            }
            Iterator<Type> ita = type.typeArgs.iterator();
            Iterator<Type.TypeParamType> itb = thisSig.typeParamTypes.iterator();
            while (ita.hasNext()) {
                tempContext.put(itb.next().name, ita.next().accept(rvisitor, null), null);
            }
        } else if (type.typeArgs != null && !type.typeArgs.isEmpty()) {
            throw new TypeException("Type argument count missmatch: " + type + " <-> " + thisSig);
        } else if (thisSig.typeParamTypes != null && !thisSig.typeParamTypes.isEmpty()) {
            throw new TypeException("Type argument count missmatch: " + type + " <-> " + thisSig);
        }
        if (endPoint == null) {
            return null;
        } else if (type.name.equals(endPoint.name)) {
            return type;
        }
        if (thisSig.superinterfaceTypes != null) {
            for (Type t : thisSig.superinterfaceTypes) {
                tempContext.addFrame();
                Type.ClassType newSuperType = (Type.ClassType) t.accept(rvisitor, null);
                Type temp = newSuperType.accept(this, endPoint);
                if (temp != null) {
                    return temp;
                }
                tempContext.removeFrame();
            }
        }
        Type.ClassType newSuperType = (Type.ClassType) thisSig.superclassType.accept(rvisitor, null);
        return newSuperType.accept(this, endPoint);
    }

    @Override
    public Type visitTypeParamType(Type.TypeParamType type, Type.ClassType endPoint) {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public Type visitWildcardType(Type.WildcardType type, Type.ClassType endPoint) {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }
}
