/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.generics.check;

import java.io.IOException;
import java.util.Iterator;
import symbol.DerivationTree;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.generics.parser.Type;
import static teajay.generics.parser.Type.WildcardType.Kind.EXTENDS;
import static teajay.generics.parser.Type.WildcardType.Kind.SUPER;
import static teajay.generics.parser.Type.WildcardType.Kind.UNBOUNDED;
import teajay.runtime.exceptions.ResolveException;
import teajay.util.io.ClassFileRef;
import teajay.util.io.ClassFinder;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 * @Deprecated
 */
@Deprecated
class TypeCheck implements Type.Visitor<Type, Type> {

    private ClassFinder finder = ClassFinder.instance();
    private GenericVisitor genericVisitor = new GenericVisitor(this);
    private final Context context;

    public TypeCheck(Context context) {
        this.context = new Context(context);
    }

    @Override
    public Type visitSimpleType(Type.SimpleType type, Type p) {
        if (type.isPrimitiveType()) {
            throw new TypeException("primitives not allowed");
        }
        if ((p instanceof Type.SimpleType)) {
            if (((Type.SimpleType) p).isPrimitiveType()) {
                throw new TypeException("primitives not allowed");
            }
            if (!type.name.equals(((Type.SimpleType) p).name)) {
                throw new TypeException("incompatible type");
            }
        }
        return null;
    }

    @Override
    public Type visitArrayType(Type.ArrayType type, Type p) {
        throw new UnsupportedOperationException("Array types are not supported yet.");
    }

    @Override
    public Type visitMethodType(Type.MethodType type, Type p) {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public Type visitClassSigType(Type.ClassSigType type, Type p) {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public Type visitClassType(Type.ClassType type, Type p) {
        try {
            if (!(p instanceof Type.ClassType)) {
                throw new TypeException("Incompatible types: " + type + " <-> " + p);
            } else {
                Type.ClassType other = (Type.ClassType) p;
                ClassFileRef a = finder.find(type.name);
                ClassFileRef b = finder.find(other.name);
                if (a == null || b == null) {
                    throw new TypeException("Class not found: " + (a == null ? type.name : other.name));
                }
                if (DerivationTree.instance().isAssignable(a, b)) {
                    if (other.typeArgs != null && type.typeArgs != null) {
                        if (other.typeArgs.size() != type.typeArgs.size()) {
                            throw new TypeException("incompatible types");
                        } else {
                            Iterator<Type> ita = type.typeArgs.iterator();
                            Iterator<Type> itb = other.typeArgs.iterator();
                            while (ita.hasNext()) {
                                ita.next().accept(genericVisitor, itb.next());
                            }
                        }
                    } else if (type.typeArgs != other.typeArgs) {
                        throw new TypeException("incompatible types");
                    }

                } else {
                    throw new TypeException("Incompatible types: " + type.name + " - " + other.name);
                }
            }
            return null;
        } catch (ByteCodeException | ResolveException | IOException ex) {
            //ex.printStackTrace(System.err);
            throw new TypeException(ex.getMessage(), ex);
        }
    }

    @Override
    public Type visitTypeParamType(Type.TypeParamType type, Type p) {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public Type visitWildcardType(Type.WildcardType type, Type p) {
        throw new TypeException("incompatible types");
    }
}

class GenericVisitor implements Type.Visitor<Type, Type> {

    private final TypeCheck typeCheck;
    private final Type object = new Type.ClassType(null, "java/lang/Object", null);

    public GenericVisitor(TypeCheck typeCheck) {
        this.typeCheck = typeCheck;
    }

    @Override
    public Type visitSimpleType(Type.SimpleType type, Type p) {
        if (!(p instanceof Type.SimpleType)) {
            throw new TypeException("incompatible type");
        }
        if (!type.name.equals(((Type.SimpleType) p).name)) {
            throw new TypeException("incompatible type");
        }
        return null;
    }

    @Override
    public Type visitArrayType(Type.ArrayType type, Type p) {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public Type visitMethodType(Type.MethodType type, Type p) {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public Type visitClassSigType(Type.ClassSigType type, Type p) {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public Type visitClassType(Type.ClassType type, Type p) {
        if (!(p instanceof Type.ClassType)) {
            throw new TypeException("incompatible types");
        }
        Type.ClassType other = (Type.ClassType) p;
        if (!type.name.equals(other.name)) {
            throw new TypeException("Incompatible types: " + type.name + " - " + other.name);
        }
        if (other.typeArgs != null && type.typeArgs != null) {
            if (other.typeArgs.size() != type.typeArgs.size()) {
                throw new TypeException("incompatible types");
            } else {
                Iterator<Type> ita = type.typeArgs.iterator();
                Iterator<Type> itb = other.typeArgs.iterator();
                while (ita.hasNext()) {
                    ita.next().accept(this, itb.next());
                }
            }
        } else if (type.typeArgs != other.typeArgs) {
            throw new TypeException("incompatible types");
        }
        return null;
    }

    @Override
    public Type visitTypeParamType(Type.TypeParamType type, Type p) {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public Type visitWildcardType(Type.WildcardType type, Type p) {
        switch (type.kind) {
            case EXTENDS:
                if (p instanceof Type.ClassType) {
                    type.boundType.accept(typeCheck, p);
                } else if (p instanceof Type.WildcardType) {
                    Type.WildcardType wt = ((Type.WildcardType) p);
                    switch (wt.kind) {
                        case EXTENDS:
                            type.boundType.accept(this, wt.boundType);
                        default:
                            type.boundType.accept(this, object);
                    }
                } else {
                    throw new TypeException("incompatible types");
                }
                break;
            case SUPER:
                if (p instanceof Type.ClassType) {
                    ((Type.ClassType) p).accept(typeCheck, type.boundType);
                } else if (p instanceof Type.WildcardType) {
                    Type.WildcardType wt = ((Type.WildcardType) p);
                    switch (wt.kind) {
                        case SUPER:
                            wt.boundType.accept(this, type.boundType);
                        default:
                            object.accept(this, type.boundType);
                    }
                } else {
                    throw new TypeException("incompatible types");
                }
                break;
            case UNBOUNDED:
            //OK
            }
        return null;
    }
}
