/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.generics.check;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import symbol.DerivationTree;
import teajay.bytecode.ClassFile;
import teajay.bytecode.cpool.ClassInfo;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.bytecode.exceptions.MalformedClassNameException;
import teajay.generics.parser.Type;
import static teajay.generics.parser.Type.WildcardType.Kind.EXTENDS;
import teajay.runtime.exceptions.ResolveException;
import teajay.util.io.ClassFileRef;
import teajay.util.io.ClassFinder;
import teajay.util.io.FileIterator;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 * @deprecated
 */
@Deprecated
public class TypeChecker {

    private static class TypeCheck implements Type.Visitor<Type, Type> {

        private ClassFinder finder = ClassFinder.instance();
        private GenericVisitor genericVisitor = new GenericVisitor(this);

        @Override
        public Type visitSimpleType(Type.SimpleType type, Type p) {
            if (!(p instanceof Type.SimpleType)) {
                throw new TypeException("incompatible type");
            }
            if (!type.name.equals(((Type.SimpleType) p).name)) {
                throw new TypeException("incompatible type");
            }


            return null;
        }

        @Override
        public Type visitArrayType(Type.ArrayType type, Type p) {
            throw new UnsupportedOperationException("Array types are not supported yet.");
        }

        @Override
        public Type visitMethodType(Type.MethodType type, Type p) {
            throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
        }

        @Override
        public Type visitClassSigType(Type.ClassSigType type, Type p) {
            throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
        }

        @Override
        public Type visitClassType(Type.ClassType type, Type p) {
            try {
                if (!(p instanceof Type.ClassType)) {
                    throw new TypeException("incompatible types");
                } else {
                    Type.ClassType other = (Type.ClassType) p;
                    ClassFileRef a = finder.find(type.name);
                    ClassFileRef b = finder.find(other.name);
                    if (a == null || b == null) {
                        throw new TypeException("Class not found: " + (a == null ? type.name : other.name));
                    }
                    if (DerivationTree.instance().isAssignable(a, b)) {
                        if (other.typeArgs != null && type.typeArgs != null) {
                            if (other.typeArgs.size() != type.typeArgs.size()) {
                                throw new TypeException("incompatible types");
                            } else {
                                Iterator<Type> ita = type.typeArgs.iterator();
                                Iterator<Type> itb = other.typeArgs.iterator();
                                while (ita.hasNext()) {
                                    ita.next().accept(genericVisitor, itb.next());
                                }
                            }
                        } else if (type.typeArgs != other.typeArgs) {
                            throw new TypeException("incompatible types");
                        }

                    } else {
                        throw new TypeException("Incompatible types: " + type.name + " - " + other.name);
                    }
                }
                return null;
            } catch (ByteCodeException | ResolveException | IOException ex) {
                //ex.printStackTrace(System.err);
                throw new TypeException(ex.getMessage(), ex);
            }
        }

        @Override
        public Type visitTypeParamType(Type.TypeParamType type, Type p) {
            throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
        }

        @Override
        public Type visitWildcardType(Type.WildcardType type, Type p) {
            throw new TypeException("incompatible types");
        }
    }

    private static class GenericVisitor implements Type.Visitor<Type, Type> {

        private final TypeCheck typeCheck;
        private final Type object = new Type.ClassType(null, "java/lang/Object", null);

        public GenericVisitor(TypeCheck typeCheck) {
            this.typeCheck = typeCheck;
        }

        @Override
        public Type visitSimpleType(Type.SimpleType type, Type p) {
            if (!(p instanceof Type.SimpleType)) {
                throw new TypeException("incompatible type");
            }
            if (!type.name.equals(((Type.SimpleType) p).name)) {
                throw new TypeException("incompatible type");
            }
            return null;
        }

        @Override
        public Type visitArrayType(Type.ArrayType type, Type p) {
            throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
        }

        @Override
        public Type visitMethodType(Type.MethodType type, Type p) {
            throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
        }

        @Override
        public Type visitClassSigType(Type.ClassSigType type, Type p) {
            throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
        }

        @Override
        public Type visitClassType(Type.ClassType type, Type p) {
            if (!(p instanceof Type.ClassType)) {
                throw new TypeException("incompatible types");
            }
            Type.ClassType other = (Type.ClassType) p;
            if (!type.name.equals(other.name)) {
                throw new TypeException("Incompatible types: " + type.name + " - " + other.name);
            }
            if (other.typeArgs != null && type.typeArgs != null) {
                if (other.typeArgs.size() != type.typeArgs.size()) {
                    throw new TypeException("Type argument count missmatch: " + type + " <-> " + other);
                } else {
                    Iterator<Type> ita = type.typeArgs.iterator();
                    Iterator<Type> itb = other.typeArgs.iterator();
                    while (ita.hasNext()) {
                        ita.next().accept(this, itb.next());
                    }
                }
            } else if (type.typeArgs == other.typeArgs) {
                //OK
            } else if (type.typeArgs == null && !other.typeArgs.isEmpty()) {
                throw new TypeException("Type argument count missmatch: " + type + " <-> " + other);
            } else if (other.typeArgs == null && !type.typeArgs.isEmpty()) {
                throw new TypeException("Type argument count missmatch: " + type + " <-> " + other);
            }
            return null;
        }

        @Override
        public Type visitTypeParamType(Type.TypeParamType type, Type p) {
            throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
        }

        @Override
        public Type visitWildcardType(Type.WildcardType type, Type p) {
            switch (type.kind) {
                case EXTENDS:
                    if (p instanceof Type.ClassType) {
                        type.boundType.accept(typeCheck, p);
                    } else if (p instanceof Type.WildcardType) {
                        Type.WildcardType wt = ((Type.WildcardType) p);
                        switch (wt.kind) {
                            case EXTENDS:
                                type.boundType.accept(this, wt.boundType);
                            default:
                                type.boundType.accept(this, object);
                        }
                    } else {
                        throw new TypeException("incompatible types");
                    }
                    break;
                case SUPER:
                    if (p instanceof Type.ClassType) {
                        ((Type.ClassType) p).accept(typeCheck, type.boundType);
                    } else if (p instanceof Type.WildcardType) {
                        Type.WildcardType wt = ((Type.WildcardType) p);
                        switch (wt.kind) {
                            case SUPER:
                                wt.boundType.accept(this, type.boundType);
                            default:
                                object.accept(this, type.boundType);
                        }
                    } else {
                        throw new TypeException("incompatible types");
                    }
                    break;
                case UNBOUNDED:
                //OK
            }
            return null;
        }
    }

    public static void checkFlatType(Type t1, Type t2) {
        t1.accept(new TypeCheck(), t2);
    }

    public static void main(String[] args) throws MalformedClassNameException {
        ClassFinder.instance().addPaths(FileIterator.BOOTPATH, FileIterator.CLASSPATH);
        Type t1 = new Type.WildcardType(Type.WildcardType.Kind.EXTENDS, new Type.SimpleType("T"));
        Type t2 = new Type.WildcardType(Type.WildcardType.Kind.EXTENDS, new Type.SimpleType("T"));
        Type tt1 = new Type.ClassType(null, "java/util/List", Arrays.asList((Type) new Type.WildcardType(Type.WildcardType.Kind.UNBOUNDED, null)));
        Type tt2 = new Type.ClassType(null, "java/util/ArrayList", Arrays.asList((Type) new Type.ClassType(null, ClassInfo.fromClass(E.class).getClassName(), null)));
        tt1.accept(new TypeCheck(), tt2);
        //t1.accept(new TypeCheck(), t2);
        //List<? extends Number> l = new ArrayList<? extends Integer>();
        //List<Number> l = new ArrayList<Integer>();
        //List<Integer> l = new ArrayList<Number>();
        List<E> lste = new ArrayList<E>();
        List<? super B> l = lste;
        l.add(new A());
        E t = lste.get(0);


        List<?> ld = new ArrayList<String>();
        ArrayList<Integer> fffff = new AA<Integer>();
        List<Tiger> jj = new ArrayList<Tiger>();
        List<Animal> aa = (List<Animal>) (Object) jj;
    }

    static class Animal {
    }

    static class Tiger extends Animal {
    }

    static class A extends B {
    }

    static class B extends C {
    }

    static class C extends E {
    }

    static class E {
    }

    static class AA<T> extends ArrayList<T> {
    }
}