/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.lang;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import teajay.util.ArrayUtils;
import teajay.util.TJType;
import teajay.util.tjattribute.TeaJayAttribute;

/**
 * Rudimentary IO Tools for TeaJay.
 * @author Michael (Burzan | Kußmann)
 */
@TJType(accessType = TeaJayAttribute.AccessType.Final)
public final class IO extends TJObject {
    
    private IO() {
    }
    /**
     * Writes the String-representation of obj to STDOUT and adds a trailing new line.
     * @param obj
     */
    public static void println(Object obj){
        System.out.println(obj);
    }
    /**
     * Writes the String-representation of obj to STDERR and adds a trailing new line.
     * @param obj
     */    public static void printlnErr(Object obj){
        System.err.println(obj);
    }
    /**
     * Writes the String-representation of obj to STDOUT.
     * @param obj
     */
    public static void print(Object obj){
        System.out.print(obj);
    }
    /**
     * Writes the String-representation of obj to STDERR.
     * @param obj
     */
    public static void printErr(Object obj){
        System.err.print(obj);
    }
    /**
     * Reads a line from STDIN.
     * @return 
     */
    public static String readLine() {
        return System.console().readLine();
    }
    /**
     * Basic IO.
     * @param out
     * @param bytes list of values between 0 and 255 inclusive
     * @throws IOException 
     */
    public static void writeBytes(OutputStream out, List<Number> bytes) throws IOException {
        writeBytes(out, bytes, Number.ZERO, bytes.size());
    }
    
    public static void writeBytes(OutputStream out, List<Number> bytes, Number offset, Number len) throws IOException {
        byte[] byteVals = new byte[len.$int_val()];
        int ilen = len.$int_val();
        for (int i = offset.$int_val(), r = 0; r < ilen; i++, r++) {
            byteVals[r] = (byte) (bytes.get(Number.$valueOf(i)).$int_val() & 0xFF);
        }
        out.write(byteVals);
    }
    
    public static void writeBytes(OutputStream out, Number... bytes) throws IOException {
        writeBytes(out, ArrayUtils.toList(bytes), Number.ZERO, Number.$valueOf(bytes.length));
    }
    
    public static Number readBytes(InputStream in, List<Number> out, Number len) throws IOException {
        return readBytes(in, out, Number.ZERO, len);
    }
    /**
     * Reads signed bytes from the supplied InputStream.
     * @param in
     * @param out
     * @param offset
     * @param len
     * @return
     * @throws IOException 
     */
    public static Number readBytes(InputStream in, List<Number> out, Number offset, Number len) throws IOException {
        byte[] buf = new byte[len.$int_val()];
        int read = in.read(buf);
        for (int i = 0; i < read; i++) {
            out.set(offset.add(Number.$valueOf(i)), Number.$valueOf((int) (buf[i] & 0xFF)));
        }
        return Number.$valueOf(read);
    }
}