/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.lang.impl;

import teajay.lang.Number;
import teajay.lang.TJException;
import teajay.util.TJType;
import teajay.util.tjattribute.TeaJayAttribute;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
@TJType(type = TeaJayAttribute.Type.Impl)
public class TJExceptionImpl extends TJException {

    private final TJException rthis;

    public TJExceptionImpl(TJException rthis) {
        this.rthis = rthis;
    }

    public TJExceptionImpl(TJException rthis, String message) {
        super(message);
        this.rthis = rthis;
    }

    public TJExceptionImpl(TJException rthis, String message, Throwable cause) {
        super(message, cause);
        this.rthis = rthis;
    }

    public TJExceptionImpl(TJException rthis, Throwable cause) {
        super(cause);
        this.rthis = rthis;
    }

    @Override
    public Number tjHashCode() {
        return Number.$valueOf(System.identityHashCode(rthis));
    }

    @Override
    @TJType(accessType = TeaJayAttribute.AccessType.Hidden)
    public int hashCode() {
        if (rthis instanceof TJExceptionImpl) {
            return ((TJExceptionImpl) rthis).tjHashCode().hashCode();
        } else {
            return super.hashCode();
        }
    }

    @Override
    public Boolean operator_eq(Object obj) {
        return this == obj;
    }
    
    @Override
    public String toString() {
        //XXX
        String str = super.toString();
        return str.replace(TJExceptionImpl.class.getName(), rthis.getClass().getSuperclass().getName());
    }

    @Override
    @TJType(accessType = TeaJayAttribute.AccessType.Hidden)
    public boolean equals(Object obj) {
        if (rthis instanceof TJExceptionImpl) {
            return ((TJExceptionImpl) rthis).operator_eq(obj);
        } else {
            return obj == this;
        }
    }
}
