/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.lang.java;

import java.lang.reflect.Array;
import teajay.lang.Number;
import teajay.util.TJType;
import teajay.util.tjattribute.TeaJayAttribute;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
@TJType(accessType = TeaJayAttribute.AccessType.Final)
public final class FloatArray<T> extends PrimitiveArray<T> {

    private final Number dimension;
    private final Object data;

    public FloatArray(teajay.lang.Number firstSize, teajay.lang.Number... nextSizes) {
        Number[] size = new Number[nextSizes.length + 1];
        size[0] = firstSize;
        System.arraycopy(nextSizes, 0, size, 1, nextSizes.length);
        if (size.length <= 0 || size.length > 255) {
            throw new IllegalArgumentException("array dimension must be between 1 and 255 inclusive");
        }
        int[] sizes = new int[size.length];
        for (int i = 0; i < sizes.length; i++) {
            sizes[i] = size[i].$int_val();
        }
        this.data = Array.newInstance(float.class, sizes);
        this.dimension = Number.$valueOf(size.length);
    }

    public FloatArray(Object obj) {
        Class<?> clazz = obj.getClass();
        if (!clazz.isArray()) {
            throw new IllegalArgumentException("expecting array argument");
        }
        this.data = obj;
        clazz = clazz.getComponentType();
        Class<?> last = null;
        int dim = 0;
        while (clazz != null) {
            dim++;
            last = clazz;
            clazz = clazz.getComponentType();
        }
        this.dimension = Number.$valueOf(dim);
        if (last == null || !last.equals(float.class)) {
            throw new IllegalArgumentException("expecting int array");
        }
    }

    @Override
    public Object getJavaArray() {
        return data;
    }

    @Override
    public Class<?> getArrayClass() {
        return data.getClass();
    }

    @Override
    public teajay.lang.Number getDimension() {
        return dimension;
    }

    @Override
    public Class<?> getLeafElementType() {
        return float.class;
    }
    
    public void operator_set(Number index, float val) {
        set(index, val);
    }
    
    public void operator_set(int index, float val) {
        set(index, val);
    }
    
    public void set(Number index, float val) {
        set(index.$int_val(), val);
    }
    
    public void set(int index, float val) {
        if (data.getClass().getComponentType().equals(float.class)) {
            Array.setFloat(data, index, val);
        } else {
            throw new IllegalArgumentException("FloatArray: incompatible types");
        }
    }

    @Override
    public void set(int index, T val) {
        if (val == null) {
            if (data.getClass().getComponentType().isArray()) {
                Array.set(data, index, null);
            } else {
                throw new IllegalArgumentException("FloatArray: incompatible types");
            }
        } else if (val instanceof FloatArray) {
            FloatArray array = (FloatArray) val;
            Class<?> componentType = array.getJavaArray().getClass();
            if (componentType != null && componentType.isArray()
                    && data.getClass().getComponentType().equals(componentType)) {
                Array.set(data, index, array.getJavaArray());
            } else {
                throw new IllegalArgumentException("IntArray: incompatible types");
            }
        } else if (val instanceof Number) {
            Number number = (Number) val;
            if (data.getClass().getComponentType().equals(float.class)) {
                Array.setFloat(data, index, number.$float_val());
            } else {
                throw new IllegalArgumentException("IntArray: incompatible types");
            }
        } else {
            throw new IllegalArgumentException("IntArray expects Number argument");
        }
    }

    @Override
    public T get(int index) {
        Object obj = Array.get(data, index);
        if (obj == null) {
            return null;
        }
        Class<?> componentType = obj.getClass();
        if (componentType != null && componentType.isArray()
                && data.getClass().getComponentType().equals(componentType)) {
            obj = new FloatArray(obj);
        } else {
            obj = Number.$valueOf(Array.getFloat(data, index));
        }
        return (T) obj;
    }
}