/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.lang.java;

import java.lang.reflect.Array;
import teajay.lang.TJObject;
import teajay.lang.Number;
import teajay.lang.TJIterable;
import teajay.lang.TJIterator;
import teajay.util.TJType;
import teajay.util.tjattribute.TeaJayAttribute;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
@TJType(accessType = TeaJayAttribute.AccessType.Hidden)
public abstract class JavaArray<T> extends TJObject implements TJIterable<T> {
    public abstract Object getJavaArray();
    
    public abstract Class<?> getArrayClass();
    
    public abstract Number getDimension();
    
    public abstract Class<?> getLeafElementType();
    
    public final void set(Number index, T value) {
        set(index.$int_val(), value);
    }
    
    public final T get(Number index) {
        return get(index.$int_val());
    }
    
    public abstract void set(int index, T value);
    
    public abstract T get(int index);
    
    public final T operator_get(Number index) {
        return get(index);
    }
    
    public final T operator_get(int index) {
        return get(index);
    }
    
    public final void operator_set(Number index, T val) {
        set(index, val);
    }
    
    public final void operator_set(int index, T val) {
        set(index, val);
    }
    
    public final Number length() {
        return Number.$valueOf(Array.getLength(getJavaArray()));
    }
    
    public final int intLength() {
        return Array.getLength(getJavaArray());
    }

    @Override
    public final TJIterator<T> iterator() {
        return new TJIterator<T>() {
            private final int len = intLength();
            private int current = 0;

            @Override
            public Boolean hasNext() {
                return current < len;
            }

            @Override
            public T next() {
                T result = get(current++);
                return result;
            }

            @Override
            public void remove() {
                throw new UnsupportedOperationException("Remove not supported.");
            }
        };
    }
}
