/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.lang.java;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.math.MathContext;
import teajay.lang.Number;
import teajay.util.TJType;
import teajay.util.tjattribute.TeaJayAttribute;
/**
 * Tools for interfacing with java types.
 * @author Michael Kußmann <michi@outx.de>
 */
@TJType(accessType = TeaJayAttribute.AccessType.Final)
public final class JavaPrimitives {
        
    private JavaPrimitives() {
    }
       
    public static Number toNumber(int val) {
        return Number.$valueOf(val);
    }
    
    public static Number toNumber(Integer val) {
        return Number.$valueOf(val);
    }
    
    public static Number toNumber(long val) {
        return Number.$valueOf(val);
    }
    
    public static Number toNumber(Long val) {
        return Number.$valueOf(val);
    }
    
    public static Number toNumber(float val) {
        return Number.$valueOf(val);
    }
    
    public static Number toNumber(Float val) {
        return Number.$valueOf(val);
    }
    
    public static Number toNumber(double val) {
        return Number.$valueOf(val);
    }
    
    public static Number toNumber(Double val) {
        return Number.$valueOf(val);
    }
    
    public static Number toNumber(short val) {
        return Number.$valueOf(val);
    }
    
    public static Number toNumber(Short val) {
        return Number.$valueOf(val);
    }
    
    public static Number toNumber(char val) {
        return Number.$valueOf(val);
    }
    
    public static Number toNumber(Character val) {
        return Number.$valueOf(val);
    }
    
    public static Number toNumber(byte val) {
        return Number.$valueOf(val);
    }
    
    public static Number toNumber(Byte val) {
        return Number.$valueOf(val);
    }
    
    public static Number toNumber(boolean val) {
        return val ? Number.ONE : Number.ZERO;
    }
    
    public static Number toNumber(Boolean val) {
        return val ? Number.ONE : Number.ZERO;
    }
    
    public static Number toNumber(BigInteger val) {
        return Number.$valueOf(val, BigInteger.ONE);
    }
    
    public static Number toNumber(BigDecimal val) {
        return Number.$valueOf(val);
    }
    
    public static boolean toBoolean(Boolean val) {
        return val.booleanValue();
    }
    
    public static byte toByte(Number val) {
        return val.$byte_val();
    }
    
    public static short toShort(Number val) {
        return val.$short_val();
    }
    
    public static char toChar(Number val) {
        return val.$char_val();
    }
    
    public static int toInt(Number val) {
        return val.$int_val();
    }
    
    public static long toLong(Number val) {
        return val.$long_val();
    }
    
    public static float toFloat(Number val) {
        return val.$float_val();
    }
    
    public static double toDouble(Number val) {
        return val.$double_val();
    }
    
    public static BigDecimal toBigDecimal(Number val, Number precision) {
        return val.$bigdec_val(new MathContext(precision.$int_val() ));
    }
    
    public static BigInteger toBigInteger(Number val) {
        return val.$bigint_val();
    }
}
