/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.runtime;

import java.io.File;
import java.io.FilenameFilter;
import teajay.lang.Closure;
import teajay.lang.List;
import teajay.lang.Number;
import teajay.util.ArrayUtils;
import teajay.util.TJSignature;
import teajay.util.TJType;
import teajay.util.tjattribute.TeaJayAttribute;
/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
@TJType(type = TeaJayAttribute.Type.Hidden)
public class FileType {
    private FileType() {
    }
    
    public static Number getFreeSpace(File file) {
        return Number.$valueOf(file.getFreeSpace());
    }
    
    public static Number getTotalSpace(File file) {
        return Number.$valueOf(file.getTotalSpace());
    }
    
    public static Number getUsableSpace(File file) {
        return Number.$valueOf(file.getUsableSpace());
    }
    
    public static Number tjHashCode(File f) {
        return Number.$valueOf(f.hashCode());
    }
    
    public static Number length(File file) {
        return Number.$valueOf(file.length());
    }
    
    public static Number lastModified(File file) {
        return Number.$valueOf(file.lastModified());
    }
    
    public static List<String> list(File file) {
        return ArrayUtils.toList(file.list());
    }
    
    @TJSignature("(Ljava/io/File;Lteajay/lang/Closure<Ljava/lang/Boolean;Ljava/io/File;Ljava/lang/String;>;)Lteajay/lang/List<Ljava/lang/String;>;")
    public static List<String> list(File file, final Closure filter) {
        return ArrayUtils.toList(file.list(new FilenameFilter() {

            @Override
            public boolean accept(File dir, String name) {
                try {
                    return (Boolean) filter.execute(dir, name);
                } catch (Throwable ex) {
                    throw new RuntimeException(ex);
                }
            }
        }));
    }
    
    public static List<File> listFiles(File file) {
        return ArrayUtils.toList(file.listFiles());
    }
    
    @TJSignature("(Ljava/io/File;Lteajay/lang/Closure<Ljava/lang/Boolean;Ljava/io/File;Ljava/lang/String;>;)Lteajay/lang/List<Ljava/io/File;>;")
    public static List<File> listFiles(File file, final Closure filter) {
        return ArrayUtils.toList(file.listFiles(new FilenameFilter() {

            @Override
            public boolean accept(File dir, String name) {
                try {
                    return (Boolean) filter.execute(dir, name);
                } catch (Throwable ex) {
                    throw new RuntimeException(ex);
                }
            }
        }));
    }
    
    public static Number tjCompareTo(File file, File other) {
        return Number.$valueOf(file.compareTo(other));
    }
    
    public static void setLastModified(File file, Number time) {
        file.setLastModified(time.$long_val());
    }
    
    public static void setExecutable(File file, Boolean executable) {
        file.setExecutable(executable);
    }
    
    public static void setExecutable(File file, Boolean executable, Boolean ownerOnly) {
        file.setExecutable(executable, ownerOnly);
    }
    
    public static void setReadable(File file, Boolean readable) {
        file.setExecutable(readable);
    }
    
    public static void setReadable(File file, Boolean readable, Boolean ownerOnly) {
        file.setExecutable(readable, ownerOnly);
    }
    
    public static void setWritable(File file, Boolean writable) {
        file.setWritable(writable);
    }
    
    public static void setWritable(File file, Boolean writable, Boolean ownerOnly) {
        file.setWritable(writable, ownerOnly);
    }
}
