/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.runtime;

import teajay.lang.List;
import teajay.util.TJType;
import teajay.util.tjattribute.TeaJayAttribute;
import teajay.lang.Number;
import teajay.lang.TJIterator;
import teajay.util.ArrayUtils;
/**
 * Method overrides for java.lang.String (making it the TeaJay-Type "String").
 * @author Michael Kußmann <michi@outx.de>
 */
@TJType(type = TeaJayAttribute.Type.Hidden)
public final class StringType {
    
    private StringType() {
    }
    
    static void $_checkIndices(Number... indices) throws IndexOutOfBoundsException {
        for (Number n : indices) {
            if (!n.isInteger()) {
                throw new IndexOutOfBoundsException("string index must be an integer");
            }
            if (n.isNegative()) {
                throw new IndexOutOfBoundsException("string index must not be negative");
            }
            if (n.compareTo(Number.INT_MAX_VAL) > 0) {
                throw new IndexOutOfBoundsException("string index must be within signed integer range");
            }
        }
    }
    
    public static CharSequence subSequence(CharSequence seq, Number start, Number end) {
        $_checkIndices(start, end);
        return seq.subSequence(start.$int_val(), end.$int_val());
    }
    
    public static Boolean operator_less(String a, String b) {
        return a.compareTo(b) < 0;
    }
    
    public static Boolean operator_leq(String a, String b) {
        return a.compareTo(b) <= 0;
    }
    
    public static Boolean operator_greater(String a, String b) {
        return a.compareTo(b) > 0;
    }
    
    public static Boolean operator_geq(String a, String b) {
        return a.compareTo(b) >= 0;
    }
    
    public static String charAt(String str, Number index) {
        $_checkIndices(index);
        return String.valueOf(str.charAt(index.$int_val())).intern();
    }
    
    public static Number length(String str) {
        return Number.$valueOf(str.length());
    }
    
    public static String substring(String str, Number beginIndex) {
        $_checkIndices(beginIndex);
        return str.substring(beginIndex.$int_val());
    }
    
    public static String substring(String str, Number beginIndex, Number endIndex) {
        $_checkIndices(beginIndex, endIndex);
        return str.substring(beginIndex.$int_val(), endIndex.$int_val());
    }
    
    public static Number lastIndexOf(String haystack, String needle, Number fromIndex) {
        $_checkIndices(fromIndex);
        int index = haystack.lastIndexOf(needle, fromIndex.$int_val());
        return Number.$valueOf(index);
    }
    
    public static Number lastIndexOf(String haystack, String needle) {
        return lastIndexOf(haystack, needle, Number.ZERO);
    }

    public static Number indexOf(String haystack, String needle, Number fromIndex) {
        $_checkIndices(fromIndex);
        int index = haystack.indexOf(needle, fromIndex.$int_val());
        return Number.$valueOf(index);
    }
    
    public static Number indexOf(String haystack, String needle) {
        return indexOf(haystack, needle, Number.ZERO);
    }
    
    public static List<String> split(String haystack, String regex) {
        String[] split = haystack.split(regex);
        return ArrayUtils.toList(split);
    }
    
    public static List<String> split(String haystack, String regex, Number limit) {
        $_checkIndices(limit);
        String[] split = haystack.split(regex, limit.$int_val());
        return ArrayUtils.toList(split);
    }
    
    public static List<String> toCharacterList(String str) {
        TJIterator<String> it = iterator(str);
        List<String> out = TJTypes.instantiate(List.class);
        while (it.hasNext()) {
            out.add(it.next().intern());
        }
        return out;
    }
    
    public static Number tjHashCode(String str) {
        return Number.$valueOf(str.hashCode());
    }
    
    public static TJIterator<String> iterator(final String str) {
        return new TJIterator<String>() {
            int pos = 0;
            
            @Override
            public Boolean hasNext() {
                return pos < str.length();
            }

            @Override
            public String next() {
                return String.valueOf(str.charAt(pos++)).intern();
            }

            @Override
            public void remove() {
                throw new UnsupportedOperationException("remove not supported");
            }
        };
    }
}
