/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.util;
import teajay.lang.Number;
import teajay.util.tjattribute.TeaJayAttribute;
/**
 * Tools for character manipulation in TeaJay.
 * 
 * @author Michael Kußmann <michi@outx.de>
 */
@TJType(accessType = TeaJayAttribute.AccessType.Abstract)
public class Characters {
    
    private static final Number CHAR_MAX_VAL = teajay.lang.Number.$valueOf(Character.MAX_VALUE);
    
    private Characters() {
    }

    public static String toString(Number c) throws IllegalArgumentException {
        if (c == null) {
            throw new NullPointerException("c");
        } else if (c.isNegative() || c.compareTo(CHAR_MAX_VAL) > 0 || !c.isInteger()) {
            throw new IllegalArgumentException("character value must be an unsigned 16-bit integer");
        }
        return String.valueOf(c.$char_val());
    }
    
    public static String concat(String str, Number c) {
        return str + toString(c);
    }
    
    public static String concat(Number c, String str) {
        return toString(c) + str;
    }
}
