/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.util;

import java.io.UnsupportedEncodingException;
import teajay.lang.List;
import teajay.util.tjattribute.TeaJayAttribute;
import teajay.lang.Number;
import teajay.runtime.TJTypes;

/**
 * Tools for converting bytes into Strings.
 * @author Michael Kußmann <michi@outx.de>
 */
@TJType(accessType = TeaJayAttribute.AccessType.Abstract)
public class Strings {

    public static String fromUnsignedBytes(List<Number> bytes) throws UnsupportedEncodingException {
        byte[] buf = new byte[bytes.size().$int_val()];
        for (int i = 0; i < buf.length; i++) {
            buf[i] = (byte) (bytes.get(Number.$valueOf(i)).$short_val() & 0xFF);
        }
        
        return new String(buf);
    }

    public static String fromUnsignedBytes(List<Number> bytes, String charset) throws UnsupportedEncodingException {
        byte[] buf = new byte[bytes.size().$int_val()];
        for (int i = 0; i < buf.length; i++) {
            buf[i] = (byte) (bytes.get(Number.$valueOf(i)).$short_val() & 0xFF);
        }
        return new String(buf, charset);
    }

    public static String fromSignedBytes(List<Number> bytes, String charset) throws UnsupportedEncodingException {
        byte[] buf = new byte[bytes.size().$int_val()];
        for (int i = 0; i < buf.length; i++) {
            buf[i] = bytes.get(Number.$valueOf(i)).$byte_val();
        }
        return new String(buf, charset);
    }

    public static String fromSignedBytes(List<Number> bytes) throws UnsupportedEncodingException {
        byte[] buf = new byte[bytes.size().$int_val()];
        for (int i = 0; i < buf.length; i++) {
            buf[i] = bytes.get(Number.$valueOf(i)).$byte_val();
        }
        return new String(buf);
    }

    public static String fromChars(List<Number> chars) {
        char[] buf = new char[chars.size().$int_val()];
        for (int i = 0; i < buf.length; i++) {
            buf[i] = chars.get(Number.$valueOf(i)).$char_val();
        }
        return new String(buf);
    }
    
    public static List<Number> toUnsignedBytes(String str) {
        byte[] bytes = str.getBytes();
        List<Number> result = TJTypes.instantiate(List.class);
        for (int i = 0; i < bytes.length; i++) {
            result.add(Number.$valueOf(bytes[i] & 0xFF));
        }
        return result;
    }

    public static List<Number> toUnsignedBytes(String str, String charset) throws UnsupportedEncodingException {
        byte[] bytes = str.getBytes(charset);
        List<Number> result = TJTypes.instantiate(List.class);
        for (int i = 0; i < bytes.length; i++) {
            result.add(Number.$valueOf(bytes[i] & 0xFF));
        }
        return result;
    }

    public static List<Number> toSignedBytes(String str, String charset) throws UnsupportedEncodingException {
        byte[] bytes = str.getBytes(charset);
        List<Number> result = TJTypes.instantiate(List.class);
        for (int i = 0; i < bytes.length; i++) {
            result.add(Number.$valueOf(bytes[i]));
        }
        return result;
    }

    public static List<Number> toSignedBytes(String str) throws UnsupportedEncodingException {
        byte[] bytes = str.getBytes();
        List<Number> result = TJTypes.instantiate(List.class);
        for (int i = 0; i < bytes.length; i++) {
            result.add(Number.$valueOf(bytes[i]));
        }
        return result;
    }
}
