/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.util;

import teajay.lang.Closure;
import teajay.lang.TJObject;
import teajay.lang.Number;
import teajay.util.tjattribute.TeaJayAttribute;

/**
 * Tools for working with TJObjects.
 * @author Michael Kußmann <michi@outx.de>
 */
@TJType(accessType = TeaJayAttribute.AccessType.Final)
public final class TJObjects extends TJObject {
    
    private TJObjects(TJObjects rthis) {
    }
    
    public static Boolean isClosure(Object a) {
        return a instanceof Closure;
    }
    
    public static void checkNull(Object a) throws NullPointerException {
        if (a == null) {
            throw new NullPointerException("");
        }
    }

    public static void checkNull(Object a, String errorMessage) throws NullPointerException {
        if (a == null) {
            throw new NullPointerException(errorMessage);
        }
    }

    public static Boolean isNull(Object a) {
        return a == null;
    }
    
    public static Boolean notNull(Object a) {
        return a != null;
    }
    /**
     * Check for referential equality.
     * This is the only way to check for referential equality frominside TeaJay.
     * @param a an Object
     * @param b another Object
     * @return true if a == b (in Java terms)
     */
    public static Boolean referenceEquals(Object a, Object b) {
        return a == b;
    }
    
    public static Number hashCode(Object a) {
        if (a == null) {
            return Number.ZERO;
        } else if (a instanceof TJObject) {
            TJObject tjObject = (TJObject) a;
            return tjObject.tjHashCode();
        } else {
            return Number.$valueOf(a.hashCode());
        }
    }

    public static Boolean equals(Object a, Object b) {
        if (a == b) {
            return true;
        } else if (a == null || b == null) {
            return false;
        } else if (b instanceof TJObject) {
            return b.equals(a);
        } else {
            return a.equals(b);
        }
    }

    public static String toString(Object a) {
        return toString(a, "null");
    }

    public static String toString(Object a, String nullDefault) {
        if (a == null) {
            return nullDefault;
        } else {
            return a.toString();
        }
    }
}
