/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.util.impl;

import java.util.Arrays;
import java.util.Comparator;
import teajay.lang.Closure;
import teajay.lang.RandomAccess;
import teajay.lang.TJComparable;
import teajay.lang.impl.ListImpl;
import teajay.util.Sorter;
import teajay.util.TJSignature;
import teajay.util.TJType;
import teajay.util.tjattribute.TeaJayAttribute;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
@TJType(type = TeaJayAttribute.Type.Impl)
public class SorterImpl extends Sorter {
    private static boolean OPTIMIZE = false;

    private final static int[] SHELLSORT_INCS = new int[]{
        1, 3, 7, 21, 48, 112, 336, 861, 1968, 4592, 13776, 33936, 86961,
        198768, 463792, 1391376, 3402672, 8382192, 21479367, 49095696,
        114556624, 343669872, 52913488, 2085837936
    };

    private SorterImpl(Sorter rthis) {
    }

    private static <T> int compare(T a, T b, Closure comparator) {
        teajay.lang.Number result;
        try {
            result = (teajay.lang.Number) comparator.execute(a, b);
            return result.$int_val();
        } catch (Throwable ex) {
            throw new RuntimeException(ex);
        }
    }

    @TJSignature("<T:Ljava/lang/Object;>(Lteajay/lang/RandomAccess<TT;>;Lteajay/lang/Closure<Lteajay/lang/Number;TT;TT;>;)V")
    public static <T> void sort(RandomAccess<T> list, final Closure comparator) {
        if (OPTIMIZE && list instanceof ListImpl) {
            Arrays.sort(((ListImpl<T>) list).getArray(), 0, list.size().$int_val(), new Comparator<T>() {
                @Override
                public int compare(T a, T b) {
                    try {
                        return ((teajay.lang.Number) comparator.execute(a, b)).$int_val();
                    } catch (Throwable ex) {
                        throw new RuntimeException(ex);
                    }
                }
            });
        } else {
            shsort(list, 0, list.size().$int_val() - 1, comparator);
        }
    }

    public static <T extends TJComparable<? super T>> void sort(RandomAccess<T> list) {
        if (OPTIMIZE && list instanceof ListImpl) {
            Arrays.sort(((ListImpl<T>) list).getArray(), 0, list.size().$int_val());
        } else {
            shsort(list, 0, list.size().$int_val() - 1, new Closure() {
                @Override
                public Object execute(Object... params) throws Throwable {
                    T a = (T) params[0];
                    T b = (T) params[1];
                    return a.tjCompareTo(b);
                }
            });
        }
    }

    private static <T> void swap(RandomAccess<T> a, int i, int j) {
        teajay.lang.Number ii = teajay.lang.Number.$valueOf(i);
        teajay.lang.Number jj = teajay.lang.Number.$valueOf(j);
        T tmp = a.get(ii);
        a.set(ii, a.get(jj));
        a.set(jj, tmp);
    }

    private static <T> void shsort(final RandomAccess<T> a, int lo, int hi, Closure comparator) {
        final int[] seq = SHELLSORT_INCS;
        for (int inc = seq.length - 1; inc >= 0; inc--) {
            int increment = seq[inc];

            for (int i = increment + lo; i <= hi; i++) {
                for (int j = i; j - increment >= lo && compare(a.get(teajay.lang.Number.$valueOf(j - increment)), a.get(teajay.lang.Number.$valueOf(j)), comparator) > 0; j -= increment) {
                    swap(a, j, j - increment);
                }
            }
        }
    }
}
